//
//  Project   : Navier-Stokes
//  File      : stokesConv.cc
//  Author    : Ronald Kriemann
//  Purpose   : test of 3D grids
//
// arch-tag: ca2358b6-3c45-4228-a8b5-b8f8b1d54162
//

extern "C" {
#include <stdlib.h>
#include <stdio.h>
}

#include "misc/timer.hh"
#include "misc/memory.hh"

#include "net/net.hh"
#include "net/nethierarchy.hh"
#include "net/T3DTetrahedron.hh"

#include "visual/drawapp.hh"

//
// mainfunction
//
int
main( int argc, char **argv ) 
{
    TTimer  timer;
    int     i;
    int     noOfNets = argc > 1 ? atoi( argv[1] ) : 3;
    char    string[256];
    
    T3DVertex    *v0 = new T3DVertex( 0, 0, 0, BOUNDARY );
    T3DVertex    *v1 = new T3DVertex( 1, 0, 0, BOUNDARY );
    T3DVertex    *v2 = new T3DVertex( 1, 0, 1, BOUNDARY );
    T3DVertex    *v3 = new T3DVertex( 0, 0, 1, BOUNDARY );
    T3DVertex    *v4 = new T3DVertex( 0, 1, 0, BOUNDARY );
    T3DVertex    *v5 = new T3DVertex( 1, 1, 0, BOUNDARY );
    T3DVertex    *v6 = new T3DVertex( 1, 1, 1, BOUNDARY );
    T3DVertex    *v7 = new T3DVertex( 0, 1, 1, BOUNDARY );

    T3DCellEdge  *e0  = new T3DCellEdge( v0, v5, BOUNDARY );
    T3DCellEdge  *e1  = new T3DCellEdge( v5, v1, BOUNDARY );
    T3DCellEdge  *e2  = new T3DCellEdge( v1, v0, BOUNDARY );
    T3DCellEdge  *e3  = new T3DCellEdge( v1, v6, BOUNDARY );
    T3DCellEdge  *e4  = new T3DCellEdge( v6, v5, BOUNDARY );
    T3DCellEdge  *e5  = new T3DCellEdge( v6, v0 );
    T3DCellEdge  *e6  = new T3DCellEdge( v6, v2, BOUNDARY );
    T3DCellEdge  *e7  = new T3DCellEdge( v2, v1, BOUNDARY );
    T3DCellEdge  *e8  = new T3DCellEdge( v2, v0, BOUNDARY );
    T3DCellEdge  *e9  = new T3DCellEdge( v5, v4, BOUNDARY );
    T3DCellEdge  *e10 = new T3DCellEdge( v4, v0, BOUNDARY );
    T3DCellEdge  *e11 = new T3DCellEdge( v4, v6, BOUNDARY );
    T3DCellEdge  *e12 = new T3DCellEdge( v0, v7, BOUNDARY );
    T3DCellEdge  *e13 = new T3DCellEdge( v7, v6, BOUNDARY );
    T3DCellEdge  *e14 = new T3DCellEdge( v7, v4, BOUNDARY );
    T3DCellEdge  *e15 = new T3DCellEdge( v2, v3, BOUNDARY );
    T3DCellEdge  *e16 = new T3DCellEdge( v3, v6, BOUNDARY );
    T3DCellEdge  *e17 = new T3DCellEdge( v0, v3, BOUNDARY );
    T3DCellEdge  *e18 = new T3DCellEdge( v7, v3, BOUNDARY );

    T3DTriangle  *f0   = new T3DTriangle( NULL,  e0,  e1,  e2, BOUNDARY );
    T3DTriangle  *f1   = new T3DTriangle( NULL,  e1,  e3,  e4, BOUNDARY );
    T3DTriangle  *f2   = new T3DTriangle( NULL,  e3,  e5,  e2 );
    T3DTriangle  *f3   = new T3DTriangle( NULL,  e5,  e0,  e4 );
    T3DTriangle  *f4   = new T3DTriangle( NULL,  e3,  e6,  e7, BOUNDARY );
    T3DTriangle  *f5   = new T3DTriangle( NULL,  e6,  e8,  e5 );
    T3DTriangle  *f6   = new T3DTriangle( NULL,  e2,  e7,  e8, BOUNDARY );
    T3DTriangle  *f7   = new T3DTriangle( NULL,  e0,  e9, e10, BOUNDARY );
    T3DTriangle  *f8   = new T3DTriangle( NULL,  e5, e10, e11 );
    T3DTriangle  *f9   = new T3DTriangle( NULL,  e9, e11,  e4, BOUNDARY );
    T3DTriangle  *f10  = new T3DTriangle( NULL,  e5, e12, e13 );
    T3DTriangle  *f11  = new T3DTriangle( NULL, e10, e12, e14, BOUNDARY );
    T3DTriangle  *f12  = new T3DTriangle( NULL, e11, e13, e14, BOUNDARY );
    T3DTriangle  *f13  = new T3DTriangle( NULL,  e6, e15, e16, BOUNDARY );
    T3DTriangle  *f14  = new T3DTriangle( NULL,  e5, e17, e16 );
    T3DTriangle  *f15  = new T3DTriangle( NULL, e17, e15,  e8, BOUNDARY );
    T3DTriangle  *f16  = new T3DTriangle( NULL, e18, e16, e13, BOUNDARY );
    T3DTriangle  *f17  = new T3DTriangle( NULL, e12, e18, e17, BOUNDARY );

    T3DTetrahedron  *c0 = new T3DTetrahedron( NULL,  f0,  f1,  f2,  f3 );
    T3DTetrahedron  *c1 = new T3DTetrahedron( NULL,  f2,  f4,  f5,  f6 );
    T3DTetrahedron  *c2 = new T3DTetrahedron( NULL,  f7,  f3,  f8,  f9 );
    T3DTetrahedron  *c3 = new T3DTetrahedron( NULL,  f8, f10, f11, f12 );
    T3DTetrahedron  *c4 = new T3DTetrahedron( NULL,  f5, f13, f14, f15 );
    T3DTetrahedron  *c5 = new T3DTetrahedron( NULL, f10, f14, f16, f17 );

    T3DNet          *net = new T3DNet;
    T3DNetHierarchy *netHier = new T3DNetHierarchy;

    net->delReference();
    netHier->add( net );
    netHier->delReference();
    
    net->add( c0 );
    net->add( c1 );
    net->add( c2 );
    net->add( c3 );
    net->add( c4 );
    net->add( c5 );

    TDrawApp *app = new TDrawApp;

    v0->delReference();
    v1->delReference();
    v2->delReference();
    v3->delReference();
    v4->delReference();
    v5->delReference();
    v6->delReference();
    v7->delReference();

    e0->delReference();
    e1->delReference();
    e2->delReference();
    e3->delReference();
    e4->delReference();
    e5->delReference();
    e6->delReference();
    e7->delReference();
    e8->delReference();
    e9->delReference();
    e10->delReference();
    e11->delReference();
    e12->delReference();
    e13->delReference();
    e14->delReference();
    e15->delReference();
    e16->delReference();
    e17->delReference();
    e18->delReference();
    
    f0->delReference();
    f1->delReference();
    f2->delReference();
    f3->delReference();
    f4->delReference();
    f5->delReference();
    f6->delReference();
    f7->delReference();
    f8->delReference();
    f9->delReference();
    f10->delReference();
    f11->delReference();
    f12->delReference();
    f13->delReference();
    f14->delReference();
    f15->delReference();
    f16->delReference();
    f17->delReference();
 
    c0->delReference();
    c1->delReference();
    c2->delReference();
    c3->delReference();
    c4->delReference();
    c5->delReference();

    app->drawNet( net, "psfiles/cnet.wrl" );

    TNet<3>  *fNet;

    for (i = 1; i <= noOfNets; i++)
    {
        timer.start();
        fNet = netHier->buildNet();
        timer.stop();
        cout << "time for refine : " << timer << endl;

        TMemory::statMemory();
//         sprintf( string, "psfiles/fnet%d.wrl", i );
//         app->drawNet( fNet, string );
    }// for

    cout << "size of vertex   : " << sizeof( *v0 ) << endl;
    cout << "size of edge     : " << sizeof( *e0 ) << endl;
    cout << "size of frontier : " << sizeof( *f0 ) << endl;
    cout << "size of cell     : " << sizeof( *c0 ) << endl;

    cerr << "noOfDofs : " << v0->getMaxInteriorName() << endl;

    delete app;
    delete netHier;

    TMemory::statistics();
}
