//
//  Project   : Navier-Stokes
//  File      : stokesConv.cc
//  Author    : Ronald Kriemann
//  Purpose   : test of 3D grids
//
// arch-tag: c2b235e2-ffb6-40c0-a81e-7687c0bdb460
//

#include "misc/var.hh"
#include "misc/memory.hh"
#include "misc/TFunction.hh"
#include "misc/matlab.hh"

#include "net/net.hh"
#include "net/TCdrBoundaryDesc.hh"
#include "net/TDomain.hh"

#include "discr/TCdrEqn.hh"
#include "discr/TConvDiffDoFManager.hh"
#include "discr/TConvDiffFE.hh"
#include "discr/TConvDiffHierarchy.hh"
#include "matrix/blockvector.hh"

#include "solver/secondNF.hh"
#include "solver/fastGaussSeidel.hh"
#include "solver/cgSolver.hh"
#include "solver/bicgStab.hh"
#include "solver/TMultigrid.hh"

#include "visual/drawapp.hh"
#include "visual/TPOVRay.hh"





#include "exfun1.hh"
#include "exeqn1.hh"
#include "geneqn.hh"

class TMyBasisCdrEqn : public TBasisCdrEqn<3>{

public:

    TMyBasisCdrEqn( TFloat aEpsilon = 1.0, 
                    TBlockVector<TFloat> *aConvVector = NULL):TBasisCdrEqn<3>(aEpsilon,aConvVector)
    {};

    virtual ~TMyBasisCdrEqn () { }
   
    void evalDiff(TVector <3> *v, TMatrix<3>  *D){
        TVector<3> * av; av=v;
        (*D)[0][0]=1;(*D)[0][1]=0; (*D)[0][2]=0;
        (*D)[1][0]=0;(*D)[1][1]=1; (*D)[1][2]=0;
        (*D)[2][0]=0;(*D)[2][1]=0; (*D)[2][2]=1;
    }

    void evalConv(TVector <3> *v, TVector<3>  *B){
        TVector<3> * av; av=v;
        (*B)[0]=1;(*B)[1]=2; (*B)[2]=3;
          
    }


    TFloat evalReac(TVector <3> *v) const{
        TVector<3> * av; av=v;
        return 0.0;
    }
   
    TFloat evalRhs(TVector <3> *v ) const{
        TVector<3> * av; av=v;
        TFloat a=(*v)[0];
        TFloat b=(*v)[1];
        TFloat c=(*v)[2];
     
        return -6;
    }   
};

//
// mainfunction
//
int
main( int argc, char **argv ) 
{
    uint maxNoOfNets=10;
    uint mf=4;
    TDrawApp *app = new TDrawApp;
    
#include "geneqn.cc"



  
    // read initial net and supply boundary Descriptons
    TNet<3>  *net3D = new TNet<3>( "grids/cube_2bounds.tri" );
    //TNet<3>  *net3D = new TNet<3>( "grids/cube.tri" );
    TNet<3>  *dNet;

    // in "grids/cube.tri", only one TBoundary is defined
    // therefore, one Description is enough

    TCdrBoundaryDesc<6> * boundDescN = new TCdrBoundaryDesc<6>(1);
    boundDescN->setType(0,NEU_TYPE);
    boundDescN->setFunc(0,Neumann);
    boundDescN->setDoFInfo();

    TCdrBoundaryDesc<3> * boundDescD = new TCdrBoundaryDesc<3>(1);
    boundDescD->setType(0,DIR_TYPE);
    boundDescD->setFunc(0,Dirichlet);
    boundDescD->setDoFInfo();


    TListIterator< TBoundary > iter(pDomain->getBoundaries()  );


    TBoundary *bound=iter.first();

    while ((bound = iter++) != NULL){
        cout<<bound->getName()<<endl;
        if(bound->getName()=="b1")
            bound->setBoundaryDesc(boundDescD);
        if(bound->getName()=="b2")
            bound->setBoundaryDesc(boundDescN);
        if(bound->getName()=="b3")
            bound->setBoundaryDesc(boundDescN);
        if(bound->getName()=="b4")
            bound->setBoundaryDesc(boundDescN);
        if(bound->getName()=="b5")
            bound->setBoundaryDesc(boundDescN);

    } 

    //bound->setBoundaryDesc(boundDescD);
 
    // build Hierarchy
    TNetHierarchy<3> *net3DHierarchy  = new TNetHierarchy<3>();
    net3DHierarchy->add(net3D);


    for (uint i=0;i<=mf;i++) {
        cout<<"i: "<<i<<endl;
        net3DHierarchy->buildNet();
        cout<<" Verfeinern fertig"<<endl;
    }

    // app.drawNet( net3D, "psfiles/cnet" );
    //app.drawNet( fnet, "psfiles/fnet" );
    //app.drawNet( ffnet, "psfiles/ffnet" );
    TBlockMatrix *aA; 
    TBlockVector<TFloat> *aRhs,*aX;
    TBlockMatrix **pMatrix     = new (TBlockMatrix*)[maxNoOfNets];
    TBlockVector<TFloat> **pRHS=new (TBlockVector<TFloat>*)[maxNoOfNets];
    TBlockVector<TFloat> **pX  =new (TBlockVector<TFloat>*)[maxNoOfNets];
    TProlongation<3> **pProlongation= new (TProlongation<3>*)[maxNoOfNets];

    for (uint i=0; i<maxNoOfNets;i++){
        pMatrix[i]=NULL;
        pRHS[i]=NULL;
        pX[i]=NULL;
        pProlongation[i]=NULL;
    }


    TConvDiffDoFManager<3> *DoFManager= new TConvDiffDoFManager<3>();
    TConvDiffFE<3> *discretiser= new TConvDiffFE<3>(Tabata,PRINT_DISCRETISATION);
  


    TConvDiffHierarchy<3>  *cdrHierarchy= new TConvDiffHierarchy<3>();
    cdrHierarchy->buildHierarchy( DoFManager,cdrEqn,discretiser,
                                  net3DHierarchy,pMatrix,pRHS,pX,
                                  pProlongation);


    // app->drawNet( dnet, "psfiles/ffcube", VRML );

      
    // check matrix

    /*
      matWriteMatrixBlock("/tmp/tpr/matrixf.mat", (*pMatrix[noOfNets-1])[0][0],
      (*pMatrix[noOfNets-1])[0][0].getNoOfRows(),"Af");
      matWriteMatrixBlock("/tmp/tpr/matrixc.mat", (*pMatrix[noOfNets-2])[0][0],
      (*pMatrix[noOfNets-2])[0][0].getNoOfRows(),"Ac");
  
      matWriteMatrixBlock( "/tmp/tpr/prol.mat",*(pProlongation[noOfNets-2]->pBlockProlong[0]), 
      pProlongation[noOfNets-2]->pBlockProlong[0]->getcDim(),"Prol" );
      */    
      

    // no of this net
    T3DNetHierarchy::TNetListIterator *iterNet;
    iterNet = new T3DNetHierarchy::TNetListIterator( net3DHierarchy->getList() );
    // last net
    dNet=iterNet->last();
    // no of this net
    uint lastnetNumber = net3DHierarchy->getNoOfNets()-1;
    // dim of the corresponding matrix
    ulong dim = (*(pMatrix[lastnetNumber]))[0][0].getSize();
 
    aA=pMatrix[lastnetNumber];
    aRhs=pRHS[lastnetNumber];
    aX=pX[lastnetNumber];
    
matWriteVectorBlock("/tmp/tpr/rhs.mat",  (*aRhs)[0], "r_neu" );
matWriteMatrixBlock("/tmp/tpr/Matrix.mat",(*aA)[0][0],(*aA)[0][0].getNoOfRows() , "A_neu" );

    // cout <<"rhs: "<<endl<<(*aRhs)[0]<<endl;

    // solve
    TFastSymmetricGaussSeidel *sgs = new TFastSymmetricGaussSeidel( );
    TSymmetricGaussSeidel *ssgs = new TSymmetricGaussSeidel( );
    TBicgStab *bicgStab = new TBicgStab(ssgs );
    TCGSolver *cg = new TCGSolver(ssgs );

    bicgStab->setAccuracy( 1e-15 );
    bicgStab->setMaxit( 400 );
    bicgStab->setVerbosity( 5);
    cg->setAccuracy( 1e-15 );
    cg->setMaxit( 40 );
    cg->setVerbosity( 0);
    aX->set( 1.0 ); 
    //bicgStab->solve(aA,aRhs,aX);


    uint fNet=2;
    uint cyc=1;
    uint steps=12;
    uint psm=2;  

    TMultigrid<3> *multigrid = new TMultigrid<3>(pMatrix, pRHS, pX, 
                                                 pProlongation,pProlongation ,
                                                 fNet,  lastnetNumber, 
                                                 cyc, sgs, sgs, cg,steps);

    //sgs->setMaxit( 20 ); 
    //sgs->setVerbosity( 7 );
    //sgs->setMode(0);
    //sgs->solve(aA,aRhs,aX);


    multigrid->setPreSmoothingSteps(1);
    multigrid->setPostSmoothingSteps(1);
    multigrid->setMode( 1 );
    multigrid->setVerbosity( 7 );

    sgs->setMaxit( psm ); 
    sgs->setVerbosity( 0 );
    sgs->setMode(2);

    ssgs->setMaxit( 1 ); 
    ssgs->setVerbosity( 0 );
    ssgs->setMode(2);
    
    multigrid->solve();


     
    // check accuracy
  
    TDynArray<TVector < 3 > * > arrCoord(dim);
    discretiser->getCoord ( dNet,arrCoord);
    cout<<"size: "<<arrCoord.getSize()<<endl;
  
    TBlockVector<TFloat> *perror= new TBlockVector<TFloat>(1,dim);
    TBlockVector<TFloat> *pexact= new TBlockVector<TFloat>(1,dim);
    
    for (ulong j=0; j<dim;j++){
        (*pexact)[0][j]= Dirichlet->eval ((arrCoord[j]));
        //  cout<<" X: "<<(*pX)[0][j]<<" coord: "<<*(arrCoord[j])<<endl;
    }
  
    (*perror)=(*aX)-(*pexact);
    cout<<"Fehler Diff: "<<setprecision(5)<<perror->euclidNorm()/sqrt(dim)<<endl;
   
    matWriteVectorBlock("/tmp/tpr/err.mat",  (*perror)[0], "e" );
    matWriteVectorBlock("/tmp/tpr/rhs.mat",  (*aRhs)[0], "r" );

    

    delete discretiser;
    delete aRhs;
    delete aX;
    delete aA;
    delete DoFManager;
    delete perror;
    delete pexact;
 
   


    TMemory::statistics();
}
 
