//
//  Project   : libnet
//  File      : convDiffHierarchy.cc
//  Author    : Sabine Gutsch / Thomas Probst 
//  Purpose   : class to build a matrix and prolongation hierarchy for
//              the convection-diffusion equation
//
// arch-tag: 07e79cf7-56d3-430a-ace2-a20c4d8824d0
//

#include "TConvDiffHierarchy.hh"

template <int N>
void 
TConvDiffHierarchy<N>::buildHierarchy ( TBasisDoFManager<N> *dofManager,
                                        TBasisEqn<N> *eqn,
                                        TBasisFE<N> *fe,
                                        TNetHierarchy<N> *netHier, 
                                        TBlockMatrix **pMatrix, 
                                        TBlockVector<TFloat> **pRHS, 
                                        TBlockVector<TFloat> **pX )
{
   
    uint noOfNets = netHier->getNoOfNets();
    uint noOfDoFs[noOfNets];
    uint blockSize[1];

    typename TNetHierarchy<N>::TNetListIterator iterNet( netHier->getList() );
 
    TNet<N> *currentNet = iterNet.first();
    TNet<N> *prevNet    = iterNet.first();
    for (uint i = 0; i < noOfNets; i++) 
    { //cout<<" i: "<<i<<endl;
          
            dofManager->initDoFDesc(currentNet); 
            pMatrix[i] = dofManager->buildStiffnessMatrix(currentNet);
            noOfDoFs[i]=(*(pMatrix[i]))[0][0].getSize();
            //cout<<" Dofs: "<< noOfDoFs[i]<<endl;
            if (noOfDoFs[i] >0){
                blockSize[0] = noOfDoFs[i];
                pX[i] = new TBlockVector<TFloat>( 1, blockSize );
                pRHS[i] = new TBlockVector<TFloat>( 1, blockSize );
                pRHS[i]->set(0.0);

                TConvDiffFE<N> *cfe=(TConvDiffFE<N>*)fe;
                cfe->discretize( eqn, currentNet, pMatrix[i], pRHS[i]);
            }
            else{
                delete pMatrix[i];
                pMatrix[i] =NULL;
                pRHS[i]=NULL;
                pX[i]=NULL;
            }// else
        
            if ( i > 0 ) 
            prevNet = currentNet;
            
            currentNet = iterNet.next();
    }// for
   
}

template class TConvDiffHierarchy<3>;
