//
//  Project   : libnet
//  File      : confDiffHierarchy.cc
//  Author    : Sabine Gutsch / Thomas Probst 
//  Purpose   : class to build a matrix and prolongation hierarchy for
//              the convection-diffusion equation
//
// arch-tag: 6ec1c894-91cb-4c5e-b994-6b40cca661f2
//

#include "convDiffHierarchy.hh"

void 
T2DConvDiffHierarchy::buildHierarchy ( TBasisDoFManager<2> *dofManager,
                                       T2DBasisEqn *eqn,
                                       T2DBasisFE *fe,
                                       TNetHierarchy<2> *netHier, 
                                       TBlockMatrix **pMatrix, 
                                       TBlockVector<TFloat> **pRHS, 
                                       TBlockVector<TFloat> **pX )
{
    uint   noOfNets = netHier->getNoOfNets();
    uint * noOfInteriorNodesNet = new uint[noOfNets];
    uint   blockSize[1];

    T2DNetHierarchy::TNetListIterator iterNet( netHier->getList() );
    
    T2DNet *currentNet = iterNet.first();
    T2DNet *prevNet    = iterNet.first();
    for (uint i = 0; i < noOfNets; i++) 
    {
        noOfInteriorNodesNet[i] = currentNet->getNoOfInteriorNodes();
        if ( noOfInteriorNodesNet[i] == 0 ) 
        {
            pMatrix[i] = NULL;
            pRHS[i] = NULL;
            pX[i] = NULL;
        }// if
        else 
        {
            pMatrix[i] = dofManager->buildStiffnessMatrix(currentNet);
            blockSize[0] = noOfInteriorNodesNet[i];
            pRHS[i] = new TBlockVector<TFloat>( 1, blockSize );
            pX[i] = new TBlockVector<TFloat>( 0.0, *pRHS[i] );
            fe->discrDiff( eqn, currentNet, pMatrix[i], pRHS[i]);
            fe->discrConv( eqn, currentNet, 
                           &((*pMatrix[i])[0][0]), pRHS[i]); 
            if ( eqn->getReacc() !=  noReac) { // reaction term != zero
                TBlockVector<TFloat> *pM = new TBlockVector<TFloat>( 0.0, *pX[i]);
                // lumped mass matrix in vector pM
                fe->discrMass(eqn, currentNet, &((*pM)[0])); 
                for (ulong j = 0; j < blockSize[0]; j++)
                    (*pMatrix[i])[0][0][j][j] += (*pM)[0][j];
                delete pM;
            }
            
            
        }// else

        if ( i > 0 ) 
            prevNet = currentNet;
       
        currentNet = iterNet.next();
    }// for

    delete[] noOfInteriorNodesNet;
}
