//
//  Project     : libgraph
//  File        : T3DFaceOrient.cc
//  Author      : Ronald Kriemann
//  Purpose     : class for a implementing methods to
//                handle orientation of 3d faces
//
// arch-tag: ff020516-618f-4c1f-a74d-1a7e0a45e441
//

#include "T3DFrontier.hh"

#include "vector.hh"

#include "T3DFaceOrient.hh"

#define CHECK_ORIENT

/////////////////////////////////////////////
//
// constructor
//
T3DFaceOrient::T3DFaceOrient ()
{}

/////////////////////////////////////////////
//
// methods to handle orientation
//

//
// set orientation of face (explicit/by adjacent face)
//
void
T3DFaceOrient::setOrient ( T3DFrontier * pFace,
                           uint i0, uint i1, uint i2 )
{
    setOrientIdx( pFace, 0, i0 );
    setOrientIdx( pFace, 1, i1 );
    setOrientIdx( pFace, 2, i2 );
}

void
T3DFaceOrient::setOrient ( T3DFrontier * pFace,
                           T3DFrontier * pOrientFace )
{
    if ((pFace == NULL) || (pOrientFace == NULL))
    {
        cout << "T3DFaceOrient::setOrient (...) : got NULL arg" << endl;
        return;
    }// if

#ifdef CHECK_ORIENT
    if ( ! hasOrient( pOrientFace ) )
    {
        cout << "T3DFaceOrient::setOrient (...) : face has no orientation" << endl;
        return;
    }// if
#endif
    
    TVector<3>  *v0, *v1, *v2;
    TVector<3>  *u0, *u1, *u2;
    bool         bOrient012, bOrient021;

    v0 = pOrientFace->getNode( getOrientIdx( pOrientFace, 0 ) );
    v1 = pOrientFace->getNode( getOrientIdx( pOrientFace, 1 ) );
    v2 = pOrientFace->getNode( getOrientIdx( pOrientFace, 2 ) );
    
    u0 = pFace->getNode( 0 );
    u1 = pFace->getNode( 2 );
    u2 = pFace->getNode( 4 );

    bOrient012 = bOrient021 = FALSE;
    
    if (u0 == v0)
    {
        if      (u1 == v1)
            bOrient021 = TRUE;
        else if (u1 == v2)
            bOrient012 = TRUE;
        else if (u2 == v1)
            bOrient012 = TRUE;
        else if (u2 == v2)
            bOrient021 = TRUE;
    }// if
    else if (u0 == v1)
    {
        if      (u1 == v0)
            bOrient012 = TRUE;
        else if (u1 == v2)
            bOrient021 = TRUE;
        else if (u2 == v0)
            bOrient021 = TRUE;
        else if (u2 == v2)
            bOrient012 = TRUE;
    }// else if
    else if (u0 == v2)
    {
        if      (u1 == v0)
            bOrient021 = TRUE;
        else if (u1 == v1)
            bOrient012 = TRUE;
        else if (u2 == v0)
            bOrient012 = TRUE;
        else if (u2 == v1)
            bOrient021 = TRUE;
    }// else if
    else if (u1 == v0)
    {
        if      (u0 == v1)
            bOrient012 = TRUE;
        else if (u0 == v2)
            bOrient021 = TRUE;
        else if (u2 == v1)
            bOrient021 = TRUE;
        else if (u2 == v2)
            bOrient012 = TRUE;
    }// else if
    else if (u1 == v1)
    {
        if      (u0 == v0)
            bOrient021 = TRUE;
        else if (u0 == v2)
            bOrient012 = TRUE;
        else if (u2 == v0)
            bOrient012 = TRUE;
        else if (u2 == v2)
            bOrient021 = TRUE;
    }// else if
    else if (u1 == v2)
    {
        if      (u0 == v0)
            bOrient012 = TRUE;
        else if (u0 == v1)
            bOrient021 = TRUE;
        else if (u2 == v0)
            bOrient021 = TRUE;
        else if (u2 == v1)
            bOrient012 = TRUE;
    }// else if
    
    if (bOrient012)
    {
        setOrient( pFace, 0, 2, 4 );
        return;
    }// if

    if (bOrient021)
    {
        setOrient( pFace, 0, 4, 2 );
        return;
    }// if

    cout << "T3DFaceOrient::setOrient (...) : no common edge between faces" << endl;
    exit( 1 );
}

//
// check if (v0,v1) is in orientation of face
//
bool
T3DFaceOrient::cmpOrient ( T3DFrontier * pFace,
                           TVector<3>  * pNode0,
                           TVector<3>  * pNode1 )
{
    if ((pFace == NULL) || (pNode0 == NULL) || (pNode1 == NULL))
    {
        cout << "T3DFaceOrient::cmpOrient (...) : got NULL arg" << endl;
        return FALSE;
    }// if

#ifdef CHECK_ORIENT
    if ( ! hasOrient( pFace ) )
    {
        cout << "T3DFaceOrient::cmpOrient (...) : face has no orientation" << endl;
        return FALSE;
    }// if
#endif
    
    TVector<3>  *v0, *v1, *v2;

    v0 = pFace->getNode( getOrientIdx( pFace, 0 ) );
    v1 = pFace->getNode( getOrientIdx( pFace, 1 ) );
    v2 = pFace->getNode( getOrientIdx( pFace, 2 ) );

    if (pNode0 == v0)
    {
        if (pNode1 == v1)
            return TRUE;
        else
            return FALSE;
    }// if

    if (pNode0 == v1)
    {
        if (pNode1 == v2)
            return TRUE;
        else
            return FALSE;
    }// if

    if (pNode0 == v2)
    {
        if (pNode1 == v0)
            return TRUE;
        else
            return FALSE;
    }// if
    
    return FALSE;
}

//
// check if orientation of (adjacent) faces is compatible
//
bool
T3DFaceOrient::chkOrient ( T3DFrontier * pFace1,
                           T3DFrontier * pFace2 )
{
    if ((pFace1 == NULL) || (pFace2 == NULL))
    {
        cout << "T3DFaceOrient::chkOrient (...) : got NULL arg" << endl;
        return FALSE;
    }// if

#ifdef CHECK_ORIENT
    if (( ! hasOrient( pFace1 ) ) || ( ! hasOrient( pFace2 ) ))
    {
        cout << "T3DFaceOrient::chkOrient (...) : face has no orientation" << endl;
        exit( 1 );
    }// if
#endif
    
    //
    // catch common nodes
    //
    TVector<3>  *v0, *v1, *v2;
    TVector<3>  *u0, *u1, *u2;

    v0 = pFace1->getNode( getOrientIdx( pFace1, 0 ) );
    v1 = pFace1->getNode( getOrientIdx( pFace1, 1 ) );
    v2 = pFace1->getNode( getOrientIdx( pFace1, 2 ) );

    u0 = pFace2->getNode( getOrientIdx( pFace2, 0 ) );
    u1 = pFace2->getNode( getOrientIdx( pFace2, 1 ) );
    u2 = pFace2->getNode( getOrientIdx( pFace2, 2 ) );

    if (v0 == u0)
    {
        if      (v1 == u1)
            return FALSE;
        else if (v1 == u2)
            return TRUE;
        else if (v2 == u1)
            return TRUE;
        else if (v2 == u2)
            return FALSE;
    }// if
    else if (v0 == u1)
    {
        if      (v1 == u0)
            return TRUE;
        else if (v1 == u2)
            return FALSE;
        else if (v2 == u0)
            return FALSE;
        else if (v2 == u2)
            return TRUE;
    }// else if
    else if (v0 == u2)
    {
        if      (v1 == u0)
            return FALSE;
        else if (v1 == u1)
            return TRUE;
        else if (v2 == u0)
            return TRUE;
        else if (v2 == u1)
            return FALSE;
    }// else if
    else if (v1 == u0)
    {
        if      (v0 == u1)
            return TRUE;
        else if (v0 == u2)
            return FALSE;
        else if (v2 == u1)
            return FALSE;
        else if (v2 == u2)
            return TRUE;
    }// else if
    else if (v1 == u1)
    {
        if      (v0 == u0)
            return FALSE;
        else if (v0 == u2)
            return TRUE;
        else if (v2 == u0)
            return TRUE;
        else if (v2 == u2)
            return FALSE;
    }// else if
    else if (v1 == u2)
    {
        if      (v0 == u0)
            return TRUE;
        else if (v0 == u1)
            return FALSE;
        else if (v2 == u0)
            return FALSE;
        else if (v2 == u1)
            return TRUE;
    }// else if

    cout << "T3DFaceOrient::chkOrient (...) : no common edge" << endl;

    return FALSE;
}

//
// check of face has legal orientation
//
bool
T3DFaceOrient::hasOrient ( T3DFrontier * pFace )
{
    if (pFace == NULL)
    {
        cout << "T3DFaceOrient::hasOrient (...) : got NULL arg" << endl;
        return FALSE;
    }// if

    uint  i0, i1, i2;

    i0 = getOrientIdx( pFace, 0 );
    i1 = getOrientIdx( pFace, 1 );
    i2 = getOrientIdx( pFace, 2 );

    //
    // check if values are 0, 2, 4
    //
    if (i0 == 0)
    {
        if (((i1 == 2) && (i2 == 4)) ||
            ((i1 == 4) && (i2 == 2)))
            return TRUE;
        else
            return FALSE;
    }// if
    
    if (i1 == 0)
    {
        if (((i0 == 2) && (i2 == 4)) ||
            ((i0 == 4) && (i2 == 2)))
            return TRUE;
        else
            return FALSE;
    }// if

    if (i2 == 0)
    {
        if (((i0 == 2) && (i1 == 4)) ||
            ((i0 == 4) && (i1 == 2)))
            return TRUE;
        else
            return FALSE;
    }// if
    
    return FALSE;
}

//
// get/set each index sep.
//
uint
T3DFaceOrient::getOrientIdx ( T3DFrontier * pFace,
                              uint idx )
{
    if (pFace == NULL)
    {
        cout << "T3DFaceOrient::getOrientIdx (...) : got NULL arg" << endl;
        return 6;
    }// if

    uint  mask = 0xff;
    uint  data = (uint) pFace->data();

    switch (idx)
    {
        case 0  :
            mask <<= 8;
            data  &= mask;
            data >>= 8;
            break;
            
        case 1  :
            mask <<= 16;
            data  &= mask;
            data >>= 16;
            break;
            
        case 2  :
            mask <<= 24;
            data  &= mask;
            data >>= 24;
            break;
            
        default :
            cout << "T3DFaceOrient::getOrientIdx (...) : wrong index" << endl;
            return 0;
    }// switch

    return data;
}

void
T3DFaceOrient::setOrientIdx ( T3DFrontier * pFace,
                              uint idx, uint i )
{
    if (pFace == NULL)
    {
        cout << "T3DFaceOrient::setOrientIdx (...) : got NULL arg" << endl;
        return;
    }// if

    uint  mask = 0xff;
    uint  data = (uint) pFace->data();

    switch (idx)
    {
        case 0  :
            mask <<= 8;
            mask   = ~mask;
            data  &= mask;
            mask   = i;
            mask <<= 8;
            data  |= mask;
            break;

        case 1  :
            mask <<= 16;
            mask   = ~mask;
            data  &= mask;
            mask   = i;
            mask <<= 16;
            data  |= mask;
            break;

        case 2  :
            mask <<= 24;
            mask   = ~mask;
            data  &= mask;
            mask   = i;
            mask <<= 24;
            data  |= mask;
            break;
            
        default :
            cout << "T3DFaceOrient::setOrientIdx (...) : wrong index" << endl;
            return;
    }// switch

    pFace->setData( (T3DFrontier::data_t) data );
}

//
// print orientation
//
void
T3DFaceOrient::showOrient ( T3DFrontier * pFace )
{
    if (pFace == NULL)
    {
        cout << "T3DFaceOrient::showOrient (...) : got NULL arg" << endl;
        return;
    }// if

#ifdef CHECK_ORIENT
    if ( ! hasOrient( pFace ) )
    {
        cout << "T3DFaceOrient::showOrient (...) : face has no orientation" << endl;
        return;
    }// if
#endif

    cout << * pFace->getNode( getOrientIdx( pFace, 0 ) ) << "    |    "
         << * pFace->getNode( getOrientIdx( pFace, 1 ) ) << "    |    "
         << * pFace->getNode( getOrientIdx( pFace, 2 ) ) << endl;
}
