//
//  Project     : libgraph
//  File        : T3DVertexData.cc
//  Author      : Ronald Kriemann
//  Purpose     : class for put add. data to vertices
//                (needed for flow surfaces)
//
// arch-tag: 81187462-e809-4771-9a66-8c7cb70fd5c7
//

#include "vertex.hh"

#include "T3DMarkableNode.hh"
#include "T3DCellEdgeGraph.hh"

#include "T3DVertexData.hh"

////////////////////////////////////////////
//
// constructors and destructors
//
T3DVertexData::T3DVertexData ( TVertex<3> * v )
{
    if (v == NULL)
        cout << "T3DVertexData::T3DVertexData (...) : got NULL node" << endl;
    else
        v->addRef();
    
    pNode        = v;
    pSurfaceNode = NULL;
    pFlowSurface = NULL;

    // remove one reference (so default is no reference)
    delRef();
}

T3DVertexData::~T3DVertexData ()
{
    if (pNode != NULL)
        pNode->free();
}

////////////////////////////////////////////
//
// access member variables
//

const bool
T3DVertexData::inSurface () const
{
    return (pFlowSurface != NULL);
}

TVertex<3> *
T3DVertexData::node () const
{
    return pNode;
}

TNode *
T3DVertexData::surfaceNode () const
{
    return pSurfaceNode;
}

void
T3DVertexData::setSurfaceNode ( TNode * pSNode )
{
    pSurfaceNode = pSNode;
}

T3DCellEdgeGraph *
T3DVertexData::flowSurface () const
{
    return pFlowSurface;
}

void
T3DVertexData::setFlowSurface ( T3DCellEdgeGraph * pFS )
{
    pFlowSurface = pFS;
}
