//
//  Project     : libgraph
//  File        : coarsener.hh
//  Author      : Thomas Probst
//  Purpose     : define coarse grid
//
// arch-tag: c2abf4f8-9937-423b-bb84-0c270d01c26d
//

#include "T2DPlanarGraph.hh"
#include "TRedBlack.hh"

#include "TCoarsener.hh"

///////////////////////////////////////////
//
// constructor and destructor
//
    
TCoarsener::TCoarsener ( TFloat xi,
                         uint   criterion,
                         uint   strategy )
{
    TCoarsener::xi        = xi;
    TCoarsener::criterion = criterion;
    TCoarsener::strategy  = strategy;
}

TCoarsener::~TCoarsener()
{}

///////////////////////////////////////////
//
// access member variables
//

const uint
TCoarsener::getStrategy () const
{
    return strategy;
}

void
TCoarsener::setStrategy ( uint strategy )
{
    TCoarsener::strategy = strategy;
}

///////////////////////////////////////////
//
// do the coarsening
//
    
TPermutation *
TCoarsener::coarse ( TMatrixBlock * pMatrix,
                     uint        & noOfReds)
{
    if (pMatrix == NULL)
    {
        cout << "TCoarsener::coarse (...) : got NULL arg" << endl;
        return NULL;
    }// if
        
    TDynArray< TVector<2> * > arrVectors( pMatrix->getNoOfRows() );
    T2DPlanarGraph            graph( arrVectors, * pMatrix,
                                     xi, criterion );
    TRedBlack                 redBlack;
    TPermutation            * pPermutation;

    pPermutation = NULL;
    
    if (strategy == SEMI)
        pPermutation = redBlack.orderBFS( graph, noOfReds );
    
    if (strategy == FULL)
        pPermutation = redBlack.orderBFS_isolated( graph, noOfReds);

    return pPermutation;
}
