//
//  Project     : libgraph
//  File        : TPlanarCycle.cc
//  Author      : Ronald Kriemann
//  Purpose     : classes for different graphs
//
// arch-tag: 464381e6-0dc0-4650-bf87-03a7a4d2cd27
//

#include "TPlanarCycle.hh"

#define DELETED     0x1
#define REALLY_NEW  0x2
#define IS_SUCC     0x4
#define IS_PRED     0x8

///////////////////////////////////////////////////
//
// class for a named cycle
//

//
// constructor
//
TPlanarCycle::TPlanarCycle ()
        : TGraph()
{
    _name    =  (uint) -2;
    _phi     =  0;
    pZNode   = NULL;
}

//
// access node in Z
//
TCycleNode *
TPlanarCycle::getZNode () const
{
    return pZNode;
}

void
TPlanarCycle::setZNode ( TCycleNode * pNode )
{
    pZNode = pNode;
}
    
//
// access the name
//
const uint 
TPlanarCycle::name () const
{
    return _name;
}

void 
TPlanarCycle::setName ( uint n )
{
    _name = n;
}

//
// access phi
//
const uint 
TPlanarCycle::phi () const
{
    return _phi;
}

void 
TPlanarCycle::setPhi ( uint p )
{
    _phi = p;
}

//
// access delflag
//
const bool 
TPlanarCycle::isDeleted () const
{
    return hasFlag( DELETED );
}

void 
TPlanarCycle::setDeleted ( bool d )
{
    if (d == TRUE)
        setFlag( DELETED );
    else
        clearFlag( DELETED );
}

//
// flag to indicate if cycle is really new
//
const bool
TPlanarCycle::isNew () const
{
    return hasFlag( REALLY_NEW );
}

void
TPlanarCycle::setNew ( bool b )
{
    if (b == TRUE)
        setFlag( REALLY_NEW );
    else
        clearFlag( REALLY_NEW );
}

//
// flags to indicate that cycle is succ/pred of some other cycle
//
const bool
TPlanarCycle::isSucc () const
{
    return hasFlag( IS_SUCC );
}

void
TPlanarCycle::setSucc ( bool b )
{
    if (b)
        setFlag( IS_SUCC );
    else
        clearFlag( IS_SUCC );
}

const bool
TPlanarCycle::isPred () const
{
    return hasFlag( IS_PRED );
}

void
TPlanarCycle::setPred ( bool b )
{
    if (b)
        setFlag( IS_PRED );
    else
        clearFlag( IS_PRED );
}

//
// access list of cycles
//

void
TPlanarCycle::addCycle ( TPlanarCycle * pCycle )
{
    listCycles.append( pCycle );
}

void
TPlanarCycle::clearCycleList ()
{
    listCycles.removeAll();
}

TSList< TPlanarCycle * > &
TPlanarCycle::cycleList ()
{
    return listCycles;
}
