// arch-tag: 64b9c8e2-52a0-4eb2-853d-b05a19b9f741
//


#include <stdio.h>

#include "timer.hh"
#include "list.hh"
#include "cmdline.hh"

#ifdef MATLAB
#include "matlab.hh"
#endif

#include "vertex.hh"
#include "regular.hh"
#include "net.hh"

#include "drawapp.hh"

#include "blockinfo.hh"
#include "graphtool.hh"
#include "nodes.hh"
#include "rcm.hh"
#include "T2DPlanarGraph.hh"
#include "T2DPlanarNode.hh"

#include "crosswind.hh"

// create graph


uint cwOrdering  (TMatrixBlock *pA,TDynArray< TVector<2> * >  &arrVectors,
                  TDynArray< TVector<2> * > &convVectors, 
                  TVectorBlock<uint> &permutation,
                  TFloat xiCrossWind, uint criterionCrossWind,
                  TFloat xiCompGraph, uint criterionCompGraph,
                  TFloat xiInnerBlocks, uint criterionInnerBlocks){
    
    TDrawApp *app = new TDrawApp;
    bool draw=param.draw;
    uint  fvsSize;
        
    if(draw==TRUE){
        T2DPlanarGraph  normalGraph( arrVectors,
                                     *pA,xiInnerBlocks,criterionInnerBlocks);
        app->draw( normalGraph,"graph" );
    }
    
    uint dim = pA->getNoOfRows();
  
    if (dim>1){
        // crosswind graph
        T2DPlanarGraph  graph( arrVectors, convVectors,
                               *pA, xiCrossWind, criterionCrossWind);
        TVectorBlock<uint>  permute; // permutation for the node wrt the components
        permute.setSize(dim);

        TNode *node;
        uint number=0;

        TListIterator<TNode>  iterNodes;
        TVector<2> z;
        uint nodeName = 0;
  
   
        // strong components of crosswind graph
        TGraphTool             graphtool;
        TList<TGraph> *plistComp = graphtool.tarjan( graph );
    
        TListIterator<TGraph>  iterComp( *plistComp );
        TGraph                *pComponent;
        uint noOfComponents= plistComp->getSize();
        uint i_Comp=0;

        cout << "number of components : " << noOfComponents << endl;

        //timer.stop();
        //cout << "time for tarjan : " << timer << endl;
        //    timer.start();


        // build graph of CW-components
        //   T2DPlanarGraph *componentGraph = new T2DPlanarGraph();
        //componentGraph->setType( _DIRECTED_ | _PLANAR_);
    
        TDynArray< T2DPlanarNode *> arrCompNodes(noOfComponents);
        TDynArray< TVector<2> * > arrCompNodesVec(noOfComponents);
        TDynArray<TNode *> arrNodes(dim);

        // vector to store block number
        TVectorBlock<uint> blockStartNumber (0,noOfComponents);
        TVectorBlock<uint> blockSize (0,noOfComponents);

   
        while ((pComponent = iterComp++) != NULL)
        {
            //  cout << "pComponent->getNoOfNodes() "<<i_Comp  << " from "<<number;
            iterNodes.setList( pComponent->getNodeList());
      
            z[0]=0.0; z[1]=0.0;
            long lengthOfComponent=0;
            blockStartNumber[i_Comp]=number;
            while ((node = iterNodes++) != NULL){
            
                permute[number++] = node->getName();  
                z+= *(arrVectors[node->getName()]);
                lengthOfComponent++;
            
                // store component number in each node
                uint *compnumber = new uint;
                *compnumber=i_Comp;
                node->setInfo(compnumber);
                // store pointer to nodes for direct access
                arrNodes[node->getName()]=node;
            }
            blockSize[i_Comp]=lengthOfComponent;
            //  cout<<" to "<<number-1<<" "<<i_Comp<<endl;
 
            // build graph of CWcomponents
 
            // average: calculate 'virtual' coordinates for a CWComponent as 
            //     average of the involved nodes' coordinates
            TFloat fac;
            fac=1.0/lengthOfComponent;
            z=fac*z;
            TVector<2> *coord = new TVector<2>(z);
        
            //  pNode = new T2DPlanarNode( i_Comp, _NO_MARK_,coord);
            //pNode->setWeight(1.0);
            //componentGraph->add( *pNode );
            //arrCompNodes[i_Comp] = pNode;

            arrCompNodesVec[i_Comp] = coord;
            i_Comp++;
        }

    
        // build graph of CW-components: 
    

        // edges for graph of CWcomponents
        // first: store information in matrix
        TMatrixBlock *adjMatrix = new TMatrixBlock (noOfComponents);
        uint  *arrIndex;
        TFloat *arrValues;
        TFloat  diagValue;
        uint    noOfEntries;
    
        iterComp.first();    
        while ((pComponent = iterComp++) != NULL)
        {
            iterNodes.setList( pComponent->getNodeList());
            while ((node = iterNodes++) != NULL){
            
                uint l = node->getName();
                arrIndex    = (*pA)[l].getIndex();
                arrValues   = (*pA)[l].getValue();
                diagValue   = (*pA)[l][l];
                noOfEntries = (*pA)[l].getNoOfEntries();
                uint compnumber=*( (uint*)node->getInfo());
                uint compnumber2;
                (*adjMatrix)[compnumber].writeableStatus(TRUE);
                (*adjMatrix)[compnumber][compnumber]=0.0;
                
                // for each edge
                for (uint i = 0; i < noOfEntries; i++){
                    uint k=arrIndex[i];
                    if (k != nodeName){
                        compnumber2 = *( (uint*)(arrNodes[k]->getInfo()));
                        (*adjMatrix)[compnumber][compnumber2]+=fabs((*pA)[l][k]);
                    } // arrIndex[i] != nodeName
                } // for 
                (*adjMatrix)[compnumber].writeableStatus(FALSE);
            } // iterNodes
        } // iterComp  
        //      cout <<" Matrix of Adja: "<<endl<< (*adjMatrix)<<endl;


        //timer.stop();
        //cout << "time for building component graph " << timer << endl;
        //timer.start();


        //  cout<<" size of componentGraph "<<componentGraph.getNoOfNodes()<<endl;
       
        // perform fvs on this graph of CWComponents
        TBlockInfo  *blockinfo = new TBlockInfo;
        blockinfo->setSortMethod( SORT_OFD );
    
        TVectorBlock<uint>  *compPerm= new TVectorBlock<uint>;
        compPerm = new TVectorBlock<uint>;
    
    
        blockinfo->buildFVSPerm(*compPerm,fvsSize,arrCompNodesVec ,
                                (*adjMatrix),  xiCompGraph,criterionCompGraph,
                                SEARCH_BFS,HEURISTIC_FVS);
        //    timer.stop();
        //cout << "time for FSV for  component graph " << timer << endl;
        //timer.start();
    
        cout<< "fvsSize of componentGraph: "<<fvsSize<<endl;
    
    
        // build inverse of compPerm
        TVectorBlock<uint> invCompPerm(*compPerm);
        invertPerm(invCompPerm,*compPerm);
  
        if(draw==TRUE){
            // graph for CWcomponents
            T2DPlanarGraph  componentGraph(arrCompNodesVec ,   (*adjMatrix), 
                                           xiCompGraph, criterionCompGraph);               
            app->draw( componentGraph,"graphC");
            app->draw( componentGraph,invCompPerm,"graphCInv");
        }

     

        //  cout<<"*(blockPerm[i]) \n"<< *(blockPerm[i])<<endl;
        //  cout<<"InvBlockPerm \n"<< invBlockPerm<<endl;
   
        // new permutation vector for all nodes

        // update permutation vector with respect to the component ordering
        TVectorBlock<uint> newPerm(permute);
        uint k=0;
        uint i;
        for ( i=0; i<noOfComponents;i++){
            uint blockNo = (*compPerm)[i];
            uint bsn = blockStartNumber[blockNo];
            uint bs  = blockSize[blockNo];
            for (uint l = bsn; l< bsn+bs;l++)
                newPerm[k++]=permute[l];
        } //for i 
   

        // sorted (original)graph
        TVectorBlock<uint> invPerm(newPerm);
        invertPerm(invPerm,newPerm);
    
        if(draw==TRUE)
            app->draw( graph, invPerm,"graph3");

    
        //timer.start();

        //reorder Matrix wrt order CWComponents
        (*pA).permute( &invPerm,&invPerm);
    
        // create diag blocks
        TVectorBlock<uint> *decompBlocks = new TVectorBlock<uint>(0,noOfComponents*2) ;
        uint bsn=0;
        for (i=0; i<noOfComponents;i++){
            uint blockNo = (*compPerm)[i];
            uint bs  = blockSize[blockNo];
            (*decompBlocks)[2*i]=bsn;
            (*decompBlocks)[2*i+1]=bsn+bs-1;
            bsn+=bs;
        }
        // decompose into diagonal blocks (each corresponting to one CWcomponent)
        (*pA).decompose(BLOCK_DIAG,decompBlocks);
   

        TVectorBlock <uint> globalPerm(0,dim);
        for (i=0; i<noOfComponents;i++){
            // order each block   
            //cout<<" ordering block component "<<i<<endl;
  
            uint localDim = blockSize[(*compPerm)[i]]; // local Blocksize
            uint localBlockStart= (*decompBlocks)[2*i];
            TVectorBlock <uint> localPerm(0,localDim);
            TVectorBlock <uint> invLocalPerm(localPerm);
            uint localFvsSize;
            
            TMatrixBlock *localMatrix=(*pA).getDiagBlock(i);
        
            TDynArray< TVector<2> * > localArrVec(localDim);
            for (uint k=0; k<localDim;k++){
                TVector<2> *vec = ((arrVectors)[newPerm[k+localBlockStart]]);
                localArrVec[k] = vec;
                //              cout<<" k: "<<k<<" k+localBlockStart : "<< k+localBlockStart<< " vec: ";
                //cout<<*localArrVec[k]<<endl;
            }
            TBlockInfo  *blockinfo2 = new TBlockInfo;
            blockinfo2->setSortMethod( SORT_OFD );
            blockinfo2->buildFVSPerm(localPerm ,localFvsSize,localArrVec,
                                     *localMatrix, xiInnerBlocks, 
                                     criterionInnerBlocks,
                                     SEARCH_BFS,HEURISTIC_FVS);
            // update (global) permutation 
            for (uint k=0; k<localDim;k++)
                globalPerm[k+localBlockStart]= newPerm[localPerm[k]+localBlockStart]; 
        }
        //    timer.stop();
        //cout << "time for FSV for inner components " << timer << endl;
   
        //matWriteMatrixBlock("/tmp/tpr/matrixFVS.mat",*pA,
        //                 pA->getNoOfRows(),"A1");

        //restore matrix order
        (*pA).permute( &newPerm,&newPerm);
  
        permutation=globalPerm;
    }// dim > 1 
    else 
        if (dim==0)
            permutation[0]=0;
    
    delete app;
    // return size of FVS 
    return fvsSize;
}// create
