//
//  Project     : libgraph
//  File        : edge.cc
//  Author      : Ronald Kriemann
//  Purpose     : classes for different edges in a graph
//  Date        : 04.08.1997
//
// arch-tag: 2587568b-44b5-40ed-910a-75155a665314
//

#include "edge.hh"

/////////////////////////////////////////////////////
//
//  TEdge
//

//
// constructor
//
TEdge::TEdge ( TNode& tail, TNode& head, uint n )
{
	pTail = &tail;
	pHead = &head;
	name  = n;

    if ((pTail == NULL) || (pHead == NULL))
        cout << "TEdge::TEdge (...) : got NULL nodes" << endl;
    
    // remove one reference (so default is no reference)
	delRef();
}

//
// destructor
//
TEdge::~TEdge ()
{}

//
// access the node pointer
//
TNode*
TEdge::getOther ( TNode * pNode ) 
{ 
	if (pTail == pNode)
		return pHead;
	else if (pHead == pNode)
		return pTail;
	else
	{
		cerr << "TEdge::getOther : no such node" << endl;
		return pHead;
	}// else
}

//
// access the name
//
const uint 
TEdge::getName () const 
{ 
	return name; 
}

//
// access the node pointer
//
TNode*
TEdge::getNode () 
{ 
	return pHead; 
}

TNode*
TEdge::getTail () 
{ 
	return pTail; 
}

TNode*
TEdge::getHead () 
{ 
	return pHead; 
}

//
// create new edge (and set specific info)
//
TEdge* 
TEdge::create ( TNode& tail, TNode& head, uint n )
{
	return new TEdge( tail, head, n );
}

//
// clone infos of an edge
//
TEdge*
TEdge::clone ( TNode& tail, TNode& head )
{
	return new TEdge( tail, head, name );
}

/////////////////////////////////////////////////////
//
//  TMarkableEdge
//

//
// constructor
//
TMarkableEdge::TMarkableEdge ( TNode& tail, TNode& head, int m, uint n )
	: TEdge( tail, head, n ), _mark( m )
{}

//
// access the mark
//
const int 
TMarkableEdge::mark () const 
{ 
	return _mark; 
}

void 
TMarkableEdge::setMark ( int m ) 
{ 
	_mark = m; 
}

//
// create new edge (and set specific info)
//
TMarkableEdge* 
TMarkableEdge::create ( TNode& tail, TNode& head, uint n )
{
	return new TMarkableEdge( tail, head, _mark, n );
}

//
// clone infos of an edge
//
TMarkableEdge*
TMarkableEdge::clone ( TNode& tail, TNode& head )
{
	return new TMarkableEdge( tail, head, _mark, name );
}

/////////////////////////////////////////////////////
//
//  TWeightedMarkableEdge
//

//
// constructor
//
TWeightedMarkableEdge::TWeightedMarkableEdge ( TNode& tail, TNode& head, 
											   int m, TFloat w, uint n )
	: TMarkableEdge( tail, head, m, n ), _weight(w)
{}

//
// access the weight
//
const TFloat 
TWeightedMarkableEdge::weight () const
{
    return _weight;
}

void 
TWeightedMarkableEdge::setWeight ( TFloat w )
{
    _weight = w;
}

//
// create new edge (and set specific info)
//
TWeightedMarkableEdge* 
TWeightedMarkableEdge::create ( TNode& tail, TNode& head, uint n )
{
	return new TWeightedMarkableEdge( tail, head, _mark, _weight, n );
}

//
// clone infos of an edge
//
TWeightedMarkableEdge*
TWeightedMarkableEdge::clone ( TNode& tail, TNode& head )
{
	return new TWeightedMarkableEdge( tail, head, _mark, _weight, name );
}
