//
//  Project     : libgraph
//  File        : nodes.cc
//  Author      : Ronald Kriemann
//  Purpose     : classes for different nodes
//
// arch-tag: 36d2d0ef-882b-41f6-b0c1-b06551d1ee27
//

#include "TPlanarCycle.hh"

#include "nodes.hh"

/////////////////////////////////////////////////////
//
//  TMarkableNode
//

//
// constructor
//
TMarkableNode::TMarkableNode ( TMarkableNode& node )
    : TNode( node )
{
    _mark = node._mark;
}

TMarkableNode::TMarkableNode ( uint n, int m )
    : TNode( n ), _mark( m )
{}

//
// access the mark
//
const int 
TMarkableNode::mark () const 
{ 
    return _mark; 
}

void 
TMarkableNode::setMark ( int m ) 
{ 
    _mark = m; 
}

//
// create a new nodes
//
TMarkableNode* 
TMarkableNode::clone () 
{ 
    return new TMarkableNode( name, _mark );
}

TMarkableNode* 
TMarkableNode::create ( uint n ) 
{ 
    return new TMarkableNode( n ); 
}

////////////////////////////////////////////////////////////
//
// class for a node which represents a cycle
//

//
// constructor
//
TCycleNode::TCycleNode ( uint n, TPlanarCycle *cycle )
    : TMarkableNode( n ), pCycle( cycle )
{}

//
// destructor
//
TCycleNode::~TCycleNode () 
{}

//
// access cycle
//
TPlanarCycle* 
TCycleNode::cycle () 
{ 
    return pCycle; 
}

void 
TCycleNode::setCycle ( TPlanarCycle *cycle ) 
{ 
    pCycle = cycle; 
}

//
// create a new nodes
//
TCycleNode* 
TCycleNode::clone () 
{ 
    return new TCycleNode( name, pCycle ); 
}

TCycleNode* 
TCycleNode::create ( uint n ) 
{ 
    return new TCycleNode( n, pCycle ); 
}
