//
//  Project     : libgraph
//  File        : number.cc
//  Author      : Ronald Kriemann
//  Purpose     : class for a numbering algorithm
//
// arch-tag: 85fe2a38-68d1-4137-b5da-47885076de89
//

#include "blockinfo.hh"
#include "number.hh"

//
// constructor
//
TNumberGraph::TNumberGraph ( TGraph& graph, TBlockInfo *blockinfo )
{
	pGraph = &graph;
	pBlockInfo = blockinfo;
}

//
// number all nodes
//
void 
TNumberGraph::number ()
{
	TListIterator<TNode> iter( pGraph->getNodeList() );
	TMarkableNode       *pNode;

	// set marks of all nodes to C
	iter.first();
	while ((pNode = (TMarkableNode*) iter++) != NULL)
		pNode->setMark( C );

	// now adjust marks
	iter.first();
	while ((pNode = (TMarkableNode*) iter++) != NULL)
	{
		if (pNode->mark() == C)
			setF( *pNode );

		if (pNode->mark() == C)
			setL( *pNode );
	}// while

	// place all C nodes in the permutation vector
	/*	if (pPermutation != NULL)
	{
		uint posC = posF;

		iter.first();
		while ((pNode = (TMarkableNode*) iter++) != NULL)
			if (pNode->mark() == C)
				(*pPermutation)[posC++] = pNode->getName();
	}// if*/
}

//
// set attr F to a node
//
void 
TNumberGraph::setF ( TMarkableNode& node )
{
	TListIterator<TEdge>  iter( node.getInEdges() );
	TEdge                *pEdge;
	TMarkableNode        *pNode;

	bool  bPredHaveF = TRUE;

	iter.first();
	while ((pEdge = iter++) != NULL)
	{
		pNode = (TMarkableNode*) pEdge->getTail();

		if (pNode->mark() != F)
		{
			bPredHaveF = FALSE;
			iter.last();
			iter++;
		}// if
	}// while

	if (bPredHaveF == TRUE)
	{
		node.setMark( F );
		if (pBlockInfo != NULL) pBlockInfo->addFNode( node );

		// do the same with all successors
		iter.setList( node.getOutEdges() );
		iter.first();
		while ((pEdge = iter++) != NULL)
		{
			pNode = (TMarkableNode*) pEdge->getHead();

			if (pNode->mark() == C)
				setF( *pNode );
		}// while
	}// if
}

//
// set attr L to a node
//
void 
TNumberGraph::setL ( TMarkableNode& node )
{
	TListIterator<TEdge>  iter( node.getOutEdges() );
	TEdge                *pEdge;
	TMarkableNode        *pNode;

	bool  bSuccHaveL = TRUE;

	iter.first();
	while ((pEdge = iter++) != NULL)
	{
		pNode = (TMarkableNode*) pEdge->getHead();
		if (pNode->mark() != L)
		{
			bSuccHaveL = FALSE;
			iter.last();
			iter++;
		}// if
	}// while

	if (bSuccHaveL == TRUE)
	{
		node.setMark( L );
		if (pBlockInfo != NULL) pBlockInfo->addLNode( node );

		// do the same with all predecessors
		iter.setList( node.getInEdges() );
		iter.first();
		while ((pEdge = iter++) != NULL)
		{
			pNode = (TMarkableNode*) pEdge->getTail();

			if (pNode->mark() == C)
				setL( *pNode );
		}// while
	}// if
}
