//
//  Project :  libvisual
//  File    :  T2DFlowView.hh
//  Purpose :  class for visualization of a 2D flow
//  Author  :  Ronald Kriemann
//
// arch-tag: 50b8f792-de8a-474f-8ffc-663654f8a187
//

#ifndef __T2DFLOWVIEW_HH
#define __T2DFlowVIEW_HH

#include "vector.hh"

#include "dynarray.hh"

#include "view.hh"

#include "basisEqn.hh"

//
//
//
class T2DFlowView : public TView
{
protected:
    // array of points in domain, we use to show
    // flow
    TDynArray< TVector<2> >  arrPoints;

    // array with flow assoc to a point in upper array
    TDynArray< TVector<2> >  arrFlow;

    // stepwidths for drawing flow
    TFloat                   stepWidthX, stepWidthY;
    
public:
    ///////////////////////////////////////////
    //
    // constructors and destructors
    //

    T2DFlowView ( T2DBasisEqn * pEqn,
                  TFloat stepX, TFloat stepY );

    ~T2DFlowView ();

    ///////////////////////////////////////////
    //
    // visualization methods
    //

    virtual void draw ( TPrinter & p );

protected:
    // encapsulate building of point/flow array
    void buildFlowArray ( T2DBasisEqn * pEqn );
};

#endif  // __T2DFLOWVIEW_HH
