//
//  Project   : libnet
//  File      : T2DFrontier.hh
//  Author    : Sabine Gutsch / Ronald Kriemann
//  Purpose   : class for a frontier (separates two cells) in 2d
//
// arch-tag: b051478b-c2c2-4c03-a5da-68ccbecc288d
//

#ifndef __T2DFRONTIER_HH
#define __T2DFRONTIER_HH

#include "frontier.hh"

//
// a 2D frontier ( with two endvertices and (possibly) one midvertex )
//
class T2DFrontier : public TFrontier<2> 
{
protected:
    // nodes defining the frontier
    TVertex<2>     **arrNodes;
    uint            noOfNodes;

    // type of frontier (INTERIOR, BOUNDARY, etc.)
    uint            frontierType;

public:
    // constructors
    T2DFrontier ( T2DFrontier *parent = NULL, uint ftype = 0 );
    T2DFrontier ( T2DFrontier *parent,
                  T2DVertex *vertex0, T2DVertex* vertex2, 
                  uint ftype = 0 );

    // destructor
    ~T2DFrontier ();

    // init memory for used arrays
    virtual void initNodePtr ( uint n );

    // make the midnode of the frontier
    virtual void buildMidNode ();

    // did we set the midnode
    virtual const bool hasMidNode () const;

    // return frontierType
    virtual const uint getFrontierType () const;

    // return number of nodes
    const uint getNoOfNodes () const;

    // return i'th node
    virtual TVertex<2> * getNode ( uint i ) const;

    // return i'th node with respect to the view of the given cell
    virtual TVertex<2> * getNode ( const TCell<2> *cell, uint i ) const;

    // build the child(ren)
    virtual void buildChild ();
};

#endif  // __T2DFRONTIER_HH
