//
//  Project     : libgraph
//  File        : T2DPlanarGraph.hh
//  Author      : Ronald Kriemann
//  Purpose     : class for a planar graph in 2D
//
// arch-tag: e017f7a9-2a08-4586-b984-126cd7c57c7d
//

#ifndef __T2DPLANARGRAPH_HH
#define __T2DPLANARGRAPH_HH

#include "dynarray.hh"

#include "basegraph.hh"

// forward decl
template <int N>   class TVector;
class TMatrixBlock;

///////////////////////////////////////////////////
//
// class for planar fvs and cyclic ordering
//
class T2DPlanarGraph : public TGraph
{
public:
	// constructor
	T2DPlanarGraph ();

	// constructor (builds graph of a matrixblock)
	T2DPlanarGraph ( TDynArray< TVector<2> * > & arrVectors,
                     TMatrixBlock & mat, 
                     TFloat xi,
                     uint criterion,
                     bool bZeroWeight = FALSE );
    
	T2DPlanarGraph ( TDynArray< TVector<2> * > & arrVectors,
                     TDynArray< TVector<2> * > & convVectors,
                     TMatrixBlock & mat, 
                     TFloat xi,
                     uint criterion,
                     bool bZeroWeight = FALSE );
    
	T2DPlanarGraph ( TDynArray< TVector<2> * > & arrVectors,
                     TMatrixBlock & diffMat,
                     TMatrixBlock & convMat,
                     TFloat xi,
                     uint criterion,
                     bool bZeroWeight = FALSE );

	// apply the one-flow-direction condition to the graph
	virtual void sortOFD ();

	// sort the edges of each nodes according to the weight
	virtual void sortEdgeWeight ();

	// sort the edges of each nodes according to the sum of
	// their and their succ. weights
	virtual void sortSuccEdgeWeight ();

	// create new graph
	virtual T2DPlanarGraph * create ();
};

#endif  // __T2DPLANARGRAPH_HH
