//
//  Project   : libnet
//  File      : T3DCellEdge.hh
//  Author    : Ronald Kriemann
//  Purpose   : class for an edge of a cell in 3D
//
// arch-tag: a88f05a1-d8b2-4a06-b6c4-9f86622fbe25
//

#ifndef __T3DCELLEDGE_HH
#define __T3DCELLEDGE_HH

#include "multiref.hh"
#include "memory.hh"
#include "TFlagable.hh"

#include "TBoundary.hh"
#include "frontier.hh"

// forward decl.
template <int N> class TVertex;

//
// an edge is defined by 2 nodes and can refine
// itself by building a midnode
//
class T3DCellEdge : public TFlagable<UINT8>, public TMultiReference, public TMemory
{
public:
    // type for add. data
    typedef void * data_t;

protected:
    // nodes defining the edge
    TVertex<3>        *arrNodes[3];

    // children of this edge
    T3DCellEdge       *arrChildren[2];
    uint              noOfChildren;

    // pointer to desc of element on boundary (NULL if interior)
    TBoundary         *pBoundary;

    // pointer to store add. data
    data_t             _data;
    
public:
    // constructor
    T3DCellEdge ( TVertex<3> *node0, TVertex<3> *node2,
                  TBoundary *boundary = INTERIOR );

    // destructor
    virtual ~T3DCellEdge ();

    // access nodes (absolute and with change of direction)
    virtual TVertex<3> * getNode ( uint i ) const;
    virtual TVertex<3> * getNode ( uint i, const bool bChangeDir ) const;

    // build midnode
    virtual void buildMidNode ();

    // return midnode info
    virtual const bool hasMidNode () const;

    // access children (absolute and with change of direction)
    virtual T3DCellEdge * getChild ( uint i ) const;
    virtual T3DCellEdge * getChild ( uint i, bool bChangeDir ) const;

    // build children
    virtual void buildChildren ();

    // return children info
    virtual const bool hasChildren () const;

    // access boundary
    virtual TBoundary * getBoundary () const;

    // access data
    virtual data_t data () const;
    virtual void setData ( data_t p );
    
    // output
    friend ostream & operator << ( ostream &os, const T3DCellEdge &edge );
};

#endif  // __T3DCELLEDGE_HH
