//
//  Project :  libvisual
//  File    :  T3DFlowView.hh
//  Purpose :  class for visualization of a 3D flow
//  Author  :  Ronald Kriemann
//
// arch-tag: be164325-2918-4591-b874-f301a51f93fb
//

#ifndef __T3DFLOWVIEW_HH
#define __T3DFlowVIEW_HH

#include "vector.hh"

#include "dynarray.hh"

#include "view.hh"

// forward decl
template <int N> class TConvEqn;

//
//
//
class T3DFlowView : public TView
{
protected:
    // array of points in domain, we use to show
    // flow
    TDynArray< TVector<3> >  arrPoints;

    // array with flow assoc to a point in upper array
    TDynArray< TVector<3> >  arrFlow;

    // stepwidths for drawing flow
    TFloat                   stepWidthX, stepWidthY, stepWidthZ;
    
public:
    ///////////////////////////////////////////
    //
    // constructors and destructors
    //

    T3DFlowView ( TConvEqn<3> * pEqn,
                  TFloat stepX, TFloat stepY, TFloat stepZ );

    ~T3DFlowView ();

    ///////////////////////////////////////////
    //
    // visualization methods
    //

    virtual void draw3D ( T3DPrinter & p );

protected:
    // encapsulate building of point/flow array
    void buildFlowArray ( TConvEqn<3> * pEqn );
};

#endif  // __T3DFLOWVIEW_HH
