//
//  Project   : libnet
//  File      : T3DFrontier.hh
//  Author    : Ronald Kriemann
//  Purpose   : class for a frontier (separates two cells) in 3d
//
// arch-tag: ce1b990d-f069-4226-a73f-b76b892d95bd
//

#ifndef __T3DFRONTIER_HH
#define __T3DFRONTIER_HH

#include "TFlagable.hh"

#include "frontier.hh"
#include "T3DCellEdge.hh"

//
// - in access to nodes it only can be guaranteed that
//   (edge[i].node[2] == edge[i+1].node[0] or
//    edge[i].node[0] == edge[i+1].node[2]) iff the given
//   edges have correct ordering (edge[i] and edge[i+1] have common node)
// - any access by direction (ccw or cw) is not possible
// - maximal number of edges is 8
//
class T3DFrontier : public TFrontier<3>, public TFlagable<UINT16>
{
public:
    // type for add. data
    typedef void * data_t;

protected:
    // in 3D, we use an array of edges, not nodes
    T3DCellEdge   **arrEdges;
    uint           noOfEdges;

    // pointer to store add. data
    data_t          _data;
    
public:
    // constructor
    T3DFrontier ( T3DFrontier *parent = NULL,
                  TBoundary *boundary = INTERIOR );

    // destructor
    virtual ~T3DFrontier ();

    // init memory for used arrays
    void initEdgePtr ( uint n );

    // set direction of each edge for continous access
    virtual const bool edgeDir ( uint i ) const;
    virtual void setEdgeDir ( uint i, const bool b );

    virtual void initEdgeDir ();

    // return number of nodes
    virtual const uint getNoOfNodes () const;

    // return i'th node
    virtual TVertex<3> * getNode ( uint i ) const;

    // return i'th node (no ccw in 3D)
    virtual TVertex<3> * getNode ( const TCell<3> *cell, uint i ) const;

    // return edges
    virtual T3DCellEdge * getEdge ( uint i ) const;

    // return the number of edges
    virtual const uint getNoOfEdges () const;

    // access data field
    data_t data () const;
    void setData ( data_t d );
};

#endif  // __T3DFRONTIER.HH
