//
//  Project  : libnet
//  File     : T3DTetrahedron.cc
//  Author   : Ronald Kriemann
//  Purpose  : class of a 3D tetrahedron (regular refinement)
//
// arch-tag: 43ee4c65-cb1c-4d7d-ad84-42f4748f88f1
//

#ifndef __T3DTETRAHEDRON_HH
#define __T3DTETRAHEDRON_HH

#include "cell.hh"
#include "T3DTriangle.hh"

// store info for collecting all subfrontiers
struct TFrontierInfo
{
    uint  type, firstNode, dir;
};

//
//
//
class T3DTetrahedron : public T3DCell
{
private:
    // idx of 4th node of tetrahedron (1-3 are on frontier 0)
    uint   _idxOfLastCorner;

    T3DTetrahedron ();
    T3DTetrahedron ( const T3DTetrahedron & );
    void operator = ( const T3DTetrahedron & );

public:
    // constructor
    T3DTetrahedron ( T3DCell *parent,
                     T3DTriangle *frontier0,
                     T3DTriangle *frontier1,
                     T3DTriangle *frontier2,
                     T3DTriangle *frontier3 );

    // divide all marked frontiers before real refinement
    virtual void prepareRefine ();

    // refine yourself (build only the children)
    virtual void refine ();

    // and the opposite : coarse
    virtual void coarsen ();

    // get i'th vertex of this cell
    virtual TVertex<3> * getNode ( uint i ) const;
  
    // return vertex opposite of i'th frontier
    virtual TVertex<3>* getOppositeNode ( uint i ) const;

    // calculate size of cell 
    virtual TFloat calcSize ();

private:
    // catch all corners
    uint idxOfLastCorner ();

    // catch all nodes from frontiers
    void collectNodes ( TVertex<3> * arrOrdNodes[], 
                        struct TFrontierInfo arrFrontierInfo[] );

    // collect all children of all frontiers
    void collectFrontierChildren ( TFrontier<3> * arrOrdFrontiers[],
                                   T3DCellEdge  * arrOrdEdges[],
                                   struct TFrontierInfo arrFrontierInfo[] );
   
    // decide which refine-strategy to use and refine inner octahedron
    void buildInnerChildren ( TVertex<3>   * arrOrdNodes[],
                              TFrontier<3> * arrOrdFrontiers[],
                              T3DCellEdge  * arrOrdEdges[] );
};

#endif  // __T3DTETRAHEDRON_HH
