//
//  Project   : libnet
//  File      : basisCdrEqn3D.hh
//  Author    : Sabine Gutsch / Thomas Probst
//  Purpose   : basis class for pdes
//
// arch-tag: a5ca7289-7c61-42e7-80d8-a08083d9ba3b
//

#ifndef __TBASISCDREQN_HH
#define __TBASISCDREQN_HH

#include "TMatrix.hh"

#include "TConvEqn.hh"

//
// basis class for a pde
//
template <int N>
class TBasisCdrEqn : public TConvEqn<N>
{
protected:
 
    TFloat epsilon;

 
    // flags whether these parts of the equations are active 
    bool diffFlag;
    bool convFlag;
    bool reacFlag;
 
    // shortcut: if the diffusion  operator is the (scaled) Laplacian,
    // discretisation is easier
    bool laplaceFlag;

    TBlockVector<TFloat> *convVector;
   
    
public:
    // constructors	
    TBasisCdrEqn ( TFloat aEpsilon = 1.0, 
                   TBlockVector<TFloat> *aConvVector = NULL ); 
   
    // destructor
    virtual ~TBasisCdrEqn () { }
   
    // get and set convection direction
    void setConvVector ( TBlockVector<TFloat> *newConvVector ) {
        convVector = newConvVector; 
    }
   
    TBlockVector<TFloat>* getConvVector ( ) { return (convVector); }
   
    // init convVector using the convection functions
    virtual void initConvVector (TDynArray< TVector <N> * > & coordVec );

    /*
    // compability mode
    virtual void initConvVector (uint size , TPArray< TVector <N>  > & coordVec ){
        // copy TPArray to TDynArray
        TDynArray < TVector <N> * > coords(size);
        for (uint i=0; i< size; i++)
            coords[i]=coordVec[i];
        initConvVector(coords);
    };
    */   

    // delete convection Vector
    void deleteConvVector ();
   
   
   
    // get and set epsilon
    TFloat getEpsilon( )const { return (epsilon); }
    void setEpsilon ( TFloat aEps ) { epsilon = aEps; }
   
    // get and set Laplacian Flag
    bool isLaplace()const {return laplaceFlag;}
    void isLaplace(const bool aflag){laplaceFlag=aflag;}
   

    // get and set Diffusion Flag
    bool Diffusion()const {return diffFlag;}
    void Diffusion(const bool aflag){diffFlag=aflag;}
   

    // get and set Convection Flag
    bool Convection()const {return convFlag;}
    void Convection(const bool aflag){convFlag=aflag;}
 
    // get and set Reaction Flag
    bool Reaction()const {return reacFlag;}
    void Reaction(const bool aflag){reacFlag=aflag;}
 
    // evaluate Diffusion matrix
    virtual  void evalDiff(TVector <N> *v, TMatrix<N>  *D)const = 0;
   
    // evaluate reaction 
    virtual TFloat evalReac(TVector <N> *v) const =0;
   
   // evaluate rhs
   virtual TFloat evalRhs(TVector <N> *v, uint i = 0 ) const =0;
}; 
// some types

#endif  // __TBASISCDREQN_HH
