//
//  Project	 : libnet
//  File     : TCycRedMatrixBlock.hh
//  Author	 : Sabine Gutsch / Ronald Kriemann / Thomas Probst   
//  Purpose	 : class for a (sparse) matrixblock 
//  Problems : 
//
// arch-tag: 3378ad2e-b44f-445e-8171-a7d325e9961e
//

#ifndef __TCYCREDMATRIXBLOCK_HH
#define __TCYCREDMATRIXBLOCK_HH

#include "defines.hh"
#include "memory.hh"
#include "dynarray.hh"

#include "blockvector.hh"
#include "sparsevector.hh"
#include "matrixblock.hh"

#include "TCoarsener.hh"

//
// class for a cyclic reducible matrixblock
//
class TCycRedMatrixBlock : public TMatrixBlock
{
protected:
    uint                NoOfReds;
    uint                NoOfBlacks;
    uint                 level;

    uint                MaxNoOfBlacks;
    uint                 MSIZE;
  
    TMatrixBlock       * pArr,*pAbr,*pArb;
    TCycRedMatrixBlock * pAbb;
  
    TPermutation       * pPerm, *pInvPerm;

    bool                 decomposed;

public:
    // constructors
    TCycRedMatrixBlock ();
	
    // init with another matrixblock
    TCycRedMatrixBlock ( const TMatrixBlock &m );
    
    // init with another CycRed matrixblock
    TCycRedMatrixBlock ( const TCycRedMatrixBlock &m, uint l );
    
    // destructor
    virtual ~TCycRedMatrixBlock ();
 
    // copy operator =
    TCycRedMatrixBlock & operator= ( const TCycRedMatrixBlock &m );
  

    uint decompose( TCoarsener& coarsener,
                    uint lev,
                    uint aMSIZE=12,
                    bool lumping=FALSE,
                    uint MaxNoOfBlacks=61,
                    uint MaxLevel=2);

    // return inner variables
    const uint getNoOfReds ()   const { return NoOfReds;   }
    const uint getNoOfBlacks () const { return NoOfBlacks; }
   
    const uint getMaxNoOfBlacks () const      { return MaxNoOfBlacks; }
    void  setMaxNoOfBlacks (const uint mnob ) { MaxNoOfBlacks=mnob;   }
   
    const uint getLevel ()       const { return level; }
    void  setLevel (const uint lev)    { level=lev;    }
   
    const uint getMSIZE ()       const { return MSIZE; }
    void  setMSIZE (const uint msize)  { MSIZE=msize;  }
   
    TPermutation * getPerm() const    { return pPerm;    }
    TPermutation * getInvPerm() const { return pInvPerm; }
  
    TMatrixBlock* getpArr()  const { return pArr;}
    TMatrixBlock* getpAbr()  const { return pAbr;}
    TMatrixBlock* getpArb()  const { return pArb;}
    TCycRedMatrixBlock* getpAbb()  { return pAbb;}
   
    const bool getDecompositionStatus() const { return decomposed;}

    // output
    friend ostream& operator << (ostream& os, TCycRedMatrixBlock& m);
};

#endif  // __TCYCREDMATRIXBLOCK_HH
