//
//  Project :  libvisual
//  File    :  TPOVRay.hh
//  Purpose :  class for a POVRay interface
//  Author  :  Ronald Kriemann
//
// arch-tag: 4fc3e2dc-18a6-40e2-9aed-1c5dd05c664d
//

#ifndef __TPOVRAY_HH
#define __TPOVRAY_HH

#include <stdio.h>

#include "memory.hh"
#include "dynarray.hh"
#include "list.hh"

#include "vector.hh"

#include "T3DPrinter.hh"

// struct for holding data for a polygon
struct TPolyData
{
    uint                      noOfNodes;
    TDynArray< TVector<3>* >  arrNodes;
    TFloat                    color[3];
};

//
// class for a POVRay v3.0 interface
//
class TPOVRay : public T3DPrinter
{
protected:
	// array of vertices for a polyhedron
	TDynArray< TVector<3>* >   arrNodes;

    // data for polygons
    struct TPolyData           polyData;

public:
	// constructor
	TPOVRay ( const char *filename );

	// destructor
	virtual ~TPOVRay ();

	// begin (end) printing
	virtual void begin ();
	virtual void end ();

	// begin (end) new section 
	virtual void beginSection ( const char *name );
	virtual void endSection ();

	//
	// misc. draw commands
	//

    virtual void drawText ( TVector<3> *orig, const char *text );
    
    virtual void drawLine ( TVector<3> *orig, TVector<3> *dest );
    
    virtual void drawCylinder ( TVector<3> *orig, TVector<3> *dest,
                                TFloat radius );

    virtual void drawCone ( TVector<3> *orig, TFloat origRad,
                            TVector<3> *dest, TFloat destRad );

    virtual void drawSphere ( TVector<3> * orig, TFloat radius );
    
protected:
    virtual void drawPolygon ();

    // write color/font def to file
    virtual void writeColor ();
    virtual void writeFont ();
};

#endif // __TPOVRAY_HH
