//
//  Project     : libgraph
//  File        : TPlanarCycle.hh
//  Author      : Ronald Kriemann
//  Purpose     : class for a cycle in a planar graph
//
// arch-tag: b6423752-c15f-4946-bfec-561a82eb3c27
//

#ifndef __TPLANARCYCLE_HH
#define __TPLANARCYCLE_HH

#include "TSList.hh"
#include "TFlagable.hh"

#include "basegraph.hh"

// forward decl.
class TCycleNode;

///////////////////////////////////////////////////
//
// class for a named cycle (has only ref to nodes)
//
class TPlanarCycle : public TGraph, public TFlagable<uchar>
{
protected:
    // node in Z we are associated with
    TCycleNode              * pZNode;
    
	// the name of the cycle
	uint                     _name;

	// value phi for nodes in this cycle
	uint                      _phi;

    // list of cycles we touch (only for updating Z)
    TSList< TPlanarCycle * >  listCycles;
    
public:
	// constructor
	TPlanarCycle ();

    // access node in Z
    TCycleNode * getZNode () const;
    void setZNode ( TCycleNode * pNode );
    
	// access the name
	const uint name () const;
	void setName ( uint n );

	// access phi
	const uint phi () const;
	void setPhi ( uint p );

	// access delflag
	const bool isDeleted () const;
	void setDeleted ( bool d );

    // flag to indicate if cycle is really new
    const bool isNew () const;
    void setNew ( bool b );

    // flags to indicate that cycle is succ/pred of some other cycle
    const bool isSucc () const;
    void setSucc ( bool b );

    const bool isPred () const;
    void setPred ( bool b );
    
    // access list of cycles
    void addCycle ( TPlanarCycle * pCycle );
    void clearCycleList ();
    TSList< TPlanarCycle * > & cycleList ();
};

#endif  // __GRAPHS_HH
