//
//  Project   : libnet
//  File      : TSOR.hh
//  Author    : Ronald Kriemann
//  Purpose   : class for a (S)SOR-based solver in second normal form
//
// arch-tag: ea313711-70e8-414a-a623-4b03ea91655e
//

#ifndef __TSOR_HH
#define __TSOR_HH

#include "secondNF.hh"

#define FORWARD_SOR   0
#define BACKWARD_SOR  1
#define SYMMETRIC_SOR 2

//
// class for SOR iteration
//
class TSOR : public TSecondNF
{
protected:
    // use symmetric SOR
    uint     method;
    
    // dampfactor for SOR
    TFloat  _omega;

    // variables for quasi-optimal choice of omega
    bool                    buildOmega, bInit;
    TBlockVector< TFloat >  vecX_old;
    TFloat                  norm_old;
    uint                    update_step;
    
public:
    ////////////////////////////////////
    //
	// constructor
    //
    TSOR ( TBlockMatrix         * pA       = NULL, 
           TBlockVector<TFloat> * pRHS     = NULL, 
           TBlockVector<TFloat> * pX       = NULL,  
           uint                   maxiter  = DEFAULT_MAX_ITERATION, 
           TFloat                 accur    = DEFAULT_ACCURACY, 
           uint                   verb     = 0,
           TFloat                 damp     = 1.0 );

    ////////////////////////////////////
    //
    // methods for solving
    //

    // implement W^-1 * v 
    TBlockVector<TFloat> WInvers ( const TBlockVector<TFloat> & v );

    //
    // access SOR-damp factor
    //
    
    const TFloat & omega () const;
    void setOmega ( const TFloat & o );

    //
    // set SOR method
    //

    void setMethod ( const uint m );

    //
    // update (here : choice of optimal omega)
    //
    virtual void update ( const TBlockVector< TFloat > & vecX );
}; 

//
// several spezializations of SOR
//
class TFSOR : public TSOR
{
public:
    TFSOR ( TBlockMatrix         * pA       = NULL, 
            TBlockVector<TFloat> * pRHS     = NULL, 
            TBlockVector<TFloat> * pX       = NULL,  
            uint                   maxiter  = DEFAULT_MAX_ITERATION, 
            TFloat                 accur    = DEFAULT_ACCURACY, 
            uint                   verb     = 0,
            TFloat                 damp     = 1.0 );
};

class TBSOR : public TSOR
{
public:
    TBSOR ( TBlockMatrix         * pA       = NULL, 
            TBlockVector<TFloat> * pRHS     = NULL, 
            TBlockVector<TFloat> * pX       = NULL,  
            uint                   maxiter  = DEFAULT_MAX_ITERATION, 
            TFloat                 accur    = DEFAULT_ACCURACY, 
            uint                   verb     = 0,
            TFloat                 damp     = 1.0 );
};

class TSSOR : public TSOR
{
public:
    TSSOR ( TBlockMatrix         * pA       = NULL, 
            TBlockVector<TFloat> * pRHS     = NULL, 
            TBlockVector<TFloat> * pX       = NULL,  
            uint                   maxiter  = DEFAULT_MAX_ITERATION, 
            TFloat                 accur    = DEFAULT_ACCURACY, 
            uint                   verb     = 0,
            TFloat                 damp     = 1.0 );
};

#endif  // __TSOR_HH
