//  Author    : Ronald Kriemann
//  Purpose   : class for a string
//  Date      : 17.01.1998
//
// arch-tag: 2a9c7cc0-a697-4abf-bac9-c75d088331c5
//

#ifndef __TSTRING_HH
#define __TSTRING_HH

extern "C" {
#include <limits.h>
#include <sys/types.h>
}

#include "defines.hh"
#include "dynarray.hh"

//
// class for a string (char-array) which acts like a string aka
// 0 means EOS (pay attention in += etc.)
//
class TString : public TDynArray<uchar>
{
public:
    // constructor
    TString ( size_t size = 0 );
    TString ( const TString &string );

    TString ( const char string[], const size_t size = ULONG_MAX );
    TString ( const uchar string[], const size_t size = ULONG_MAX );

    // destructor
    virtual ~TString ();

    // access string status
    virtual const size_t length () const;
    static  const size_t charSize ();
    virtual const bool isEmpty () const;

    // set string length
    virtual void setLength ( size_t n );

    // import (export) from (to) normal strings
    virtual void copyTo ( uchar buffer[], size_t size = ULONG_MAX ) const;
    virtual void copyTo ( char buffer[], size_t size = ULONG_MAX ) const;
    virtual void copyFrom ( const uchar buffer[], size_t size = ULONG_MAX );
    virtual void copyFrom ( const char buffer[], size_t size = ULONG_MAX );

    // access substrings [pos,pos+len) and [pos,length)
    virtual TString substr ( size_t pos1, size_t len ) const;
    virtual TString index ( size_t pos ) const;

    // locate substrings or char.
    virtual TString strchr ( char c ) const;
    virtual TString strchr ( uchar c ) const;
    virtual TString strstr ( const TString & needle ) const;

    // return substrings, seperated by delim
    // (remove all char before delim in string)
    virtual TString strtok ( char delim );
    virtual TString strtok ( uchar delim );
    
    //
    // various operators for conversion and manipulation
    //

    virtual TString & operator += ( const TString &string );
    virtual TString & operator += ( const uchar c );

    virtual TString & operator << ( const TString &string );

    virtual const bool operator == ( const TString &string ) const;
    virtual const bool operator != ( const TString &string ) const;

    virtual TString & operator >> ( char & c );
    virtual TString & operator >> ( uchar & c );

    virtual TString & operator >> ( int & n );
    virtual TString & operator >> ( long & n );

    virtual TString & operator >> ( float & f );
    virtual TString & operator >> ( double & f );

    virtual TString & operator <<= ( size_t n );
    virtual TString & operator >>= ( size_t n );

    //
    // misc. methods
    //

    // remove 0's at end
    virtual void shorten ();
};

//
// concatenation
//
TString operator + ( const TString & string1, const TString & string2 );

//
// output
//
ostream & operator << ( ostream &os, const TString &string );

#endif  // __TSTRING_HH
