//
//  Project  : libnet
//  File     : baensch.hh
//  Author   : Sabine Gutsch / Ronald Kriemann
//  Purpose  : class of cell which implements the Baensch-triangulation
//
// arch-tag: de17cbcf-5c23-46a9-ae63-4ad2c05f088e
//

#ifndef __BAENSCH_HH
#define __BAENSCH_HH

#include "T2DFrontier.hh"
#include "cell.hh"

//
// class for a Baensch-cell ( for 2D = 3 frontiers )
// the frontiers are numbered ccw
//
class T2DBaenschCell : public TCell<2> 
{
protected:
    // which frontier should be devided
    int  nMark;

public:
    // constructors
    T2DBaenschCell ();
    T2DBaenschCell ( TCell<2> *parent, 
                     T2DFrontier *frontier0, 
                     T2DFrontier *frontier1, 
                     T2DFrontier *frontier2 );
    
    // divide all marked frontiers before real refinement
    virtual void prepareRefine ();

    // refine yourself (build only the children)
    virtual void refine ();

    // and the opposite : coarse
    virtual void coarsen ();

    // return wich frontier is marked
    virtual const uint getMark () const;

    // set mark of frontier for refinement
    virtual void setMark ( uint i );

    // set mark automagically (at start -> the biggest frontier :) )
    virtual void setMark ();

    // output for easier debugging
    friend ostream& operator<< (ostream& os, const T2DBaenschCell& cell);
};

#endif  // __BAENSCH_HH
