//
//  Project     : libgraph
//  File        : basenode.hh
//  Author      : Ronald Kriemann
//  Purpose     : base class for all nodes
//  Date        : 04.08.1997
//
// arch-tag: d690c4cb-64ef-48ad-a5be-7a1b62c6a54b
//

#ifndef __BASENODE_HH
#define __BASENODE_HH

#include "memory.hh"
#include "multiref.hh"
#include "list.hh"

#include "edge.hh"

////////////////////////////////////////////////
//
// base class for a node
//
class TNode : public TMemory, public TMultiReference
{
protected:
	// name of our node
	uint  name;

	// lists of successors and predecessors
	TList<TEdge>  listOutEdges;
	TList<TEdge>  listInEdges;

public:
	// pointer for additional informations or capabilities
	// ( This pointer can be used by algorithms to store informations
	//   for every node )
	void * pInfo;

public:
	// constructor
	TNode ( uint n );
	TNode ( TNode& node );

	// destructor
	virtual ~TNode ();

	// access the name
	virtual const uint getName () const;
    virtual void setName ( uint n );
    
	// return the degrees
	virtual const uint getInDegree () const;
	virtual const uint getOutDegree () const;
	virtual const uint getDegree () const;

	// adds an in (out) edge
	virtual void addOutEdge ( TEdge& edge );
	virtual void addInEdge ( TEdge& edge );

	// remove an in (out) edge (we use the edge-name !!!)
	virtual void remOutEdge ( TEdge& edge );
	virtual void remInEdge ( TEdge& edge );

	// remove a succ (pred)
	virtual void remSucc ( TNode& node );
	virtual void remPred ( TNode& node );

	// check if pred (succ)
	virtual const bool isPred ( TNode &node ) const;
	virtual const bool isSucc ( TNode &node ) const;

	// return list with in (out) edges
	virtual TList<TEdge>& getOutEdges ();
	virtual TList<TEdge>& getInEdges ();
	virtual const TList<TEdge>& getOutEdges () const;
	virtual const TList<TEdge>& getInEdges () const;

	// isolate node (remove all incident edges)
	virtual void isolate ();

	// access the additional information
	virtual void* getInfo ();
	virtual const void* getInfo () const;
	virtual void setInfo ( void *info );

	// create a new nodes
	virtual TNode* clone ();
	virtual TNode* create ( uint n );

	// output 
	friend ostream& operator << ( ostream& os, const TNode& node );
};

#endif  // __BASENODE_HH
