//
//  Project   : libnet
//  File      : TBasisFE.hh
//  Author    : Sabine Gutsch / Thomas Probst
//  Purpose   : basis class for finite elements
//
// arch-tag: f1f72d7c-2150-442f-b0c7-4b8955d6c338
//

#ifndef __TBASISFE_HH
#define __TBASISFE_HH

#include "memory.hh"
#include "dynarray.hh"

#include "vector.hh"
#include "blockvector.hh"
#include "blockmatrix.hh"

#include "nethierarchy.hh"

#include "basisEqn.hh"

typedef TFloat (*PFloatFunc)( TFloat x );

TFloat simple ( TFloat x );
TFloat Samarskij ( TFloat x );

// (upwind) discretization types for convection
enum TDiscrType { Tabata, Box, Null };
//enum TUpwindType { Simple, Samarskij };

//
// basis class for a finite element 
//
class T2DBasisFE : public TMemory {
  
protected:
    // type of discretisation of the convective part (Box/Tabata)  
    TDiscrType DiscrType; 
    // function for upwinding (simple or Samarskij)
    PFloatFunc PHI;
public:
    // constructors	
    T2DBasisFE ( TDiscrType aDiscrType = Null ) 
    { 
        DiscrType = aDiscrType;
    }
  // destructor
  virtual ~T2DBasisFE () { }
  
  // set new DiscrType
  void setDiscrType(const TDiscrType newDiscrType  ) 
  { DiscrType = newDiscrType; }
  
  // get new DiscrType
  TDiscrType getDiscrType() { return DiscrType; }
  
  // set Phi for box method (Samarskij or simple)
  void setSamarskij() { PHI = Samarskij; }
  void setSimple() { PHI = simple; }
 
  // calculate global stiffness matrix for diffusion part
  // and write it into global matrix
  virtual void discrDiff ( T2DBasisEqn *eqn, T2DNet *net, 
                           TBlockMatrix *A, 
                           TBlockVector<TFloat> *rhs ) = 0; 
  
  // calculate (non-symmetric) convection part
  virtual void discrConv ( T2DBasisEqn *eqn, T2DNet *net, 
                           TMatrixBlock *N, 
                           TBlockVector<TFloat> *rhs ) = 0; 


  // calculate (lumped) massmatrix
  virtual void discrMass ( T2DBasisEqn *eqn, T2DNet *net, 
                           TVectorBlock<TFloat> *M) = 0; 


  // get coordinates of net
  virtual void getCoord ( T2DNet *net, TDynArray< TVector<2> * >& arrCoord ) = 0;
  
};

#endif  // __TBASISFE_HH
