//
//  Project   : libnet
//  File      : blocksolver.hh
//  Author    : Sabine Gutsch / Thomas Probst
//  Purpose   : block solver based on concentric cycles
//
// arch-tag: 68aebd59-7b37-48db-98bd-b7068fc05bc4
//

#ifndef __BLOCKSOLVER_HH
#define __BLOCKSOLVER_HH

#include "secondNF.hh"

//
// class for block lower Gauss-Seidel iteration (based on concentric 
// cycle decomposition)
//
class TBlockForwardGS : public TSecondNF {
public:
	// constructor
    TBlockForwardGS ( TBlockMatrix *paA = NULL, 
                    TBlockVector<TFloat> *paRHS = NULL, 
                    TBlockVector<TFloat> *paX = NULL,  
                    uint amaxiter = DEFAULT_MAX_ITERATION, 
                    TFloat aaccur = DEFAULT_ACCURACY, 
                    uint averb = 0, TFloat aDamp = 1.0 );

    // calculate WInvers
    TBlockVector<TFloat> WInvers ( const TBlockVector<TFloat> &v );
}; 

//
// class for block upper Gauss-Seidel iteration (based on concentric 
// cycle decomposition)
//
class TBlockBackwardGS : public TSecondNF {
public:
	// constructor
    TBlockBackwardGS ( TBlockMatrix *paA = NULL, 
                    TBlockVector<TFloat> *paRHS = NULL, 
                    TBlockVector<TFloat> *paX = NULL, 
                    uint amaxiter = DEFAULT_MAX_ITERATION, 
                    TFloat aaccur = DEFAULT_ACCURACY, 
                    uint averb = 0, TFloat aDamp = 1.0 );

    // calculate WInvers
    TBlockVector<TFloat> WInvers ( const TBlockVector<TFloat> &v );
}; 

//////////////////////////////////////////////////
//
// class for sym GS
//

class TBlockSymmetricGS : public TSecondNF {
public:
	// constructor
    TBlockSymmetricGS ( TBlockMatrix *paA = NULL, 
                        TBlockVector<TFloat> *paRHS = NULL, 
                        TBlockVector<TFloat> *paX = NULL, 
                        uint amaxiter = DEFAULT_MAX_ITERATION, 
                        TFloat aaccur = DEFAULT_ACCURACY, 
                        uint averb = 0, TFloat aDamp = 1.0 );

    // calculate WInvers
    TBlockVector<TFloat> WInvers ( const TBlockVector<TFloat> &v );
}; 

//
// class for  backward sym GS
//

class TBlockBackSymmetricGS : public TSecondNF {
public:
	// constructor
    TBlockBackSymmetricGS ( TBlockMatrix *paA = NULL, 
                        TBlockVector<TFloat> *paRHS = NULL, 
                        TBlockVector<TFloat> *paX = NULL, 
                        uint amaxiter = DEFAULT_MAX_ITERATION, 
                        TFloat aaccur = DEFAULT_ACCURACY, 
                        uint averb = 0, TFloat aDamp = 1.0 );

    // calculate WInvers
    TBlockVector<TFloat> WInvers ( const TBlockVector<TFloat> &v );
}; 


#endif  // __BLOCKSOLVER_HH
