//
//  Project   : libnet
//  File      : confDiffFE.hh
//  Author    : Sabine Gutsch / Thomas Probst 
//  Purpose   : class for piecewise linear finite elements for
//              the convection-diffusion equation
//
// arch-tag: 8c35eed1-6aa8-4ca1-81f2-ed7e9fc4e546
//

#ifndef __CONVDIFFFE_HH
#define __CONVDIFFFE_HH

//
// class for a piecewise linear element discretization for
// the convection-diffusion equation
//

#include "basisFE.hh"

class T2DConvDiffFE : public T2DBasisFE
{
protected:
	// gradients of basis functions on cell
    TVector<2> gradT[3];
    // determinants of transformation for cell
    TFloat     detJ;
    // dofs for u
    uint       dof_u[3];
	// coordinates of vertices of cell
    TVector<2> coord[3];   

    // boundary names are larger than BOUNDARY_NAME
	uint BOUNDARY_NAME; 

public:
	// constructors	
    T2DConvDiffFE ( TDiscrType aDiscrType = Tabata);
 
	// destructor
    ~T2DConvDiffFE ();

	// get coordinates of vertices of cell
    void getCoord ( TCell<2> *cell );

	// write coordinates of all (interior) vertices in net
	// into (global) vector coord
    void getCoord ( TNet<2>  *net, TDynArray< TVector<2> * >& arrCoord );

	// initialize grad, det, c and  dof_u for a particular cell
    void initDiscr ( TCell<2> *cell );

	// calculate global stiffness matrix and write it into global matrix
    void discrDiff ( T2DBasisEqn *eqn, TNet<2>  *net,  
                     TBlockMatrix *A, TBlockVector<TFloat> *rhs );

	// calculate global stiffness matrix of non-symmentric part
    virtual void discrConv ( T2DBasisEqn *eqn, TNet<2> *net, 
                     TMatrixBlock *N, TBlockVector<TFloat> *rhs );

    // calculate global  (lumped) mass matrix and write it into global vector
    void discrMass ( T2DBasisEqn *eqn, TNet<2> *net,  
                     TVectorBlock<TFloat> *M);

      // calculate element stiffness matrices
    virtual void discrDiff ( T2DBasisEqn *eqn, TBlockMatrix *A, 
                     TBlockVector<TFloat> *rhs );
    virtual void discrConv ( T2DBasisEqn *eqn, TMatrixBlock *N, 
                     TBlockVector<TFloat> *rhs );
    virtual void discrConv ( T2DBasisEqn *eqn, TMatrixBlock *N, 
                     TBlockVector<TFloat> *rhs,  
                     TVectorBlock<TFloat> &suppSize );
    void  discrMass ( T2DBasisEqn *eqn, TVectorBlock<TFloat> *M);
    void getSuppSize ( TVectorBlock<TFloat> &suppSize );
};

#endif  // __CONVDIFFFE_HH
