//
//  Project   : libmisc
//  File      : flowdef.hh
//  Author    : Ronald Kriemann
//  Purpose   : defintion of flow-examples
//
// arch-tag: 56178c4a-4690-49a0-819f-e05651e4e1f0
//

#ifndef __FLOWDEF_HH
#define __FLOWDEF_HH

#include "vector.hh"
#include "blockvector.hh"

#include "TFunction.hh"

#include "TCdrEqn.hh"
#include "TStokesBasisEqn.hh"

/////////////////////////////////////////////////////
//
// base functions
//

//
// conv-diff eqn
//
class TMyCdrEqn : public TCdrEqn<3>
{
public:
    TMyCdrEqn ( TFloat aEpsilon = 1.0, 
                TBlockVector<TFloat> *aConvVector = NULL );

    void evalDiff ( TVector <3> *, TMatrix<3> *D ) const;
            
    TFloat evalReac ( TVector <3> * ) const;
   
    TFloat evalRhs ( TVector <3> * v, uint i = 0 ) const;
};

//
// stokes eqn
//
class TMyStokesEqn : public TStokesBasisEqn<3>
{
public:
    TMyStokesEqn ( TFloat eps = 1.0, 
                   TBlockVector<TFloat> *convVec = NULL);

    TFloat evalReac ( TVector <3> * ) const;
    TFloat evalRhs ( TVector<3> *v, uint i = 0 ) const;
};

/////////////////////////////////////////////////////
//
// conv-diff eqn's
//

class TNoConvCdrEqn : public TMyCdrEqn
{
public:
    TNoConvCdrEqn ( TFloat eps = 1.0,
                    TBlockVector<TFloat> * convVec = NULL )
            : TMyCdrEqn ( eps, convVec )
    {}
    
    void evalConv ( TVector<3> *, TVector<3> * b ) const;
};

class TXLineCdrEqn : public TMyCdrEqn
{
public:
    TXLineCdrEqn ( TFloat eps = 1.0,
                   TBlockVector<TFloat> * convVec = NULL )
            : TMyCdrEqn ( eps, convVec )
    {}
    
    void evalConv ( TVector<3> *, TVector<3> * b ) const;
};

class TCurveCdrEqn : public TMyCdrEqn
{
public:
    TCurveCdrEqn ( TFloat eps = 1.0,
                   TBlockVector<TFloat> * convVec = NULL )
            : TMyCdrEqn ( eps, convVec )
    {}
    
    void evalConv ( TVector<3> * x, TVector<3> * b ) const;
};

class TCircleCdrEqn : public TMyCdrEqn
{
public:
    TCircleCdrEqn ( TFloat eps = 1.0,
                    TBlockVector<TFloat> * convVec = NULL )
            : TMyCdrEqn ( eps, convVec )
    {}
    
    void evalConv ( TVector<3> * x, TVector<3> * b ) const;
};

class TDiagCircleCdrEqn : public TMyCdrEqn
{
public:
    TDiagCircleCdrEqn ( TFloat eps = 1.0,
                        TBlockVector<TFloat> * convVec = NULL )
            : TMyCdrEqn ( eps, convVec )
    {}
    
    void evalConv ( TVector<3> * x, TVector<3> * b ) const;
};

class T4CirclesCdrEqn : public TMyCdrEqn
{
public:
    T4CirclesCdrEqn ( TFloat eps = 1.0,
                      TBlockVector<TFloat> * convVec = NULL )
            : TMyCdrEqn ( eps, convVec )
    {}
    
    void evalConv ( TVector<3> * x, TVector<3> * b ) const;
};

class TBallCdrEqn : public TMyCdrEqn
{
public:
    TBallCdrEqn ( TFloat eps = 1.0, 
                  TBlockVector<TFloat> * convVec = NULL )
            : TMyCdrEqn ( eps, convVec )
    {}
            
    void evalConv ( TVector<3> * x, TVector<3> * b ) const;
};

class TVortexCdrEqn : public TMyCdrEqn
{
public:
    TVortexCdrEqn ( TFloat eps = 1.0, 
                    TBlockVector<TFloat> * convVec = NULL )
            : TMyCdrEqn ( eps, convVec )
    {}
            
    void evalConv ( TVector<3> * x, TVector<3> * b ) const;
};

/////////////////////////////////////////////////////
//
// stokes eqn's
//

class TNoConvStokesEqn : public TMyStokesEqn
{
public:
    TNoConvStokesEqn ( TFloat eps = 1.0,
                       TBlockVector<TFloat> * convVec = NULL )
            : TMyStokesEqn ( eps, convVec )
    {}
    
    void evalConv ( TVector<3> *, TVector<3> * b ) const;
};

class TXLineStokesEqn : public TMyStokesEqn
{
public:
    TXLineStokesEqn ( TFloat eps = 1.0,
                      TBlockVector<TFloat> * convVec = NULL )
            : TMyStokesEqn ( eps, convVec )
    {}
    
    void evalConv ( TVector<3> *, TVector<3> * b ) const;
};

class TCurveStokesEqn : public TMyStokesEqn
{
public:
    TCurveStokesEqn ( TFloat eps = 1.0,
                      TBlockVector<TFloat> * convVec = NULL )
            : TMyStokesEqn ( eps, convVec )
    {}
    
    void evalConv ( TVector<3> * x, TVector<3> * b ) const;
};

class TCircleStokesEqn : public TMyStokesEqn
{
public:
    TCircleStokesEqn ( TFloat eps = 1.0,
                       TBlockVector<TFloat> * convVec = NULL )
            : TMyStokesEqn ( eps, convVec )
    {}
    
    void evalConv ( TVector<3> * x, TVector<3> * b ) const;
};

class TDiagCircleStokesEqn : public TMyStokesEqn
{
public:
    TDiagCircleStokesEqn ( TFloat eps = 1.0,
                           TBlockVector<TFloat> * convVec = NULL )
            : TMyStokesEqn ( eps, convVec )
    {}
    
    void evalConv ( TVector<3> * x, TVector<3> * b ) const;
};

class T4CirclesStokesEqn : public TMyStokesEqn
{
public:
    T4CirclesStokesEqn ( TFloat eps = 1.0,
                         TBlockVector<TFloat> * convVec = NULL )
            : TMyStokesEqn ( eps, convVec )
    {}
    
    void evalConv ( TVector<3> * x, TVector<3> * b ) const;
};

class TVortexStokesEqn : public TMyStokesEqn
{
public:
    TVortexStokesEqn ( TFloat eps = 1.0, 
                     TBlockVector<TFloat> * convVec = NULL )
            : TMyStokesEqn ( eps, convVec )
    {}
            
    void evalConv ( TVector<3> * x, TVector<3> * b ) const;
};

#endif  // __FLOWDEF_HH
