//
//  Project     : libgraph
//  File        : heuristicFVS.hh
//  Author      : Ronald Kriemann
//  Purpose     : class for a heuristic FVS algorithm
//
// arch-tag: d0f9fa31-a1d1-4cfb-acb0-dc0f7875058f
//

#ifndef __HEURISTICFVS_HH
#define __HEURISTICFVS_HH

#include "memory.hh" 
#include "list.hh"
#include "dynarray.hh"

#include "nodes.hh"
#include "basegraph.hh"

//
// class for heuristic FVS
//
class THeuristicFVS : public TMemory
{
protected:
    // flag to look globaly for a maximal-degree node
    static bool       bLookGlobally;
    static bool       bUseTarjan;
    
    // no. of not deleted nodes
    long              noOfNotDeleted;
    
    // maximum degree
    uint              maxDegree;
    TMarkableNode    *pMaxNode;
    
public:
    // constructor and destructor
    THeuristicFVS ();
    virtual ~THeuristicFVS ();

    // builds the FVS
    TList<TNode> * buildFVS ( TGraph & graph );
    TList<TNode> * buildFVS_SCC ( TGraph * pGraph );

    // set global-flag
    static void lookGlobally ( const bool & b );
    static void useTarjan ( const bool b );
    
protected:
    //
    // misc. methods for transforming the graph
    //
    virtual void transformLoop     ( TMarkableNode * pNode, TList<TNode> &listFVS );
    virtual void transformZeroPred ( TMarkableNode & node , TList<TNode> &listFVS );
    virtual void transformZeroSucc ( TMarkableNode & node , TList<TNode> &listFVS );
    virtual void transformOnePred  ( TMarkableNode & node , TList<TNode> &listFVS );
    virtual void transformOneSucc  ( TMarkableNode & node , TList<TNode> &listFVS );

    // add a node to fvs
    virtual void addFVS ( TMarkableNode & node, TList<TNode> &listFVS );
};

#endif  // __HEURISTICFVS_HH
