//
//	Project		: libmisc
//	File		: matlab.hh
//	Author		: Ronald Kriemann / Sabine Gutsch
//	Purpose		: functions to write matrix and vector in matlab format
//	Date		: 23.07.1997
//
// arch-tag: 1b168ade-4f25-4481-9d8a-aed5b9bc11e5
//

#ifndef __MATLAB_HH
#define __MATLAB_HH

#ifdef MATLAB
#include <mat.h>
#include <matrix.h>

#include "defines.hh"

#include "blockmatrix.hh"
#include "blockvector.hh"

int
matWriteMatrixBlock( char *filename, const TMatrixBlock& block, uint noOfColumns,
					 char *name );

int
matWriteBlockMatrix( char *filename, const TBlockMatrix& mat, char *name );

int
matWriteCompleteMatrix ( char *filename, const TBlockMatrix &mat, char *name );


#if DIMENSION == 3 
int
matWriteBlockProlongation( char *filename, const TProlongation<3> &mat, char *name );
#endif

#if DIMENSION == 2 
int
matWriteBlockProlongation( char *filename, const TProlongation &mat, char *name );
#endif

template <class T>
int
matWriteVectorBlock( char *filename, const TVectorBlock<T> & block, char *name );

template <class T>
int
matWriteBlockVector( char *filename, const TBlockVector<T> & vec, char *name );

int
matWriteCompleteVector( char *filename, const TBlockVector<TFloat> & vec, char *name );

int
matReadVectorBlock( char *filename,  TVectorBlock<TFloat> & block, char *name );

int
matReadBlockVector( char *filename,  TBlockVector<TFloat> & vec, char *name );

#endif  // MATLAB

#endif  // __MATLAB_HH
