//
//  Project   : libmisc
//  File      : multiref.hh
//  Author    : Kai Helms / Ronald Kriemann
//  Purpose   : class with multi-reference-ability
//
// arch-tag: befd705b-60c9-4a25-a09a-b748faa26a45
//

#ifndef __MULTIREF_HH
#define __MULTIREF_HH

#include "defines.hh"

//
// multiple references for c++-objects (intelligent pointers)
//
// this class is baseclass for objects, which are referenced by
// more than one objects and must only be deleted if no reference
// has left
//
class TMultiReference
{
public:
    // counter for the number of references
    int references;

public:
    // constructor
    TMultiReference ()
    {
        references = 1;
    }
    
    // destructor
    virtual ~TMultiReference ()
    { 
        if (references > 0) 
            cerr << "TMultiReference::~MultiReference () "
                 << ": deleting an object with " << references
                 << " references on it" << endl; 
    }

    // add another reference to the object
    void addReference()
    {
        ++references;
    }

    // remove a reference
    bool delReference()
    {
        return (!--references);
    }

    // abbr. for the above
    void addRef ()
    {
        ++references;
    }

    bool delRef ()
    {
        return (!--references);
    }

    // do not copy number of referencess with an object
    TMultiReference & operator = ( const TMultiReference & )
    {
        return *this;
    }

    //
    // decrement reference counter and delete object if no ref left
    //
    bool free ()
    {
        if ((--references) > 0)
            return FALSE;

        delete this;

        return TRUE;
    }
};

#endif  // __MULTIREF_HH
