//
//  Project   : libnet
//  File      : nonlinearSolver.hh
//  Author    : Sabine Gutsch
//  Purpose   : nonlinear solver class (basis class)
//
// arch-tag: 0aa3d873-ecf6-49aa-8f84-d154f78da1e7
//

#ifndef __NONLINEARSOLVER_HH
#define __NONLINEARSOLVER_HH

#include "nethierarchy.hh"

#include "basisFE.hh"
#include "stokesHierarchy.hh"

#include "blockinfo.hh"

#include "drawapp.hh"

#include "multigrid.hh"

class TNonLinearSolver : public TMemory {

protected:
    char                 *name;          // name of solver
    TBasisEqn<2>         *eqn;           // Navier-Stokes equations
    TBasisFE<2>          *fe;            //  finite element (to update N)
	T2DNetHierarchy      *netHier;    //  net hierarchy (to update N)
    T2DStokesHierarchy   *stokesHier;  // net hierarchy
    
    TVectorBlock<uint> **vecPerm;      // permutation vector
    TVectorBlock<uint> **vecInvPerm;   // inverse permutation vector
    TVectorBlock<uint> **cyclicBlocks; // cycle information
    uint                  noOfVectors;    // number of permutation vectors
    TDynArray< TVector<2> * > *arrVectors;    // coordinates on finest grid
    uint                  ordering;

    TFloat                accuracy;
    uint                 maxit;
    

public:
	// constructor
    TNonLinearSolver ( TBasisEqn<2> *aEqn, TBasisFE<2> *aFE, 
                       T2DNetHierarchy *aNetHier, 
                       T2DStokesHierarchy *aStokesHier ); 

	// destructor
    virtual ~TNonLinearSolver ( );

    // get and set name
    char* getName ();
    void  setName ( const char *n );

	// set and get maximal number of iterations
    void setMaxit( uint aMaxit ) { maxit = aMaxit; }
    uint getMaxit() { return (maxit); }

	// set and get ordering
    void setOrdering( uint aOrder ) { ordering = aOrder; }
    uint getOrdering() { return (ordering); }

	// update hierarchy (first re-establish original ordering, then
    // update (convective part of) hierarchy, determine new ordering 
	// and reorder)
    void updateHierarchy( TMultigrid *mgSolver );

    virtual TFloat step ( TMultigrid *mgSolver ) = 0;

    TBlockVector<TFloat> solve ( TMultigrid *mgSolver ); 
};



#endif  // __NONLINEARSOLVER_HH
