//
//  Project     : libgraph
//  File        : planarBlock.hh
//  Author      : Ronald Kriemann
//  Purpose     : class building cycles (blocks) in a planar graph
//
// arch-tag: 05f02a70-a434-4fe6-8ae8-19e3c28c25ba
//

#ifndef __PLANARBLOCK_HH
#define __PLANARBLOCK_HH

#include "memory.hh"

#include "T2DPlanarGraph.hh"
#include "T2DPlanarNode.hh"
#include "TPlanarCycle.hh"

#include "blockinfo.hh"
#include "planarFVS.hh"

//
// a simple version of the algorithms in TPlanarFVS
// seperated for maintenance reasons
//
class TPlanarBlock : public TMemory
{
public:
    ////////////////////////////////////////////////
    //
    // constructor and destructor
    //
    
    TPlanarBlock ();

    ~TPlanarBlock ();

    ////////////////////////////////////////////////
    //
    // methods for planar block algorithm
    //
    
    // reduce graph by combining all nodes with weak edges
    void reduce ( T2DPlanarGraph &graph );

    // reduce single node
    void reduceNode ( T2DPlanarGraph &graph, 
                      T2DPlanarNode &node );

    // melt nodes connected by a weak edge (node has not one in- or one out-edge)
    void reduceWeakNode ( T2DPlanarGraph &graph, 
                          T2DPlanarNode &node );

    // remove all weak edges if an opposite strong edge exists
    void removeNonRealWeak ( T2DPlanarNode &node );

    // sort edges according to ofd-cond
    void sortWeakOFD ( T2DPlanarNode &node );

    // melt nodes on right (left) side
    void meltNodesRight ( T2DPlanarNode &node,
                          T2DPlanarNode &succ );
    void meltNodesLeft ( T2DPlanarNode &node,
                         T2DPlanarNode &succ );

    // check if node has at least one strong in (out) edge
    bool hasStrongInEdge ( T2DPlanarNode &node );
    bool hasStrongOutEdge ( T2DPlanarNode &node );

    // check if node has a weak edge
    bool hasWeakEdge ( T2DPlanarNode &node );

    // remove all weak edges
    void removeWeak ( T2DPlanarNode &node );

    ////////////////////////////////////////////////
    //
    // methods for building minimal/maximal cycles
    //
    
    // build cycles (blocks)
    TList<TGraph> * buildCycles ( TGraph * pGraph, uint type,
                                  TBlockInfo * pBlockInfo );

    // create single cycle
    TPlanarCycle * getCycle ( TGraph * pGraph, uint type, long cycleName );

    // look for a suitable startnode of a cycle
    TPlanarNode * getStartNode ( TGraph * pGraph, uint type );

    // look for suitable succ of given node
    TPlanarNode * getSucc ( TNode * pNode, uint type );
    
    ////////////////////////////////////////////////
    //
    // local version of numbering algorithm
    //

    void localNumber ( TGraph * pGraph, uint type,
                       TBlockInfo * pBlockInfo );
    
    void localNumberCycle ( TPlanarCycle * pCycle, uint type,
                            TBlockInfo * pBlockInfo );
    
    void localSetF ( TPlanarNode * pNode, uint type,
                     TBlockInfo * pBlockInfo, TPlanarCycle * pCycle );
    
    void localSetL ( TPlanarNode * pNode, uint type,
                     TBlockInfo * pBlockInfo, TPlanarCycle * pCycle );
};

#endif  // __PLANARBLOCK_HH
