//
//  Project   : libnet
//  File      : regular.hh
//  Author    : Sabine Gutsch / Ronald Kriemann
//  Purpose   : class of cell which implements a regular triangulation
//
// arch-tag: ff8d6af8-435c-4562-95e9-37edda931341
//

#ifndef __REGULAR_HH
#define __REGULAR_HH

#include "T2DFrontier.hh"
#include "cell.hh"

//
// class for a regular cell ( for 2D = 3 frontiers )
// (divided into 4 triangles by connecting the midpoints in refinement)
// the frontiers are numbered ccw
//
class T2DRegularCell : public TCell<2> 
{
public:
    // constructors
    T2DRegularCell ();
    T2DRegularCell ( TCell<2> *parent, 
                     T2DFrontier *frontier0, 
                     T2DFrontier *frontier1, 
                     T2DFrontier *frontier2 );

    // divide all marked frontiers before real refinement
    virtual void prepareRefine ();

    // refine yourself (build only the children)
    virtual void refine ();

    // and the opposite : coarse
    virtual void coarsen ();

    // output for easier debugging
    friend ostream& operator << (ostream& os, const T2DRegularCell& cell);
};

#endif  // __REGULAR_HH
