//
//  Project  : libnet
//  File     : sort.hh
//  Author   : Sabine Gutsch / Thomas Probst
//  Purpose  : class for sorting
//
// arch-tag: cbfdc4b3-d68c-4e8a-8432-47a574a509b5
//

#ifndef __SORT_HH
#define __SORT_HH


#include <stdlib.h>

#include "convDiffFE.hh"
#include "memory.hh"

#include "vector.hh"

#include "blockvector.hh"
#include "blockmatrix.hh"

enum TSortType { lexicographical, concCycle };

class TSort {

protected:
    T2DNet *pNet;

    TBlockMatrix         *pA;   // matrix 
    TBlockVector<TFloat> *pRHS; // right hand side
    TBlockVector<TFloat> *pX;   //  Solution;
    uint dim;                  // length of permvector

    TVectorBlock<uint> *perm;
    TVectorBlock<uint> *invperm;

public:
 
 //constructor
 
 TSort ( TBlockMatrix *apA = NULL, TBlockVector<TFloat> *apRHS=NULL, 
         TBlockVector<TFloat> *apX = NULL, T2DNet *Net = NULL){
 
     pA   = apA; 
     pRHS = apRHS;
     pX   = apX; 
     pNet = Net; 
     dim  = (*pRHS)[0].getSize();
     
     perm = new TVectorBlock<uint>( 0, dim);
     invperm = new TVectorBlock<uint>( 0, dim);     
 }

 // destructor
 ~TSort() { delete(perm); delete(invperm); }

 // get perm or invperm
 TVectorBlock<uint> *getPermutationvector() { return (perm); }
 TVectorBlock<uint> *getInvPermutationvector() { return (invperm); }

 void sort( TSortType aSortType );
 void redBlack();
 void order( TBlockMatrix *pM );
 void order( TMatrixBlock *pM );
 void order( TBlockVector<TFloat> *pV);

 void restore( );
 void restore( TBlockMatrix *pM );
 void restore( TBlockVector<TFloat> *pV);

};

#endif // __SORT_HH
