//
//  Project :  libvisual
//  File    :  vectorfield.hh
//  Purpose :  class for visualization of a vectorfield
//  Author  :  Ronald Kriemann / Sabine Gutsch
//
// arch-tag: e40c01fd-8805-48b1-b962-2be309d71b75
//

#ifndef __VECTORFIELD_HH
#define __VECTORFIELD_HH

#include "blockvector.hh"
#include "vector.hh"

#include "net.hh"

#include "dynarray.hh"

#include "view.hh"

//
// this class gets an array of vertices (the origins of the vectors)
// and a blockvector (the directions of the vectors)
// we use only the first two blocks (u and v fields)
//
class TVectorField : public TView
{
protected:
	// array with all vertices
	TDynArray< TVector<2> * >  *parrVertices;

	// vector with the vectorcomponents
	TBlockVector<TFloat>  *pvecSolution;

	// min (max) distance between two nodes in a cell
	TFloat   minDistNodes, maxDistNodes;

	// max. value of int the solution
	TFloat   maxSolution;

	// scale the values or equal size
	bool     bScale;

public:
	// constructor
	TVectorField ( TDynArray< TVector<2> * >& vertices,
				   TBlockVector<TFloat>& solution,
				   const T2DNet& net,
				   bool scale );

	// draw the vectorfield
	void draw ( TPrinter& p );
};

#endif  // __VECTORFIELD_HH
