//
//  Project :  libvisual
//  File    :  viewport.hh
//  Purpose :  class for a world <-> viewport transformation
//  Author  :  Ronald Kriemann
//
// arch-tag: 902c29ec-c825-484e-ae81-98b48c43ee33
//

#ifndef __VIEWPORT_HH
#define __VIEWPORT_HH

#include "defines.hh"

//
// this class transforms world-coord into viewport-coord
// and back
//
class TViewport 
{
protected:
	// scalingfactors
	TFloat  scaleX;
	TFloat  scaleY;

	// coord. of the window and the viewport
	TFloat  viewXmin    , viewYmin;
	TFloat  viewXlength , viewYlength;
	TFloat  worldXmin   , worldYmin;
	TFloat  worldXlength, worldYlength;

public:
	// constructor
	TViewport ();
	TViewport ( TFloat wXmin   , TFloat wYmin, 
				TFloat wXlength, TFloat wYlength,
				TFloat vXmin   , TFloat vYmin,
				TFloat vXlength, TFloat vYlength );

	// return the scalings
	const TFloat getScaleX () const;
	const TFloat getScaleY () const;

	// transformation world -> viewport
	const TFloat viewX( TFloat x ) const;
	const TFloat viewY( TFloat y ) const;

	// transformation viewport -> world
	const TFloat worldX( TFloat x ) const;
	const TFloat worldY( TFloat y ) const;

	// set the worldcoord.
	void setWorld( TFloat wXmin   , TFloat wYmin,
				   TFloat wXlength, TFloat wYlength );

	// get the worldborders
	void getWorld( TFloat& minX, TFloat& minY, TFloat& maxX, TFloat maxY ) const;

	// set the viewportcoord.
	void setViewport( TFloat vXmin   , TFloat vYmin,
					  TFloat vXlength, TFloat vYlength );
	
	// get the viewportborders
	void getViewport( TFloat& minX, TFloat& minY, TFloat& maxX, TFloat& maxY ) const;
	
	// check if the point is in the window
	bool isInWin( TFloat x, TFloat y ) const;
	
	// check if the point is in the viewport
	bool isInView( TFloat x, TFloat y ) const;

	// copy operator
	const TViewport& operator=( const TViewport& viewPort );
};


#endif // __VIEWPORT_HH
