//
//  Project : libnet
//  File    : TCycRedmatrixBlock.cc
//  Author  : Sabine Gutsch / Ronald Kriemann / Thomas Probst
//  Purpose : 
//
// arch-tag: 54d148a4-e636-4173-8b2c-c2e79b8be876
//

#include "matlab.hh"
#include "sortable_matrixrow.hh"

#include "fastGaussSeidel.hh"

#include "blockinfo.hh"

#include "TCycRedMatrixBlock.hh"

//////////////////////////////////////////////////////
//
//  TMatrixBlock
//

//
// constructors
//
TCycRedMatrixBlock::TCycRedMatrixBlock ()
        : TMatrixBlock ()
{
    level=0; 
    MSIZE=12;
    MaxNoOfBlacks=61;
  
    decomposed=FALSE;
    NoOfReds=0;
    NoOfBlacks=0;
 
    pPerm=NULL;
    pInvPerm=NULL;
 
    pArr=NULL;
    pAbr=NULL;
    pArb=NULL;
    pAbb=NULL;
  
}

//
//
//
TCycRedMatrixBlock::TCycRedMatrixBlock ( const TMatrixBlock &m )
        : TMatrixBlock(m)
{
    level=0;
    MSIZE=12;
    MaxNoOfBlacks=61;
  
    // cout<<" m: "<<m<<endl;
    //cout<<" this: "<<*this<<endl;
    decomposed=FALSE;
   
    NoOfReds=0;
    NoOfBlacks=0;
    pArr=NULL;
    pAbr=NULL;
    pArb=NULL;
    pAbb=NULL;
  
    pPerm=NULL;
    pInvPerm=NULL;
}

//
//
//
TCycRedMatrixBlock::TCycRedMatrixBlock ( const TCycRedMatrixBlock &m, uint l )
        : TMatrixBlock(m)
{
    level=l;
    MaxNoOfBlacks=m.getMaxNoOfBlacks(); 
    MSIZE=m.getMSIZE();

 
    decomposed=FALSE;
    NoOfReds=0;
    NoOfBlacks=0;
  
    pPerm=NULL;
    pInvPerm=NULL;

    pArr=NULL;
    pAbr=NULL;
    pArb=NULL;
    pAbb=NULL;
}

//
// destructor
//
TCycRedMatrixBlock::~TCycRedMatrixBlock () 
{ 
    cout << "TCycRedMatrixBlock::~TCycRedMatrixBlock () : deleting matrixblock level "
         << level << endl;
   
    if (decomposed)
    {
        NoOfReds=0;
        NoOfBlacks=0;

        if (pPerm)
            delete pPerm;
        
        if (pInvPerm)
            delete pInvPerm;
 
        if (pArr)
            pArr->free();
        
        if (pAbr)
            pAbr->free();
        
        if (pArb)
            pArb->free();
        
        if (pAbb)
            pAbb->free();
    }// if
}

//
// copy operator =
//

TCycRedMatrixBlock& 
TCycRedMatrixBlock::operator= ( const TCycRedMatrixBlock &m ) 
{
    NoOfReds=m.getNoOfReds(); 
    NoOfBlacks=m.getNoOfBlacks(); 
    level= m.getLevel();
    MaxNoOfBlacks=m.getMaxNoOfBlacks(); 
    MSIZE=m.getMSIZE();

    // copy data
    uint blockSize = m.getNoOfRows(); 
    setSize( blockSize );
    for (uint i = 0; i < blockSize; i++)
        (*this)[i].set( m[i] );
    pDecomp = NULL; 
    decompType = TDecompType( NONE );

    pPerm=NULL;
    pInvPerm=NULL;

    pArr=NULL;
    pAbr=NULL;
    pArb=NULL;
    pAbb=NULL;

    decomposed=FALSE;
 
    return (*this);
}

uint 
TCycRedMatrixBlock::decompose ( TCoarsener& coarsener,
                                uint  lev,
                                uint aMSIZE,
                                bool lumping,
                                uint aMaxNoOfBlacks,
                                uint aMaxLevel)
{ 
    level=lev;
    //cout<<lev<<" "<<aMSIZE<<" "<<aMaxNoOfBlacks<<" "<<aMaxLevel<<endl;
    cout << "TCycRedMatrixBlock::decompose (...) : level = " << level << endl;
    uint maxlevel;
    uint dim = getNoOfRows();
   
    if ((dim>aMaxNoOfBlacks) && ( lev<aMaxLevel)) {
        // start decomposition
        decomposed=TRUE;
      
        // prepare verPerm
        if (!pInvPerm)
            pInvPerm = new TPermutation( dim );
  
        pPerm = coarsener.coarse( this, NoOfReds );
   
        *pInvPerm = pPerm->invert();
      
        // copy into matrixblock
        NoOfBlacks=dim-NoOfReds;
      
        // permute
        this->permute( pInvPerm, pInvPerm );
    
        uint i,j;

        cout << "TCycRedMatrixBlock::decompose (...) : NoOfBlacks = "
             << NoOfBlacks << endl;
        cout << "TCycRedMatrixBlock::decompose (...) : NoOfReds   = "
             << NoOfReds << endl;
 
        // copy block data Abb and Abr
        pAbb = new TCycRedMatrixBlock(); 
        pAbr = new TMatrixBlock();       
        pArr = new TMatrixBlock();       
        pArb = new TMatrixBlock();       

        uint largeSize,BlackSize,RedSize;
        pAbb->setNoOfRows(NoOfBlacks);

        pAbr->setNoOfRows(NoOfBlacks);
        pArb->setNoOfRows(NoOfReds);
        pArr->setNoOfRows(NoOfReds);
   
      
        for ( i = 0; i <NoOfBlacks; i++){
  
            largeSize=(*this)[i].getNoOfEntries();
            BlackSize=0;
            RedSize=0;
         
            TFloat Bvalues[largeSize],Rvalues[largeSize];
            uint Bindex[largeSize],Rindex[largeSize];
            uint ind;
     
            for (j=0;j<largeSize;j++){
                ind=(*this)[i].getIndex(j);
                if (ind<NoOfBlacks){
                    // copy Black value
                    Bvalues[BlackSize]=(*this)[i].getValue(j);
                    Bindex[BlackSize]=ind;
                    BlackSize++;
                }
                else{ 
                    // copy red Values
                    Rvalues[RedSize]=(*this)[i].getValue(j);
                    Rindex[RedSize]=ind;
                    RedSize++;
                }
            }
     
            (*pAbb)[i].set(BlackSize,Bindex,Bvalues);
            (*pAbr)[i].set(RedSize,Rindex,Rvalues);
        }
        // adjust level   for Abb
        pAbb->setLevel(this->level+1);


        // copy Arb and Arr
        for ( i = 0; i <NoOfReds; i++){
  
            largeSize=(*this)[i+NoOfBlacks].getNoOfEntries();
            BlackSize=0;
            RedSize=0;
      
            TFloat Bvalues[largeSize],Rvalues[largeSize];
            uint Bindex[largeSize],Rindex[largeSize];
            uint ind;
     
            for (j=0;j<largeSize;j++){
                ind=(*this)[i+NoOfBlacks].getIndex(j);
                if (ind<NoOfBlacks){
                    // copy Black value
                    Bvalues[BlackSize]=(*this)[i+NoOfBlacks].getValue(j);
                    Bindex[BlackSize]=ind;
                    BlackSize++;
                }
                else{ 
                    // copy red Values
                    Rvalues[RedSize]=(*this)[i+NoOfBlacks].getValue(j);
                    Rindex[RedSize]=ind-NoOfBlacks;
                    RedSize++;
                }
            }
     
            (*pArb)[i].set(BlackSize,Bindex,Bvalues);
            (*pArr)[i].set(RedSize,Rindex,Rvalues);
        }
 

        // calculate Schur-Complement

 
        // help matrix t
        TMatrixBlock *T=new TMatrixBlock(*pAbr);        
                                                      
        // copy Abr                                     
        TMatrixBlock *pAbrCopy=new TMatrixBlock(*pAbr); 
 
        uint k,l,r,n=0,rni;

        for (i=0;i<NoOfBlacks;i++){
            // row of T = red neighbours of i
            k=(*T)[i].getNoOfEntries();
            //  cout<<i<<endl;
            (*T)[i].writeableStatus(TRUE);
            // red neighbours of each red neighbour
            for(l=0; l<k;l++){  
                r=(*T)[i].getIndex(l);  // index r of red neighbour 
                n=(*this)[r].getNoOfEntries();
                for(j=0;j<n;j++){   
                    // get index rni  of neighbours of r
                    rni=(*this)[r].getIndex(j);
                    //  cout<<"   rni "<<rni<<" from r "<<r<<" of "<<i<<" ";
                    if (rni>=NoOfBlacks)  // is rni red?
                        (*T)[i][rni]=0.0;                  
                } // for j (each neighbour of r)
            } // for l  (each (red) neighbour of i)
            (*T)[i].writeableStatus(FALSE);
        } // for i 

     

        // enlarge pAbb
        uint u,v,w;
        for (u=0;u<NoOfBlacks;u++){
            k=(*this)[u].getNoOfEntries();
            (*pAbb)[u].writeableStatus(TRUE);
            //  cout<<u<<endl;
            // red neighbours of u
            for(l=1; l<k;l++){  
                v=  (*this)[u].getIndex(l);  
                if (v >=NoOfBlacks) // v red?
                    n=(*this)[v].getNoOfEntries();
        
                for(j=0;j<n;j++){   // j=1 is diagonal element
                    // get index bni  of neighbours of r
                    w=(*this)[v].getIndex(j);
                    //  cout<<"   bni "<<bni<<" from r "<<r<<" of "<<i<<" ";
                    if (w<NoOfBlacks) // w  black?
                        if( (*pAbb)[u][w] >0) // this  creates  the matrix entry (*pAbb)[u][w]
                        {}
                } // for j (each neighbour of v)
            } // for l  (each  neighbour of u)
            (*pAbb)[u].writeableStatus(FALSE);
        } // for u 

   
        for (u=0;u<NoOfBlacks;u++){
            k=(*this)[u].getNoOfEntries();
            //cout<<u<<endl;
            // red neighbours of u
            for(l=1; l<k;l++){  
                v=  (*this)[u].getIndex(l);  
                if (v >=NoOfBlacks){ // v red?
                    n=(*this)[v].getNoOfEntries();
                    for(j=0;j<n;j++){  
                        // get indexw of neighbours of v
                        w=(*this)[v].getIndex(j);
                        // cout<<"  w "<<w<<" from v "<<v<<" of "<<u<<endl;;
                        if (w<NoOfBlacks) // w  black?
                            (*pAbb)[u][w] = (*pAbb)[u][w]-(*this)[u][v]*(*this)[v][w]/(*this)[v][v];
                        else // w red 
                        {
                            (*T)[u][w] = (*T)[u][w]+(*this)[u][v]*(*this)[v][w]/(*this)[v][v];
                        }
                    } // for j (each neighbour of v)
                }
            } // for l  (each  neighbour of u)
        } // for u
      

        // *pAbr-=*T; 
        for (u=0;u<NoOfBlacks;u++){
            k=(*T)[u].getNoOfEntries();
            (*pAbr)[u].writeableStatus(TRUE);
            //cout<<u<<endl;
            // red neighbours of u
            for(l=0; l<k;l++){  
                v=  (*T)[u].getIndex(l);  
                if (v >=NoOfBlacks) // v red?
                    (*pAbr)[u][v]=(*pAbr)[u][v]-(*T)[u][v];
            }
            (*pAbr)[u].writeableStatus(FALSE);
        }
  

     
 
        // second step:

        // vector d ( row sum of Arr)
        TVectorBlock<TFloat> *d = new TVectorBlock<TFloat>(0,NoOfReds);
        TFloat sum;
        for (u=0;u<NoOfReds;u++){
            v=u+NoOfBlacks; // red indices
            k=(*this)[v].getNoOfEntries();
            sum=0.0;
            for(l=0; l<k;l++){  
                if ((*this)[v].getIndex(l)>=NoOfBlacks) // red ?
                    sum+=(*this)[v].getValue(l);
            }
            (*d)[u]=sum;
        }
      
        for (u=0;u<NoOfBlacks;u++){
            k=(*pAbr)[u].getNoOfEntries();
            (*pAbb)[u].writeableStatus(TRUE);
            // red neighbours of u
            for(l=0; l<k;l++){  
                v=(*pAbr)[u].getIndex(l); 
                if (v >=NoOfBlacks){ // v red?
                    n=(*pArb)[v-NoOfBlacks].getNoOfEntries();
                    for(j=0;j<n;j++){  
                        // get indexw of neighbours of v
                        w=(*pArb)[v-NoOfBlacks].getIndex(j);
                        if (w<NoOfBlacks) // w  black
                            (*pAbb)[u][w] = (*pAbb)[u][w]-(*pAbr)[u][v]*(*this)[v][w]/(*d)[v-NoOfBlacks];
                    } // for j (each neighbour of v)
                }
            } // for l  (each  neighbour of u)
            (*pAbb)[u].writeableStatus(FALSE);
        } // for u
      
   

  
        //   if ((NoOfBlacks>aMaxNoOfBlacks) && ( lev+1<aMaxLevel)) {
        //will the next level be decomposed? if yes:
        
        // lumping
        if(lumping)
        {
            cout << "TCycRedMatrixBlock::decompose (...) : lumping" << endl;
            for (u=0;u<NoOfBlacks;u++)
            {
                TMatrixData* mr=&((*pAbb)[u]);
                uint length=mr->getNoOfEntries();
            
                if (length>=aMSIZE)
                {  SortableMatrixRow*  smr= new SortableMatrixRow(mr);
                smr->heapsort(); // sort matrix row
                // check for zeros
                uint SIZE=length;
                TFloat eps = 1e-12;
                while(fabs(mr->getValue(SIZE-1))<eps )
                    SIZE--;
              
                SIZE=MIN(aMSIZE,SIZE);
                TFloat diag=(*mr)[u];
                for (i=SIZE;i<length;i++) // lumping of the suppressed coefficients
                    diag +=mr->getValue(i);
                // diag -=mr->getValue(i);
                (*mr)[u]=diag;
                mr->set(SIZE); // copy MSIZE largest entries
              
                //if (u<5)cout <<" after: size: "<<mr->getNoOfEntries()<<" "<<*mr<<endl; 
                delete smr;

           
                }
            }
        }
     
        // recursive call
        uint liv=pAbb->decompose( coarsener,
                                  lev+1,
                                  aMSIZE,
                                  lumping,
                                  aMaxNoOfBlacks,
                                  aMaxLevel );
        maxlevel=MAX(level,liv);
  
        // restore Abr
     
        TMatrixBlock *help=pAbr;
        pAbr=pAbrCopy;

        help->free();
        T->free();
        delete d;
    } //decompose if dim > MaxNoOfBlacks
    else
    {
        maxlevel=level;
        //  cout<<" writing   matrix A 1"<<endl;
        //matWriteMatrixBlock("matrixA1.mat", *this,   dim,"A1");
    }
//     if (level==1){
//         cout<<" writing   matrix A 1"<<endl;
//         matWriteMatrixBlock("matrixA1.mat", *this,   dim,"A1");
//     }
//     if (level==2){
//         cout<<" writing   matrix A 2"<<endl;
//         matWriteMatrixBlock("matrixA2.mat", *this,   dim,"A2");
//     }
 
//     if (level==3){
//         cout<<" writing   matrix A 3"<<endl;
//         matWriteMatrixBlock("matrixA3.mat", *this,   dim,"A3");
//     }
 
//     if ((level==1) &&(decomposed)){
//         cout<<" writing   matrices 1"<<endl;
//         matWriteMatrixBlock("matrixAbb1.mat", *pAbb,    NoOfBlacks,"Abb1");
//         matWriteMatrixBlock("matrixAbr1.mat", *pAbr, NoOfBlacks+   NoOfReds,"Abr1");
//         matWriteMatrixBlock("matrixArb1.mat", *pArb,    NoOfBlacks,"Arb1");
//         matWriteMatrixBlock("matrixArr1.mat", *pArr,    NoOfReds,"Arr1");
//     }
//     if  ((level==2) &&(decomposed)){
//         cout<<" writing   matrices 2"<<endl;
//         matWriteMatrixBlock("matrixAbb2.mat", *pAbb,    NoOfBlacks,"Abb2");
//         matWriteMatrixBlock("matrixAbr2.mat", *pAbr, NoOfBlacks+   NoOfReds,"Abr2");
//         matWriteMatrixBlock("matrixArb2.mat", *pArb,    NoOfBlacks,"Arb2");
//         matWriteMatrixBlock("matrixArr2.mat", *pArr,    NoOfReds,"Arr2");
//     }
      
    return maxlevel;
}// decompose

//
// output
//
ostream& 
operator<< (ostream& os, TCycRedMatrixBlock& matrixblock)
{
    uint prec=matrixblock.getOutputPrecision();
    os.precision(prec);
  
    for (uint i = 0; i < matrixblock.getNoOfRows(); i++) 
        os << "(" << i << ") : " << matrixblock[i] << "\n";
    os.precision(4);
    return (os);
}
