//
//  Project     : libmatrix
//  File        : TPermutation.cc
//  Author      : Ronald Kriemann
//  Purpose     : class for a permutation vector
//
// arch-tag: c9066846-2200-4aba-bb0e-f09a5ac4d5ea
//

#include "TPermutation.hh"

///////////////////////////////////////////
//
// constructor and destructor
//

TPermutation::TPermutation ( uint size )
        : TDynArray< uint >( size )
{
}

TPermutation::TPermutation ( const TPermutation & perm )
        : TDynArray< uint >( perm )
{
}

TPermutation::~TPermutation ()
{
}

///////////////////////////////////////////
//
// misc. methods for perm. handling
//

//
// invert permutation
//
TPermutation
TPermutation::invert ()
{
    TPermutation invPerm;
    uint        i;
	uint        size = getSize();

	invPerm.setSize( size );

	for (i = 0; i < size; i++)
		invPerm[(*this)[i]] = i;

    return invPerm;
}

//
// reverse permutation order
//
TPermutation
TPermutation::reverse ()
{
    TPermutation  revPerm;
	uint         i;
    uint         size = getSize();

	revPerm.setSize( size );
	
	for (i = 0; i < size; i++)
		revPerm[size-1-i] = (*this)[i];

    return revPerm;
}

///////////////////////////////////////////
//
// various operators
//

//
// set all entries in perm to n
//
TPermutation &
TPermutation::operator = ( const uint n )
{
    uint  i;
    uint  size = getSize();

    for (i = 0; i < size; i++)
        (*this)[i] = n;

    return *this;
}

//
// output
//
ostream &
operator << ( ostream & os, const TPermutation & perm )
{
    uint  i;
    uint  size = perm.getSize();

    for (i = 0; i < size; i++)
        os << perm[i] << endl;

    return os;
}
