//
//  Project : libnet
//  File    : prolongation.cc
//  Author  : Sabine Gutsch / Thomas Probst
//  Purpose : prolongation between refinement levels
//
// arch-tag: 55cb7d4a-e3ea-4d6a-8c88-d775de220f56
//

#include "T2DFrontier.hh"
#include "T3DFrontier.hh"

#include "TProlongation.hh"

//
// constructor
// fBoundary is the number of boundary nodes for which we need
// a prolongation/restriction
//

TBlockProlongation<3>::TBlockProlongation ( TNet<3> *cNet, TNet<3> *fNet,
                                            uint acDim, uint afDim, 
                                            uint M ) 
        : TMatrixBlock ( afDim ) 
{
    fDim = afDim;
    cDim = acDim;
    
    uint cNoOfInnerDoFs = cNet->getNoOfInteriorNodes();
    uint fNoOfInnerDoFs = fNet->getNoOfInteriorNodes();
    
    uint jj;
    TFrontier<3> *pFrontier;
    T3DFrontier  *p3DFrontier;
    
    T3DCellEdge *pEdge;
    TBoundary   *pBoundary;
    TVertex<3>   *node0, *node1, *midNode;
   
    //  name of fine/coarse node
    long fName0, fName1, fName2;
    long cName0, cName1;
    long Name;

    // flag  wether DoFs are on the edge
    uint prolType=0; // 0 : no prolongation
    // 1 :  prolongation of inner nodes
    // 2 :  prolongation of boundary nodes with DoFs
 
    TListIterator<TCell <3> >  iter(cNet->getList());
    for ( TCell<3> *cell = iter.first(); cell != NULL; 
          cell = iter.next()) 
	{
        for (uint i = 0; i < 4; i++) 
		{
            pFrontier = cell->getFrontier(i);
            p3DFrontier = (T3DFrontier*) pFrontier;

            for (uint j = 0; j < 3; j++) 
			{
                pEdge = p3DFrontier->getEdge(j);
                if (pEdge->hasMidNode() == TRUE) 
                { 
					// prolongation necessary
                    prolType=0;
                    pBoundary = pEdge->getBoundary();
                    if (pBoundary == NULL) 
						// inner edge
                        prolType=1;
                    else
                        if (pBoundary->getBoundaryDesc()->isDoF(M)) 
							// boundary edge  with DoF
                            prolType=2;
                    
                    if (prolType > 0)
					{ 
						// do prolongation
                        // get nodes
                        node0 = pEdge->getNode(0);
                        node1 = pEdge->getNode(2);
                        midNode = pEdge->getNode(1);
                        
                        // midNode name = midpoint name
                        if (prolType == 1) // inner edge <=> prolType=1 
                            fName2 = midNode->getName();
                        else
						{
							// boundary edge <=> prolType=2
                            Name = midNode->getDoFDesc()->getName(M);   
                            if (Name == NOT_DEFINED)
                                cerr << " TBlockProlongation<3>::TBlockProlongation: dof of vertex " << *midNode << " is not defined " << endl;
                            fName2 = Name+fNoOfInnerDoFs;   
                        }
                        
                        // node0 name
                        if (node0->getBoundary() == NULL)
                            cName0 = fName0 = node0->getName(); 
						// inner node
                        else
						{
							// boundary node
                            //coarse name
                            Name = node0->getDoFDesc()->getName(M);   
                            if (Name == NOT_DEFINED)
                                cerr <<" TBlockProlongation<3>::TBlockProlongation: dof of vertex " << *node0 << " is not defined " << endl;
                            cName0 = Name;
                            if (Name > -1)
							{
								// if valid DoF, assign real DoF name
                                cName0 += cNoOfInnerDoFs;
                                
                            }
                            // fine name
                            Name = node0->getDoFDesc()->getName(M);   
                            if (Name == NOT_DEFINED)
                                cerr <<" TBlockProlongation<3>::TBlockProlongation: dof of vertex " << *node0 << " is not defined " << endl;
                            fName0 = Name;
                            if (Name > -1)
								// if valid DoF, assign real DoF name
                                fName0 += fNoOfInnerDoFs;
                        }  // node 0 boundary node
                        
                        // node1 name
                        if (node1->getBoundary() == NULL)
                            cName1 = fName1 = node1->getName(); 
						// inner node
                        else
						{
							// boundary node
                            Name = node1->getDoFDesc()->getName(M);   
                            if (Name == NOT_DEFINED)
                                cerr << " TBlockProlongation<3>::TBlockProlongation: dof of vertex " << *node0 << " is not defined " << endl;
                            cName1 = Name;
                            if (Name > -1)
								// if valid DoF, assign real DoF name
                                cName1 += cNoOfInnerDoFs;
                            // fine name
                            Name = node1->getDoFDesc()->getName(M);   
                            if (Name == NOT_DEFINED)
                                cerr << " TBlockProlongation<3>::TBlockProlongation: dof of vertex " << *node0 << " is not defined " << endl;
                            fName1 = Name;
                            if (Name > -1)
								// if valid DoF, assign real DoF name
                                fName1 += fNoOfInnerDoFs;
                        } // else node 1 boundary node
                        /*    cout<<"prolType     : "<<prolType<<endl;
                              cout<<"       cName0: "<<cName0;
                              cout<<"       cName1: "<<cName1<<endl;
                              cout<<"       fName0: "<<fName0;
                              cout<<"       fName1: "<<fName1<<endl;
                              cout<<"       fName2: "<<fName2<<endl;
                              */

                        if ( (*this)[fName2].getNoOfEntries() == 0 ) 
						{
                            // not done yet
                            uint  newIndex[2];
                            TFloat newValue[2];
                            jj = 0;

                            if ( cName0 > -1 ) 
							{
                                newIndex[jj] = cName0; 
                                newValue[jj++] = 0.5;
                            }
                            if ( cName1 > -1 ) 
							{
                                newIndex[jj] = cName1; 
                                newValue[jj++] = 0.5;
                            }
                        
                            //  if ( j == 0 ){ 
                            //      cerr << "TBlockProlongation::TBlockProlongation:  Frontier has only boundary ";
                            //     cerr << "nodes, but no boundary flag " << endl; 
                            //}
                            if ( jj > 0 ) 
                                (*this)[fName2].set( jj, newIndex, newValue );
                            // cout<<" Matrixeintrag fuer "<<fName2<<"  :"<<(*this)[fName2]<<endl;
                        }

                        // and now the identity part of the prolongation/
                        // restriction matrix
                        // first (coarse) vertex
                        if ( fName0 > -1 ) 
                            if ( (*this)[fName0].getNoOfEntries() == 0 )
							{  
                                // not done yet
                                uint  newIndex[1];
                                TFloat newValue[1];
                                newIndex[0] = cName0;
                                newValue[0] = 1.0;
                                (*this)[fName0].set( 1, newIndex, newValue );
                            }
                        // second (coarse) vertex
                        if ( fName1 >-1 ) 
                            if ( (*this)[fName1].getNoOfEntries() == 0 )
							{  
                                // not done yet
                                uint  newIndex[1];
                                TFloat newValue[1];
                                newIndex[0] = cName1;
                                newValue[0] = 1.0;
                                (*this)[fName1].set(1,newIndex,newValue);
                            }
                    } // prolType>0
                } // prolongation necessary
            }// for each edge
        }// for each frontier
    } // for each cell
} // constructor TBlockProlongation


template<int N>
TBlockProlongation<N>::TBlockProlongation ( TNet<N> *cNet, TNet<N> *fNet,
                                            uint acDim, uint afDim,
                                            uint M = 0) 
{
    cerr << "TBlockProlongation<N>::TBlockProlongation, N = " << N;
    cerr << " : Sorry, not implemented " << end;
};




//
//  matrix dependent prolongation/restriction
//
template<int N>
void
TBlockProlongation<N>::matrixDependent ( TMatrixBlock *mb )
{
    uint i,j;
    TFloat newvalue;
    for (i = 0; i < getNoOfRows(); i++) 
        if ( (*this)[i].getNoOfEntries() > 0 && (*this)[i].getValue( 0 ) < 0.999 )
            for (j = 0; j < (*this)[i].getNoOfEntries(); j++)
            {
                //cout << i << "  " << (*this)[i] << endl;
                //cout << j << "  " << (*this)[i].getIndex( j ) << endl;
                //cout << (*mb)[i] << endl;
                newvalue = (*mb)[i][(*this)[i].getIndex( j )] / (*mb)[i].getValue( 0 );
                (*this)[i].setValue( j, newvalue );
            }
	
}

//
// prolongation
//
template<int N>
TVectorBlock<TFloat> 
TBlockProlongation<N>::prolong( TVectorBlock<TFloat> &cVec ) 
{
    TVectorBlock<TFloat> fVec( 0.0, fDim );
    fVec = (*this) * cVec;
    return (fVec);
}

//
// restriction
//
template<int N> 
TVectorBlock<TFloat> 
TBlockProlongation<N>::restrict( TVectorBlock<TFloat> &fVec ) 
{
    TVectorBlock<TFloat> cVec( 0.0, cDim );
    (*this).transposeMult( fVec, cVec);
    return (cVec);
}

//////////////////////////////////////////////////////////////

//
// prolongation
//
template<int N>
TBlockVector<TFloat> 
TProlongation<N>::prolong ( TBlockVector<TFloat> &cVec ) 
{
    uint i;
    uint size[n];
	
    for (i = 0; i < n; i++) 
        size[i] = pBlockProlong[i]->getfDim();
	
    TBlockVector<TFloat> fVec( n, size );

    for (i = 0; i < n; i++) 
        fVec[i] = pBlockProlong[i]->prolong( cVec[i] );
	
    return (fVec);
}

//
// restriction
//
template <int N>
TBlockVector<TFloat> 
TProlongation<N>::restrict( TBlockVector<TFloat> &fVec ) 
{
    uint i;
    uint size[n];
	
    for (i = 0; i < n; i++) 
        size[i] = pBlockProlong[i]->getcDim();
	
    TBlockVector<TFloat> cVec( n, size );

    for (i = 0; i < n; i++) 
        cVec[i] = pBlockProlong[i]->restrict( fVec[i] );

    return (cVec);
}


template class TProlongation<2>;
template class TProlongation<3>;
