//
//  Project     : libnet
//  File        : sparsevector.cc
//  Author      : Ronald Kriemann 
//  Purpose     : class for a sparse blockvector
//
// arch-tag: b0547549-0f2e-47fd-a01d-83f29cabcb03
//

#include "sparsevector.hh"

////////////////////////////////////////////////////////////
//
// TSparseBlockVector : class for a sparse blockvector
//

// constructor
// init an empty vector
TSparseBlockVector::TSparseBlockVector () 
{}

// init an empty vector with n blocks
TSparseBlockVector::TSparseBlockVector ( int n ) 
{
    setSize( n );
}

//
// init an vector with given number of indices in n blocks
//
TSparseBlockVector::TSparseBlockVector ( int n, uint size[] ) 
{
    uint  i;
    uint  noOfBlocks = n;

    setSize( n );

    for ( i = 0; i < noOfBlocks; i++ )
        (*this)[i].set( size[i] );
}

// copy constructor
TSparseBlockVector::TSparseBlockVector ( const TSparseBlockVector & vec )
{
    (*this) = vec;
}

// destructor
TSparseBlockVector::~TSparseBlockVector () 
{}

// set a new number of blocks
void 
TSparseBlockVector::setNoOfBlocks ( uint n )
{
    setSize( n );
}

//
// add operator (we assume, that both vectors have same structure)
//
TSparseBlockVector& 
TSparseBlockVector::operator+= ( const TSparseBlockVector& vec )
{
    uint  i;
    uint  noOfBlocks = getSize();

    for ( i = 0; i < noOfBlocks; i++ )
        (*this)[i] += vec[i];

    return *this;
}

TSparseBlockVector 
TSparseBlockVector::operator+ ( const TSparseBlockVector& vec )
{
    TSparseBlockVector vecTmp( *this );
    
    return vecTmp += vec;
}

//
// sub operator (we assume, that both vectors have same structure)
//
TSparseBlockVector& 
TSparseBlockVector::operator-= ( const TSparseBlockVector& vec )
{
    uint  i;
    uint  noOfBlocks = getSize();

    for ( i = 0; i < noOfBlocks; i++ )
        (*this)[i] -= vec[i];

    return *this;
}

TSparseBlockVector 
TSparseBlockVector::operator- ( const TSparseBlockVector& vec )
{
    TSparseBlockVector vecTmp( *this );
    
    return vecTmp -= vec;
}

//
// scalar mult
//
TSparseBlockVector& 
TSparseBlockVector::operator*= ( const TFloat& scalar )
{
    uint  i;
    uint  noOfBlocks = getSize();

    for ( i = 0; i < noOfBlocks; i++ )
        (*this)[i] *= scalar;

    return *this;
}

//
// inner produkt between sparsevectors (we assume, that both
// vectors have same structure)
//
TFloat 
TSparseBlockVector::operator* ( const TSparseBlockVector& vec )
{
    uint    i;
    uint    noOfBlocks = getSize();
    TFloat  res = 0.0;

    for ( i = 0; i < noOfBlocks; i++ )
        res += (*this)[i] * vec[i];

    return res;
}

TSparseBlockVector 
TSparseBlockVector::operator* ( const TFloat& scalar )
{
    TSparseBlockVector vecTmp( *this );
    
    return vecTmp *= scalar;
}

TSparseBlockVector 
operator* ( const TFloat & scalar,
            const TSparseBlockVector & vec )
{
    TSparseBlockVector vecTmp( vec );
    
    return vecTmp *= scalar;
}

// copy operator
TSparseBlockVector& 
TSparseBlockVector::operator= ( const TSparseBlockVector& vec )
{
    return *this = vec;
}

//
// ouput
//
ostream& 
operator<< (ostream& os, const TSparseBlockVector& vec) 
{
    uint  i;
    uint  noOfBlocks = vec.getSize();

    os.setf( ios::fixed, ios::floatfield );
    os.precision( 2 );

    for ( i = 0; i < noOfBlocks; i++ )
        os << vec[i];

    return os;
}
