//
//  Project   : libnet
//  File      : vector.cc
//  Author    : Ronald Kriemann
//  Purpose   : template class for a vector (template in dimension)
//
// arch-tag: 2a4102ac-95fa-48bc-aaf7-1b1fbe5cbe2d
//

#include "vector.hh"

//
// constructors
//
template<int N>
TVector<N>::TVector ()
{
    *this = 0;
}

template<int N>
TVector<N>::TVector ( TFloat coord[] )
{
    uint  i;
    
    for ( i = 0; i < N; i++ )
        arrCoord[i] = coord[i];
}


TVector<2>::TVector ( const TFloat & x, const TFloat & y ) 
{
    arrCoord[X] = x;
    arrCoord[Y] = y;
}

TVector<3>::TVector ( const TFloat & x, const TFloat & y, const TFloat & z ) 
{
    arrCoord[X] = x;
    arrCoord[Y] = y;
    arrCoord[Z] = z;
}

//
// add-and-set operators
//
template <int N>
TVector<N>&
TVector<N>::operator+= ( const TVector<N>& vector ) 
{
    uint  i;
    
    for ( i = 0; i < N; i++)
        arrCoord[i] += vector.arrCoord[i];
    
    return *this;
}

//
// sub-and-set operators
//
template <int N>
TVector<N> &
TVector<N>::operator -= ( const TVector<N>& vector ) 
{
    uint  i;
    
    for ( i = 0; i < N; i++)
        arrCoord[i] -= vector.arrCoord[i];
    
    return *this;
}

//
// scalar multiplication
//
template <int N>
TVector<N> &
TVector<N>::operator *= ( const TFloat & scalar ) 
{
    uint  i;
    
    for ( i = 0; i < N; i++ )
        arrCoord[i] *= scalar;
    
    return *this;
}

//
// inner produkt (euclid)
//
template<int N>
TFloat
TVector<N>::operator * ( const TVector<N>& vector ) const
{
    uint   i;
    TFloat s = 0;
    
    for ( i = 0; i < N; i++ )
        s += arrCoord[i] * vector.arrCoord[i];
    
    return s;
}

//
// array operator
//
template<int N>
TFloat& 
TVector<N>::operator[] ( uint i ) 
{ 
#ifdef ERR_BOUNDS
    if (i >= N)
        cerr << "TVector::operator[] (" << i << ") : "
             << "out of bound (dim=" << N<< ")" << endl;
#endif
    return (arrCoord[i]); 
}

template<int N>
const TFloat& 
TVector<N>::operator[] ( uint i ) const 
{ 
#ifdef ERR_BOUNDS
    if (i >= N)
        cerr << "const TVector::operator[] (" << i << ") const : "
             << "out of bound (dim=" << N << ")" << endl;
#endif

    return (arrCoord[i]); 
}

//
// copy operator
//
template<int N>
TVector<N>&
TVector<N>::operator = ( const TVector<N>& vector )
{
    uint  i;
    
    for (i = 0; i < N; i++)
        arrCoord[i] = vector.arrCoord[i];
    
    return *this;
}

template<int N>
TVector<N>&
TVector<N>::operator = ( const TFloat & scalar )
{
    uint  i;

    for ( i = 0; i < N; i++ )
        arrCoord[i] = scalar;

    return *this;
}

//
// comparing
//
template<int N>
const bool 
TVector<N>::operator == ( const TVector<N> &vector ) const
{
    uint  i;
    
    for (i = 0; i < N; i++)
    {
        if (arrCoord[i] != vector.arrCoord[i])
            return FALSE;
    }// for
    
    return TRUE;
}

template<int N>
const bool 
TVector<N>::operator != ( const TVector<N> &vector ) const
{
    return !(*this == vector);
}

//
// cross product 
//
TVector<3> 
TVector<3>::cross (const TVector<3> & v2 )
{
    TVector<3>  vecTmp;
    
    vecTmp[0] = (*this)[1]*v2[2] - (*this)[2]*v2[1];
    vecTmp[1] = (*this)[2]*v2[0] - (*this)[0]*v2[2];
    vecTmp[2] = (*this)[0]*v2[1] - (*this)[1]*v2[0];

    return vecTmp;
}

template class TVector<2>;
template class TVector<3>;
template class TVector<4>;
template class TVector<6>;
