//
//  Project   : libmisc
//  File      : TFlagable.cc
//  Author    : Ronald Kriemann
//  Purpose   : class providing ability to store and access flags
//
// arch-tag: 91b19d6d-fdcf-46d7-a2b3-1a2fe6b7ea68
//

#include "TFlagable.hh"

////////////////////////////////////////
//
// constructor and destructor
//

template <class T>
TFlagable<T>::TFlagable ()
        : _flags( 0 )
{}

template <class T>
TFlagable<T>::TFlagable ( const T & mask )
        : _flags ( mask )
{}

template <class T>
TFlagable<T>::TFlagable ( const TFlagable<T> & flag )
{
    *this = flag;
}

template <class T>
TFlagable<T>::~TFlagable ()
{}

////////////////////////////////////////
//
// access member variable
//

template <class T>
const bool
TFlagable<T>::flag ( const T & mask ) const
{
    return _flags & mask;
}

template <class T>
const bool
TFlagable<T>::hasFlag ( const T & mask ) const
{
    return _flags & mask;
}

template <class T>
const T &
TFlagable<T>::flags () const
{
    return _flags;
}

template <class T>
void
TFlagable<T>::setFlag ( const T & mask )
{
    _flags |= mask;
}

template <class T>
void
TFlagable<T>::clearFlag ( const T & mask )
{
    _flags &= ~mask;
}

template <class T>
void
TFlagable<T>::clearFlags ()
{
    _flags = 0;
}

////////////////////////////////////////
//
// misc operators
//

template <class T>
TFlagable<T> &
TFlagable<T>::operator = ( const TFlagable<T> & flag )
{
    _flags = flag._flags;

    return *this;
}

template <class T>
ostream &
operator << ( ostream & os, const TFlagable<T> & flag )
{
    int    i;
    T  mask = 1;
    T  size = sizeof( T ) * 8;

    mask <<= size - 1;
    
    for ( i = size; i > 0; i-- )
    {
        if (flag._flags & mask)
            os << 1;
        else
            os << 0;

        mask >>= 1;
    }// for

    return os;
}

/////////////////////////////////////////
//
// template instantion for usual types
//

template class TFlagable<uchar>;
template class TFlagable<ushort>;
template class TFlagable<uint>;
template class TFlagable<ulong>;
