//
//  Project   : libmisc
//  File      : TOctave.cc
//  Author    : Ronald Kriemann
//  Purpose   : class for a octave interface
//
// arch-tag: 5e7e6a4a-58a6-4281-9f0d-921b2c9e8bd5
//

extern "C" {
#include <stdio.h>
#include <string.h>
}

#include "TOctave.hh"

// prefix/suffix for octave files
#define PREFIX  "files/"
#define SUFFIX  ".mat"

/////////////////////////////////////
//
// IO of matrices (assumed to be quadratic)
//

void
TOctave::saveMatrix ( TMatrixBlock * pMatrix,
                      const char * filename, 
                      const char * matname )
{
    if ((pMatrix == NULL) || (filename == NULL) || (matname == NULL))
    {
        cout << "TOctave::saveMatrix (...) : got NULL args" << endl;
        return;
    }// if

    uint     i, j;
    FILE   * file;
    char     buffer[256];
    uint     size;
    uint    noOfEntries, index;
    uint  * entries;
    TFloat * values;

    sprintf( buffer, "%s%s%s", PREFIX, filename, SUFFIX );
    if ((file = fopen( buffer, "w+" )) == NULL)
    {
        perror( "TOctave::saveMatrix (...) : fopen = " );
        return;
    }// if

    size = pMatrix->getNoOfRows();
    
    //
    // write header
    //
    sprintf( buffer, "# name: %s\n", matname );
    fputs( buffer, file );
    fputs( "# type: matrix\n", file );
    sprintf( buffer, "# rows: %d\n", size );
    fputs( buffer, file );
    sprintf( buffer, "# columns: %d\n", size );
    fputs( buffer, file );

    //
    // write matrix
    //

    for ( i = 0; i < size; i++ )
    {
        noOfEntries = (*pMatrix)[i].getNoOfEntries();
        entries     = (*pMatrix)[i].getIndex();
        values      = (*pMatrix)[i].getValue();

        for ( j = 0; j < size; j++ )
        {
            if ((index = isIndex( j, entries, noOfEntries )) >= 0)
            {
                sprintf( buffer, " %g", double(values[index]) );
                fputs( buffer, file );
            }// if
            else
                fputs( " 0", file );
        }// for

        fputs( "\n", file );
    }// for
    
    fclose( file );
}

/////////////////////////////////////
//
// IO of vectors
//

void
TOctave::saveVector ( TVectorBlock<TFloat> * pVector,
                      const char * filename, 
                      const char * vecname )
{
    if ((pVector == NULL) || (filename == NULL) || (vecname == NULL))
    {
        cout << "TOctave::saveVector (...) : got NULL args" << endl;
        return;
    }// if

    uint     i;
    FILE   * file;
    char     buffer[256];
    uint     size;

    sprintf( buffer, "%s%s%s", PREFIX, filename, SUFFIX );
    if ((file = fopen( buffer, "w+" )) == NULL)
    {
        perror( "TOctave::saveMatrix (...) : fopen = " );
        return;
    }// if

    size = pVector->getSize();
    
    //
    // write header
    //
    sprintf( buffer, "# name: %s\n", vecname );
    fputs( buffer, file );
    fputs( "# type: matrix\n", file );
    sprintf( buffer, "# rows: %d\n", size );
    fputs( buffer, file );
    fputs( "# columns: 1\n", file );

    //
    // write vector
    //

    for ( i = 0; i < size; i++ )
    {
        sprintf( buffer, " %g\n", double( (*pVector)[i] ) );
        fputs( buffer, file );
    }// for
    
    fclose( file );
}

//
// check if index is an index in entries
// (return pos or -1 of not an entry)
//
long
TOctave::isIndex ( uint index, uint * entries, uint noOfEntries )
{
    uint  i;
    
    for ( i = 0; i < noOfEntries; i++ )
    {
        if (entries[i] == index)
            return i;
    }// for

    return -1;
}
