//
//  Project   : libmisc
//  File      : TRandom.cc
//  Author    : Ronald Kriemann
//  Purpose   : class for a random numer generator
//
// arch-tag: a9445801-5621-4c31-92b1-164b3f86a274
//


extern "C"
{
#include <stdlib.h>
#include <unistd.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
}

#include "defines.hh"
#include "TRandom.hh"

//
// all static variables
//
bool  TRandom::bInitialized = FALSE;

//
// constructor
//
TRandom::TRandom ()
{}

//
// init. random number generator
//
void
TRandom::seed ( int s )
{
    int  r;
  
    r = s;
    
    if (s == 0)
    {
#ifdef LINUX  
        int  fd;
        if ((fd = open( "/dev/random", O_RDONLY )) >= 0)
        {
            if (read( fd, (char*) &r, sizeof(r) ) < 0)
                r = int(getpid());
            
            close(fd);
        }// if
        else
            r = int(getpid());
#else
        r = int(getpid());
#endif
    }// if
    else
        r = s;

    srand( r );
    
    bInitialized = TRUE;
}

//
// return random number
//
double
TRandom::random ( int s )
{
	if (s != 0)
        seed( s );

    // if not init. do it with default value
    if (!bInitialized)
        seed( 0 );

    return double(rand()) / double(RAND_MAX);
}

//
// return random number in [a,b]
//
double
TRandom::random ( double a, double b, int s )
{
    return (a + ((b - a) * random( s )));
}

//  
// return integer random in [0,RAND_MAX)
//
long
TRandom::randomInt ( int s )
{
    return long( random( 0, RAND_MAX, s) );
}

//
// return ineger random in [a,b)
//
long
TRandom::randomInt ( long a, long b, int s )
{
    return long( random( a, b, s) );
}
