//
//  Project   : libmisc
//  File      : scanner.cc
//  Author    : Ronald Kriemann
//  Purpose   : class for a lex. scanner
//
// arch-tag: 927522bb-72d7-4ff7-981d-b17b3b804970
//

#include <string.h>
#include <ctype.h>

#include "scanner.hh"

//
// constructor
//
TScanner::TScanner ( const char *string )
{
    strcpy( szString, string );
    nPos = 0;
}

//
// returns next token and sets value of token
//
int 
TScanner::nextToken( TTokenData& data ) 
{
	char *strpos;

	while (szString[nPos] != 0)
	{
		switch (szString[nPos])
		{
		case 'V':
			// got a vertex
			data.token = tokVertex;
			// read the name (an int)
			nPos++;
			return (getInt(data.nInt));

		case 'F':
			// got a frontier
			data.token = tokFrontier;
			// read the name (an int)
			nPos++;
			return (getInt(data.nInt));
				
		case 'C':
			// got a cell
			data.token = tokCell;
			// read the name (an int)
			nPos++;
			return (getInt(data.nInt));

		case 'I':
			// frontier typ is interior
			data.token = tokInterior;
			nPos++;
			return (SUCCESS);

		case 'D':
			// frontier typ is Dirichlet
			data.token = tokDirichlet;
			nPos++;
			return (SUCCESS);

		case 'N':
			// frontier typ is Neumann
			data.token = tokNeumann;
			nPos++;
			return (SUCCESS);

		case '0': case '1': case '2': case '3':
		case '4': case '5': case '6': case '7':
		case '8': case '9': case '-':
		{
			int nInt;
			// a number follows
			if (getInt( nInt ) == SUCCESS)
			{
				if (szString[nPos] == '.')
				{
					nPos++;
					// it is a float
					data.token  = tokFloat;
					data.fFloat = nInt;
					// get numbers after point
					if ((getInt(nInt) == SUCCESS) && (nInt >= 0))
					{
						TFloat fFloat = nInt;
						while (fFloat >= 1)
							fFloat /= 10;
						if (data.fFloat >= 0)
							data.fFloat += fFloat;
						else
							data.fFloat -= fFloat;
						return (SUCCESS);
					}// if
					else
						return (ERROR);
				}// if
				else
				{
					// it is a int
					data.token = tokInt;
					data.nInt  = nInt;
					return (SUCCESS);
				}// else
			}// if
			else
				return (ERROR);
		}

		case '#':
		case '\n':
			// end of line
			data.token = tokEOL;
			return (SUCCESS);

		default:
			// more complicated strings
			if ((strpos = strstr( szString+nPos, "BAENSCH" )) != NULL)
			{
				nPos += 7;
				data.token = tokBaensch;
				return (SUCCESS);
			} 
			else if ((strpos = strstr( szString+nPos, "REGULAR" )) != NULL)
			{
				nPos += 7;
				data.token = tokRegular;
				return (SUCCESS);
			}
			break;
		}// switch

		// next char
		nPos++;
	}// while

	// end of line
	data.token = tokEOL;

	return (SUCCESS);
}

//
// tries to read an integer and gives it back
//
int 
TScanner::getInt( int &nInt ) 
{
	bool noNumberYet = TRUE;
	bool isNegativ   = FALSE;

	nInt = 0;

	// scan for an int
	while (szString[nPos] != 0)
	{
		if (isdigit(szString[nPos]) != 0)
		{
			// a number
			noNumberYet = FALSE;
			if (isNegativ == FALSE)
				nInt = nInt * 10 + alpha2digit(szString[nPos]);
			else
				nInt = nInt * 10 - alpha2digit(szString[nPos]);
		} else if (isspace(szString[nPos]) != 0)
		{
			// we got a number
			if (noNumberYet == FALSE)
				return (SUCCESS);
		} else if (szString[nPos] == '-')
		{
			// we got a number
			if (noNumberYet == FALSE)
				return (SUCCESS);
			else // a minus -> invert the value
				isNegativ = !isNegativ;
		} else
		{
			// we got a number
			if (noNumberYet == FALSE)
				return (SUCCESS);
			else // error
				return (ERROR);
		}

		// next char
		nPos++;
	}// while

	// we got a number
	if (noNumberYet == FALSE)
		return (SUCCESS);
	else
		return (ERROR);
}

//
// converts alpha char in digit
//
int
TScanner::alpha2digit( char alpha )
{
    return (int(alpha) - 48);
}
