//
//  Project   : libmisc
//  File      : timer.cc
//  Author    : Ronald Kriemann
//  Purpose   : class for a timer (for speed tests)
//
// arch-tag: 0cba9a2b-b70b-4f14-ae9f-dc26775cb4b7
//

#include "timer.hh"

//
// constructor
//
TTimer::TTimer () 
{}

//
// sets the first timer
//
void 
TTimer::start () 
{
	getrusage( RUSAGE_SELF, &timerStart );
}

//
// sets the second timer
//
void 
TTimer::stop () 
{
	getrusage( RUSAGE_SELF, &timerEnd );
}
	
//
// returns time between start and end as float
//
const TFloat
TTimer::stat () 
{
	long   seconds, m_seconds;
	TFloat fTime;

	convertTime( timerStart.ru_utime, timerEnd.ru_utime, 
				 seconds, m_seconds );
	
	fTime = TFloat(seconds) + TFloat(m_seconds) / 1000.0;

	return fTime;
}

//
// converts timeval to in sec and m_sec
//
void 
TTimer::convertTime( struct timeval start, struct timeval end,
					 long &sec, long &m_sec ) const
{
	long seconds   = end.tv_sec  - start.tv_sec;
	long u_seconds = end.tv_usec - start.tv_usec;

	if (u_seconds < 0) 
	{
		seconds--;
		u_seconds = (1000000 + end.tv_usec) - start.tv_usec;
	}// if

	if (u_seconds > 1000000)
	{
		seconds++;
		u_seconds -= 1000000;
	}// if

	sec   = seconds;
	m_sec = u_seconds / 1000;
}

//
// stream output (only the time)
//
ostream& 
operator<< ( ostream& os, const TTimer& timer )
{
	long seconds, m_seconds;
	timer.convertTime( timer.timerStart.ru_utime, timer.timerEnd.ru_utime, 
					   seconds, m_seconds );

	os << seconds << '.';
	os.width(3); 
	os.fill('0');
	os << m_seconds << "s";

	return os;
}
