//
//  Project   : libnet
//  File      : TDomain.cc
//  Author    : Ronald Kriemann
//  Purpose   : class for representing a domain in R^n
//
// arch-tag: c691bba5-aa35-4393-86d3-ab17fc5bfba8
//

#include "TDomain.hh"

////////////////////////////////////////////////////////////
//
// constructor and destructor
//

TDomain::TDomain ()
{
}

TDomain::~TDomain ()
{
    TListIterator< TBoundary >  iterBound( listBound );
    TBoundary                  *boundary;

    iterBound.first();
    while ((boundary = iterBound++) != NULL)
    {
        delete boundary;
    }// while
}

///////////////////////////////////////////////////////////
//
// access member variables
//

TVector<3> *
TDomain::min ()
{
    return & vecMin;
}

TVector<3> *
TDomain::max ()
{
    return & vecMax;
}

void
TDomain::setMin ( TVector<3> * pVector )
{
    if (pVector != NULL)
        vecMin = *pVector;
    else
        cout << "TDomain::setMin (...) : got NULL arg" << endl;
}

void
TDomain::setMax ( TVector<3> * pVector )
{
    if (pVector != NULL)
        vecMax = *pVector;
    else
        cout << "TDomain::setMax (...) : got NULL arg" << endl;
}

const TList< TBoundary > &
TDomain::getBoundaries () const
{
    return listBound;
}

///////////////////////////////////////////////////////////
//
// boundary management
//
    
//
// add a boundary to list
//
void
TDomain::addBoundary ( TBoundary * pBoundary )
{
    if (pBoundary != NULL)
        listBound.add( *pBoundary );
    else
        cout << "TDomain::addBoundary () : got NULL boundary" << endl;
}

//
// return boundary (or INTERIOR) assoc. to given point
//
TBoundary *
TDomain::boundary ( TFloat x, TFloat y, TFloat z )
{
    cout << "TDomain::boundary (...) : MUST be overloaded" << endl;

    return NULL;
}

TBoundary *
TDomain::boundary ( TVector<3> * v )
{
    if (v == NULL)
    {
        cout << "TDomain::boundary (...) : got NULL arg" << endl;
        return NULL;
    }// if
    
    return boundary( (*v)[X], (*v)[Y], (*v)[Z] );
}
    
///////////////////////////////////////////////////////////
//
// misc methods
//

//
// characteristic function of domain
//
bool
TDomain::charFunc ( TVector<3> * pVector ) const
{
    return FALSE;
}
