%{
#include "defines.hh"
#include "../net/parser.y.hh"

extern int nLine;
extern int nPos;

void update();
%}

LETTERS    [a-zA-Z]
NUM        [0-9]
SPECIAL    [_]
NAME       {LETTERS}+({NUM}|{LETTERS}|{SPECIAL})*
WHITESPACE [ \t]
COMMENT    "//"

%%

"dim"   { update(); return DIM; }

"node"     { update(); return STRUCTNODE; }
"edge"     { update(); return STRUCTEDGE; }
"frontier" { update(); return STRUCTFRONTIER; }
"cell"     { update(); return STRUCTCELL; }
"boundary" { update(); return STRUCTBOUND; }
"domain"   { update(); return STRUCTDOMAIN; }

"coord"      { update(); return COORD; }
"nodes"      { update(); return NODEFIELD; }
"edges"      { update(); return EDGEFIELD; }
"frontiers"  { update(); return FRONTIERFIELD; }
"type"       { update(); return TYPEFIELD; }
"refine"     { update(); return REFINEFIELD; }
"boundaries" { update(); return BOUNDFIELD; }

"interior"  { update(); return INTERIORFIELD; }

{NUM}*("."){NUM}+ {
    update(); 
    yylval.fFloat = atof( yytext );
    return FLOAT;
}

{NUM}+ {
    update();
    yylval.nInt = atoi( yytext );
    return INT;
}

{NAME} {
    update();
    strcpy( yylval.szString, yytext );
    return STRING;
}

"="|";"|"{"|"}" { update(); return yytext[0]; }

\n { nLine++; nPos = 0; }

({COMMENT}.*)|({WHITESPACE}+) { update(); } /* eat it up */

%%

/*
 * things to do while scanning
 */
void
update ()
{
    nPos += strlen( yytext );
}

#ifndef yywrap
int yywrap(void)    /* do this to avoid -lfl */
{
	return 1;
}
#endif
