//
//  Project  : libnet
//  File     : cgSolver.cc
//  Author   : Sabine Gutsch / Thomas Probst
//  Purpose  : cg solver
//
// arch-tag: 33c19fa6-8804-4ca3-80e7-1156307399c8
//

extern "C" {
#include <stdio.h>

#include <math.h>
}

#include "cgSolver.hh"

//
// constructor
//
TCGSolver::TCGSolver ( TSolver *prec, TBlockMatrix *paA, 
					   TBlockVector<TFloat> *paRHS, 
					   TBlockVector<TFloat> *paX, 
					   uint amaxiter, 
					   TFloat aaccur, 
					   uint averb ) 
        : TPrecSolver ( prec, paA, paRHS, paX, amaxiter, aaccur, averb) 
{
    setName( "CG" );
}

//
// init. iteration
//
void 
TCGSolver::init ()
{ 
    pPrecond->setA(pA);
    
    // r = b - Ax
	r  = (*pRHS);
	r -= (*pA) * (*pX);
    
    if (isPrecSecondNF())
        p = ((TSecondNF*)pPrecond)->WInvers( r );
    else
    {
        p = *pX;
        pPrecond->setRHS( &r );
        pPrecond->setX( &p );
        pPrecond->solve();
    }// else
    
	rho = p * r;

    // set correct structure to q
    q = *pX;
}

#define EPSILON  1e-20

//
// one iteration step
//
int 
TCGSolver::step ()
{ 
    TFloat  lambda, rhoNew;
  
    a      = (*pA) * p;
    lambda = rho / ( a * p );
    
    // x = x + lambda*p
    (*pX) += lambda * p;
    
    // r = r - lambda*a
    r -= lambda * a;
    
    if (isPrecSecondNF())
        q = ((TSecondNF*)pPrecond)->WInvers( r );
    else
    {
        q = 0;
        pPrecond->setRHS( &r );
        pPrecond->setX( &q );
        pPrecond->solve();
    }// else
    rhoNew = q * r;
  
    // p = q + (rhowNew / rho) * p
    p *= rhoNew / rho;
    p += q;

    if (fabs(rho = rhoNew) < EPSILON)
        return RETURN_BREAKDOWN;
    
    return RETURN_CONTINUE;
}
