//
//  Project  : libnet
//  File     : kaczmarz.cc
//  Author   : Sabine Gutsch
//  Purpose  : kaczmarz solver
//
// arch-tag: 119fc342-7494-4c43-907d-1e2b10d738d4
//

#include "kaczmarz.hh"

//
// constructor
//
TKaczmarz::TKaczmarz( TBlockMatrix *paA, 
                      TBlockVector<TFloat> *paRHS, 
					  TBlockVector<TFloat> *paX, 
					  uint amaxiter,
					  TFloat aaccur,
					  uint averb ) 
	: TSolver ( paA, paRHS, paX, amaxiter, aaccur, averb) 
{
	setName("Kaczmarz"); 
}

//
// one iteration step
//
int 
TKaczmarz::step () 
{ 
	uint  i,j,k;

	TFloat s;

	for ( i = 0; i < pRHS->getNoOfBlocks(); i++) {
		for ( j = 0; j < (*pRHS)[i].getSize(); j++)
		{
			pA->getTransposedRow( a, i, j );

			s = 0.0 - (*pRHS)[i][j];

			for ( k = 0; k < pRHS->getNoOfBlocks(); k++) 
			{
				if ( (*pA)[i][k].getNoOfRows() > 0 )
					s += (*pA)[i][k][j] * (*pX)[k]; 
			}// for

			s = s / ( a * a );

			// x  = x - s*a
			a *= s;
			(*pX) -= a;
		}// for
	}

	return (RETURN_CONTINUE);
}
