//
//  Project  : libnet
//  File     : squaredSmoother.cc
//  Author   : Sabine Gutsch / Thomas Probst
//  Purpose  : smoother for a general system in second normal form
//
// arch-tag: e0fd377c-c745-4f58-adea-5baf9a27e4bc
//

#include "squaredSmoother.hh"

//
// constructor
//
TSquaredSmoother::TSquaredSmoother ( TBlockMatrix *paA, 
                                     TBlockVector<TFloat> *paRHS, 
                                     TBlockVector<TFloat> *paX, 
                                     uint amaxiter,
                                     TFloat aaccur, 
                                     uint averb,
                                     TFloat aDamp )
        : TSecondNF ( paA, paRHS, paX, amaxiter, aaccur, averb, aDamp) 
{ 
	setName("Squared smoother");
    h = 1.0;
    epsilon = 1.0;
}
 
//
// calculate WInvers
//
TBlockVector<TFloat> 
TSquaredSmoother::WInvers ( const TBlockVector<TFloat> &r )
{ 

    uint n = r.getNoOfBlocks();
	TBlockVector<TFloat> vecTmp( r );
    TFloat scalFactor[3];
    scalFactor[0] = scalFactor[1] = scalFactor[2] = 1.0;
    TFloat omega;

    if ( n > 1 ) {
        h = 1.0 / ( sqrt(vecTmp[n-1].getSize()) - 1.0);
        //h = 1.0 / (sqrt(vecTmp[0].getSize()) + 1.0);

        scalFactor[n-1] = sqrt(epsilon) / h;
        vecTmp[2] *= (epsilon / (h*h));
	}

    vecTmp = pA->transposeMult( vecTmp );

    if ( n > 1 ) 
        vecTmp[2] *= (epsilon / (h*h));

    omega = pA->rowSumNorm( scalFactor, scalFactor );
	//cout << "1/(omega*omega): " << 1.0 / (omega * omega) << "    1/(h*h): " << 1/(h*h) << endl;

    vecTmp *= (1.0 / (omega * omega));

    return (vecTmp);
}
