//
//  Project :  libvisual
//  File    :  TConvView.cc
//  Purpose :  class for convergence visualization
//  Author  :  Ronald Kriemann
//
// arch-tag: de04a29e-1831-4aed-b326-89f7c8e8a945
//

#include "TConvView.hh"

//
// constructor
//
TConvView::TConvView ( TDynArray<TFloat> *parrConv )
{
	if (parrConv != NULL)
		arrConv = *parrConv;
	else
		cerr << "TConvView::TConvView (TDynArray) : no array was given" << endl;
}

//
// draw the net
//
void 
TConvView::draw ( TPrinter& p )
{
	uint    i, maxIt;
	TFloat  x, maxConv;
	char    szString[64];

	if (arrConv.getSize() == 0)
		return;

	p.setFont( "Courier", 12 );

	//
	// init. viewport with suitable values
	//

	maxIt   = arrConv.getSize()-1;
	maxConv = 0;

	for (i = 0; i < maxIt; i++)
	{
		if (arrConv[i] > maxConv)
			maxConv = arrConv[i];
	}// for

	p.getView().setWorld( -2, -0.2, maxIt + 4, 1.2 );

	// draw coord system
	p.drawLine( -1, 0, maxIt + 1, 0 );
	p.drawLine(  0, -0.1, 0, 1.1 );

	for (i = 1; i <= maxIt; i++)
	{
		p.drawLine( i, 0, i, -0.02 );

		sprintf( szString, "%d", i );
		p.drawText( TFloat(i)-0.06, -0.05, szString );
	}// for

	for (x = 0.1; x <= 1.0; x += 0.1)
	{
		p.drawLine( 0, x, -0.2, x );
	}// for

	p.drawText( 0.2, 1.05, "log |r|" );
	p.drawText( -1, 0.99, "1.0" );
	p.drawText( maxIt + 1, -0.04, "# it" );

	//
	// draw conv. rates
	//
	for (i = 1; i <= maxIt; i++)
	{
		p.drawPoint( i, arrConv[i] );
	}// for
}
