//
//  Project :  libvisual
//  File    :  TPOVRay.cc
//  Purpose :  class for a VRML interface
//  Author  :  Ronald Kriemann
//
// arch-tag: eb7ddd14-5575-4892-ab67-8a709220faae
//

#include <string.h>

#include "defines.hh"

#include "TPOVRay.hh"

//
// constructor
//
TPOVRay::TPOVRay ( const char *filename )
        : T3DPrinter( filename )
{}

//
// destructor
//
TPOVRay::~TPOVRay ()
{
    arrNodes.setSize( 0 );
}

//
// begin (end) printing
//
void 
TPOVRay::begin ()
{
    T3DPrinter::begin();

    if (state() != UNDEF)
	{
        // set initial values (include, camera, light)
        indentWrite( "#version 3.0" );
        indentWrite( "#include \"colors.inc\"" );

        beginSection( "global_settings" );
        indentWrite( "assumed_gamma 2.2" );
        indentWrite( "max_trace_level 100" );
        endSection();

        beginSection( "camera" );
//         indentWrite( "orthographic" );
//         indentWrite( "location <15, 15, -20>" );
//         indentWrite( "right (1.6*4/3)*x" );
//         indentWrite( "up 1.6*y" );
//         indentWrite( "look_at  <0, 0, 1>" );
        indentWrite( "location <1.1, 0.9, -0.8>" );
        indentWrite( "right 1.3*x" );
        indentWrite( "up 1.3*y" );
        indentWrite( "look_at  <0.1, 0.15, 0.9>" );
        endSection();

        beginSection( "light_source" );
        indentWrite( "<5, 30, -30>" );
        indentWrite( "color White" );
        indentWrite( "shadowless" );
        endSection();

        beginSection( "background" );
        indentWrite( "White" );
        endSection();

//         beginSection( "plane" );
//         indentWrite( "y, -10" );
//         setColor( 1, 1, 1 );
//         writeColor();
//         endSection();
	}// else
}

void 
TPOVRay::end ()
{
    if (state() != UNDEF)
        endSection();

    T3DPrinter::end();
}

//
// begin (end) new section 
//
void 
TPOVRay::beginSection ( const char *name )
{
	char  szString[256];

    if (state() != UNDEF)
	{
		if (name != NULL)
			sprintf( szString, "%s {", name );
		else
        {
			cerr << "TPOVRay::beginNode (char*) : NULL node was given" << endl;
            return;
        }// if

		indentWrite( szString );
		indentUp();
	}// ifr
}

void 
TPOVRay::endSection ()
{
    if ((state() != UNDEF) && (indentLevel() > 0))
	{
		indentDown();
		indentWrite( "}" );
	}// if
}

void
TPOVRay::drawText ( TVector<3> *orig, const char *text )
{
    char  szString[256];

    if (orig == NULL)
    {
        cerr << "TPOVRay::drawText (...) : gave NULL node" << endl;
        return;
    }// if
    
    if (text == NULL)
    {
        cerr << "TPOVRay::drawText (...) : gave NULL text" << endl;
        return;
    }// if
    
    beginSection( "text" );
    
    switch (font().font)
    {
        case FONT_SERIF      : indentWrite( "ttf \"cyrvetic.ttf\"," ); break;
        case FONT_SANS       : indentWrite( "ttf \"crystal.ttf\"," ); break;
        case FONT_TYPEWRITER : indentWrite( "ttf \"timrom.ttf\"," ); break;
        default              : break;
    }// switch

    sprintf( szString, "\"%s\",", text );
    indentWrite( szString );

    sprintf( szString, "%.4g, 0", double( font().width ) );
    indentWrite( szString );

    sprintf( szString, "translate <%.4g, %.4g, %.4g>",
             double( (1/font().size) * transX( orig ) ),
             double( (1/font().size) * transY( orig ) ),
             double( (1/font().size) * transZ( orig ) ) );
    indentWrite( szString );

    sprintf( szString, "scale %.4g",
             double( font().size ) );
    indentWrite( szString );

    writeColor();

    endSection();
}
    
void
TPOVRay::drawLine ( TVector<3> *orig, TVector<3> *dest )
{
    drawCylinder( orig, dest, LINE_THICKNESS );
}
    
void
TPOVRay::drawCylinder ( TVector<3> *orig, TVector<3> *dest,
                        TFloat radius )
{
    char  szString[256];
    
    if ((orig == NULL) || (dest == NULL))
    {
        cerr << "TPOVRay::drawCylinder (...) : gave NULL node" << endl;
        return;
    }// if

    if ( *orig == *dest )
        return;
    
    beginSection( "cylinder" );

    sprintf( szString, "<%.4g, %.4g, %.4g>,",
             double( transX( orig ) ),
             double( transY( orig ) ),
             double( transZ( orig ) ) );
    indentWrite( szString );
    
    sprintf( szString, "<%.4g, %.4g, %.4g>,",
             double( transX( dest ) ),
             double( transY( dest ) ),
             double( transZ( dest ) ) );
    indentWrite( szString );

    sprintf( szString, "%.4g", double( radius ) );
    indentWrite( szString );

    writeColor();

    endSection();
}

void
TPOVRay::drawCone ( TVector<3> *orig, TFloat origRad,
                    TVector<3> *dest, TFloat destRad )
{
    char  szString[256];
    
    if ((orig == NULL) || (dest == NULL))
    {
        cerr << "TPOVRay::drawCone (...) : gave NULL node" << endl;
        return;
    }// if

    if ( *orig == *dest )
        return;
    
    beginSection( "cone" );

    sprintf( szString, "<%.4g, %.4g, %.4g>,",
             double( transX( orig ) ),
             double( transY( orig ) ),
             double( transZ( orig ) ) );
    indentWrite( szString );
    
    sprintf( szString, "%.4g", double( origRad ) );
    indentWrite( szString );

    sprintf( szString, "<%.4g, %.4g, %.4g>,",
             double( transX( dest ) ),
             double( transY( dest ) ),
             double( transZ( dest ) ) );
    indentWrite( szString );

    sprintf( szString, "%.4g", double( destRad ) );
    indentWrite( szString );

    writeColor();

    endSection();
}

void
TPOVRay::drawSphere ( TVector<3> * orig, TFloat radius )
{
    char  szString[256];
    
    if (orig == NULL)
    {
        cerr << "TPOVRay::drawSphere (...) : gave NULL node" << endl;
        return;
    }// if

    beginSection( "sphere" );

    sprintf( szString, "<%.4g, %.4g, %.4g>,",
             double( transX( orig ) ),
             double( transY( orig ) ),
             double( transZ( orig ) ) );
    indentWrite( szString );
    
    sprintf( szString, "%.4g", double( radius ) );
    indentWrite( szString );

    writeColor();
    
    endSection();
}
    
void
TPOVRay::drawPolygon ()
{
    uint                         i, noOfNodes;
    TListIterator< TVector<3> >  iterNodes( polygonData().listNodes );
    TVector<3>                  *v, *pFirstNode;
    char                         szString[256];
    bool                         bClosed;

    
    noOfNodes = polygonData().listNodes.getSize();
    
    if (noOfNodes == 0)
        return;

    beginSection( "polygon" );

    // look if first and last node are equal
    if (iterNodes.first() == iterNodes.last())
        bClosed = TRUE;
    else
        bClosed = FALSE;

    if ( ! bClosed )
        noOfNodes++;
    
    sprintf( szString, "%d,", noOfNodes );
    indentWrite( szString );

    i = 1;
    pFirstNode = iterNodes.first();
    while ((v = iterNodes++) != NULL)
    {
        if (i++ < noOfNodes)
        {
            sprintf( szString, "<%.4g, %.4g, %.4g>,",
                     double( transX( v ) ),
                     double( transY( v ) ),
                     double( transZ( v ) ) );
        }// if
        else
            sprintf( szString, "<%.4g, %.4g, %.4g>",
                     double( transX( v ) ),
                     double( transY( v ) ),
                     double( transZ( v ) ) );

        indentWrite( szString );
    }// while

    if ( !bClosed && (pFirstNode != NULL))
    {
        sprintf( szString, "<%.4g, %.4g, %.4g>",
                 double( transX( pFirstNode ) ),
                 double( transY( pFirstNode ) ),
                 double( transZ( pFirstNode ) ) );
        
        indentWrite( szString );
    }// if
    
    writeColor();
    
    endSection();
}

//
// write color/font def to file
//
void
TPOVRay::writeColor ()
{
    char  szString[256];

    if ((color().amb_r != 0) || (color().amb_g != 0) || (color().amb_b != 0))
        beginSection( "texture" );

    if (color().filter > 0)
    {
        if (color().transmit > 0)
            sprintf( szString, "pigment { color rgbft <%.4g, %.4g, %.4g, %.4g, %.4g> }",
                     double( color().r ), double( color().g ), double( color().b ),
                     double( color().filter ), double( color().transmit ) );
        else
            sprintf( szString, "pigment { color rgbt <%.4g, %.4g, %.4g, %.4g> }",
                     double( color().r ), double( color().g ), double( color().b ),
                     double( color().filter ) );
    }// if      
    else
    {
        if (color().transmit > 0)
            sprintf( szString, "pigment { color rgbt <%.4g, %.4g, %.4g, %.4g> }",
                     double( color().r ), double( color().g ), double( color().b ),
                     double( color().transmit ) );
        else
            sprintf( szString, "pigment { color rgb <%.4g, %.4g, %.4g> }",
                     double( color().r ), double( color().g ), double( color().b ) );
    }// else        
    
    indentWrite( szString );
    
    if ((color().amb_r != 0) || (color().amb_g != 0) || (color().amb_b != 0))
    {
        sprintf( szString, "finish { ambient rgb <%.4g, %.4g, %.4g> }",
                 double( color().amb_r ), double( color().amb_g ), double( color().amb_b ) );
        indentWrite( szString );

        endSection();
    }// if

}

void
TPOVRay::writeFont ()
{
}
