//
//  Project :  libvisual
//  File    :  matrixview.cc
//  Purpose :  class for blockmatrix-visualization
//  Author  :  Ronald Kriemann / Sabine Gutsch
//
// arch-tag: 679a43b8-07de-4d2a-ba5b-ebc0a25f34ec
//

#include "cmdline.hh"

#include "criterion.hh"

#include "matrixview.hh"

bool TMatrixView::_useColors = FALSE;

//
// constructor
//
TMatrixView::TMatrixView ( TBlockMatrix * pMatrix, TFloat x, bool smaller )
{
    pBlockMatrix = pMatrix;
    pMatrixBlock = NULL;
    xi           = x;
    bDrawSmaller = smaller;

    pArrBlockStruct = NULL;
}

TMatrixView::TMatrixView ( TMatrixBlock * pMatrix, TFloat x, bool smaller )
{
    pBlockMatrix = NULL;
    pMatrixBlock = pMatrix;
    xi           = x;
    bDrawSmaller = smaller;

    pArrBlockStruct = NULL;
}

//
// set usage of colors
//
void
TMatrixView::useColors ( bool b )
{
    _useColors = b;
}
    
//
// set blockstructure
//
void
TMatrixView::setBlockStructure ( TDynArray< uint > * pBlockStruct )
{
    pArrBlockStruct = pBlockStruct;
}
    
//
// draw the matrix
//
void 
TMatrixView::draw ( TPrinter & p )
{
    if (pBlockMatrix != NULL)
    {
        if (pBlockMatrix->getNoOfRows() > 1)
            drawMatrix( p, pBlockMatrix );
        else
            drawBlock( p, & (*pBlockMatrix)[0][0] );
    }// if

    if (pMatrixBlock != NULL)
        drawBlock( p, pMatrixBlock );
}

void 
TMatrixView::drawMatrix ( TPrinter & p, TBlockMatrix * pMatrix )
{
    uint           i, j, k, l;
    uint           noOfEntries;
    uint        * indices;
    TFloat       * values;
    TFloat         diagValue;
    TMatrixBlock * pBlock;
    uint           noOfRows, noOfColumns;
    uint           size, blockSize;
    uint           offset_x, offset_y;
    int            lines, dist;

    noOfRows    = pMatrix->getNoOfRows();
    noOfColumns = (*pMatrix)[0].getNoOfBlocks();

    int  blockSizes[noOfRows][noOfRows][2];
    
    for ( i = 0; i < noOfRows - 1; i++ )
    {
        for ( j = 0; j < noOfColumns - 1; j++ )
        {
            blockSizes[i][j][0] = (*pMatrix)[i][i].getNoOfRows();
            blockSizes[i][j][1] = (*pMatrix)[i][i].getNoOfRows();
        }// for
    }// for

    for ( i = 0; i < noOfRows-1; i++ )
    {
        blockSizes[i][noOfColumns-1][0] = (*pMatrix)[i][i].getNoOfRows();
        blockSizes[i][noOfColumns-1][1] = (*pMatrix)[noOfRows-1][0].getNoOfRows();
    }// for
    for ( i = 0; i < noOfColumns-1; i++ )
    {
        blockSizes[noOfRows-1][i][0] = (*pMatrix)[noOfRows-1][0].getNoOfRows();
        blockSizes[noOfRows-1][i][1] = (*pMatrix)[i][i].getNoOfRows();
    }// for
    blockSizes[noOfRows-1][noOfColumns-1][0] = (*pMatrix)[noOfRows-1][0].getNoOfRows();
    blockSizes[noOfRows-1][noOfColumns-1][1] = (*pMatrix)[noOfRows-1][0].getNoOfRows();

    size = 0;
    for (i = 0; i < noOfRows; i++ )
    {
        for ( j = 0; j < noOfColumns; j++ )
        {
            pBlock = & (*pMatrix)[i][j];
            
            if ((pBlock == NULL) || (pBlock->getNoOfRows() == 0))
                continue;
            
            size += pBlock->getNoOfRows();
            
            break;
        }// for
    }// for
    
    // resize viewport
    lines = size * ELEMENT_SIZE;
    dist  = lines / 20;
    p.getView().setWorld( -dist, -dist, lines+2*dist, lines+2*dist );

    offset_y = 0;
    
    for (i = 0; i < noOfRows; i++ )
    {
        offset_x = 0;

        for ( j = 0; j < noOfColumns; j++ )
        {
            pBlock = & (*pMatrix)[i][j];

            if (pBlock == NULL)
                blockSize = 0;
            else
                blockSize = pBlock->getNoOfRows();
            
            // draw block
            for ( l = 0; l < blockSize; l++ )
            {
                noOfEntries = (*pBlock)[l].getNoOfEntries();
                indices     = (*pBlock)[l].getIndex();
                values      = (*pBlock)[l].getValue();

                if ((indices == NULL) || (values == NULL))
                    continue;
                
                diagValue   = values[0];
                
                for ( k = 0; k < noOfEntries; k++ )
                {
                    if (bDrawSmaller == FALSE)
                    {
                        if ( isStrong( l, indices[k] ) )
                            p.drawPoint( (offset_x+indices[k])*ELEMENT_SIZE,
                                         (size-(offset_y + int(l)))*ELEMENT_SIZE );
                    }// if
                    else
                    {
                        if ( ! isStrong( l, indices[k] ) )
                            p.drawPoint( (offset_x + indices[k])*ELEMENT_SIZE,
                                         (size-(offset_y + int(l)))*ELEMENT_SIZE );
                    }// else
                }// for
            }// for

            offset_x += blockSizes[i][j][1];
        }// for

        offset_y += blockSizes[i][0][0];
    }// for
}

void 
TMatrixView::drawBlock ( TPrinter & p, TMatrixBlock * pMatrix )
{
    uint           i, j;
    uint           noOfEntries;
    uint        * indices;
    TFloat       * values;
    TFloat         diagValue, value;
    
    if (pMatrix == NULL)
    {
        cout << "TMatrixView::draw (...) : have NULL matrixblock" << endl;
        return;
    }// if
    
    uint    size = pMatrix->getNoOfRows();
    TFloat  max  = maxEntry( pMatrix );
    
    // resize viewport
    int lines = size * ELEMENT_SIZE;
    int dist  = lines / 20;
    p.getView().setWorld( -dist, -dist, lines+2*dist, lines+2*dist );

    //
    // draw blockstructure of matrixblock
    //
    if ((pArrBlockStruct != NULL) &&
        (pArrBlockStruct->getSize() > 1))
    {
        uint  blockSize = pArrBlockStruct->getSize();
        long   blockPos  = (*pArrBlockStruct)[0];
        long   offset    = ELEMENT_SIZE / 2;
        
        p.setRGBColor( COLOR_LTGRAY );

        for ( i = 1; i < blockSize; i++ )
        {
            p.drawLine( (blockPos * ELEMENT_SIZE) - offset,
                        0,
                        (blockPos * ELEMENT_SIZE) - offset,
                        (size * ELEMENT_SIZE) + offset );
            
            p.drawLine( - offset,
                        ((size - blockPos) * ELEMENT_SIZE) + offset,
                        (size * ELEMENT_SIZE),
                        ((size - blockPos) * ELEMENT_SIZE) + offset );

            blockPos += (*pArrBlockStruct)[i];
        }// for

        // surrounding lines
        p.drawLine( - offset,
                    0,
                    - offset,
                    (size * ELEMENT_SIZE) + offset );
        p.drawLine( (size * ELEMENT_SIZE),
                    0,
                    (size * ELEMENT_SIZE),
                    (size * ELEMENT_SIZE) + offset );
            
        p.drawLine( - offset,
                    0,
                    (size * ELEMENT_SIZE),
                    0 );
        p.drawLine( - offset,
                    (size * ELEMENT_SIZE) + offset,
                    (size * ELEMENT_SIZE),
                    (size * ELEMENT_SIZE) + offset );
    }// if
    
    p.setRGBColor( COLOR_BLACK );

    // draw matrix
    for ( i = 0; i < size; i++ )
    {
        noOfEntries = (*pMatrix)[i].getNoOfEntries();
        indices     = (*pMatrix)[i].getIndex();
        values      = (*pMatrix)[i].getValue();
        diagValue   = values[0];

        for ( j = 0; j < noOfEntries; j++ )
        {
            value = values[j];
            
            if (bDrawSmaller == FALSE)
            {
                // if value is not too small
                if ( isStrong( i, indices[j] ) )
                {
                    // draw a point
                    if (_useColors)
                    {
                        p.setHSVColor( 360 * ( 1 - (fabs(value) / max) ), 1, 1 );
                        p.drawFilledArc( indices[j]*ELEMENT_SIZE,
                                         (size-int(i))*ELEMENT_SIZE,
                                         (TFloat(ELEMENT_SIZE)) * (fabs(value) / max),
                                         0, 360 );
                    }// if
                    else
                        p.drawPoint( indices[j]*ELEMENT_SIZE,
                                     (size-int(i))*ELEMENT_SIZE );
                }// if
            }// if
            else
            {
                // if value is not too big
                if ( ! isStrong( i, indices[j] ) )
                {
                    // draw a point
                    p.drawPoint( indices[j]*ELEMENT_SIZE,
                                 (size-int(i))*ELEMENT_SIZE );
                }// if
            }// else
        }// for
    }// for
}

//
// return the shown matrixblock
//
TMatrixBlock&
TMatrixView::getMatrixBlock () const 
{
    if (pBlockMatrix != NULL)
        return ((*pBlockMatrix)[0][0]);
    else
        return * pMatrixBlock;
}

//
// return maximum entry in matrixblock
//
TFloat
TMatrixView::maxEntry ( TMatrixBlock * pMatrix )
{
    uint           i, j;
    uint           noOfEntries;
    TFloat       * values, value;
    TFloat         max = 0;
    uint           size = pMatrix->getNoOfRows();

    // draw matrix
    for ( i = 0; i < size; i++ )
    {
        noOfEntries = (*pMatrix)[i].getNoOfEntries();
        values      = (*pMatrix)[i].getValue();

        for ( j = 0; j < noOfEntries; j++ )
        {
            if (fabs(value = values[j]) > max)
                max = fabs(value);
        }// for
    }// for

    return max;
}

//
// check if matrixentry is strong w.r.t. graph-criterion
//
bool
TMatrixView::isStrong ( uint i, uint j, uint bi, uint bj )
{
    if (i == j)
        return TRUE;
    
    TFloat          diagValue, conValue0, conValue1;
    TMatrixBlock  * pBlock;
    
    if (pBlockMatrix != NULL)
        pBlock = & ((*pBlockMatrix)[bi][bj]);
    else
        pBlock = pMatrixBlock;

    if (pBlock == NULL)
        return FALSE;
    
    if (param.criterion == E3)
    {
        diagValue = (*pBlock)[i][i];
        conValue0 = (*pBlock)[i][j];
        
        if (fabs(conValue0) > fabs(xi * diagValue))
            return TRUE;
        else
            return FALSE;
    }// if

    if (param.criterion == E7)
    {
        conValue0 = (*pBlock)[i][j];
        conValue1 = (*pBlock)[j][i];
    
        if (fabs(conValue0) > fabs(xi * conValue1))
            return TRUE;
        else
            return FALSE;
    }// if

    return FALSE;
}
